<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;

use App\Models\Providers\Provider;
use App\Models\Providers\ProviderAvailability;
use App\Models\Customers\Appointment;
use App\Models\Providers\ProviderSlot;
use DB;


use App\Jobs\SendMailJob;

class ProviderNextAvailableSlot extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'provider:next_available_slot';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Find Provider next available slot';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */

    public function handle()
    {
        $providers = Provider::where([
                    'status'=>'active',
                    'register_status'=>'completed',
                    'register_step'=>5])->get();
       
        if($providers->count())
        {
            $prev_day  = date('Y-m-d', strtotime(date('Y-m-d')." -1 day"));
            $curr_day  = date('Y-m-d', strtotime(date('Y-m-d')));
            $next_day  = date('Y-m-d', strtotime(date('Y-m-d')." +1 day"));

            $dates = [$prev_day, $curr_day, $next_day];
            
            foreach($providers as $key => $provider) 
            {
                $availableSots = [];
                foreach ($dates as $date) {
                    $day  = date('l', strtotime($date));
                    $slots = ProviderSlot::where(['provider_id' => $provider->id, 'day' => $day])->get();
                    if($slots->count()){
                        foreach ($slots as $key => $value)
                        {     
                            $datetime = $date." ".$value->slot;
                            /* convert data time utc to local  */
                            $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, $provider->timezone));
                            /* convert data time utc to local  */

                            $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$provider->id,'date'=>$date,'time'=> $value->slot])->first();
                            if(empty($appointment) || !empty($appointment->cancell_appointment) ){
                                $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                    'time' => date("H:i", $slotDateTime),
                                    'available' => 'yes',
                                ];
                            }
                        }
                    }else{
                        DB::table('providers')
                            ->where('id', $provider->id)
                            ->update(['next_available_slot' => null]);
                   }  
                }

                $allSlots = !empty($availableSots[$curr_day]) ? $availableSots[$curr_day] : [];
                //print_r($availableSots);

                /* provider applied day off  */
                $leaves = [];
                foreach ($allSlots as $key => $slot) {
                    $providerTimeOff = DB::table('provider_time_off')->where(['provider_id' => $provider->id])->get();
                    foreach ($providerTimeOff as $key => $value) {
                        $startDate = dateFormat($value->start_date);
                        $endDate = dateFormat($value->end_date);

                        $start_time = date("H:i", strtotime(convertTimeUtcToLocal($value->start_time, $provider->timezone)));
                        $end_time   = date("H:i", strtotime(convertTimeUtcToLocal($value->end_time, $provider->timezone)));

                        if(($curr_day >= $startDate)  && ($curr_day <= $endDate))
                        {
                            if((strtotime($curr_day." ".$slot['time']) >= strtotime($curr_day." ".$start_time) && strtotime($curr_day." ".$slot['time']) < strtotime($curr_day." ".$end_time)))
                            {
                                $leaves[] =  [
                                    'time' => $slot['time'],
                                    'available' => $slot['available']
                                ];
                            }
                        }
                    }
                }
                /* provider applied day off  */

                $finalSlots = checkLeave($allSlots, $leaves);

                if(count($finalSlots))
                {
                    foreach($finalSlots as $v){
                        $nextSlot = nextAvailableslotDateTimeLocalToUtc(date("Y-m-d")." ".$v['time'], $provider->timezone);
                        if($nextSlot >= date('Y-m-d H:i')){
                            $v = nextAvailableslotTimeLocalToUtc($v['time'], $provider->timezone);
                            $nexAvailableSlot = $v;
                            DB::table('providers')
                                   ->where('id', $provider->id)
                                   ->update(['next_available_slot' => $nexAvailableSlot]);
                                   break;
                        }else{
                            DB::table('providers')
                                   ->where('id', $provider->id)
                                   ->update(['next_available_slot' => null]);
                        }
                    }
                }else{
                        DB::table('providers')
                                   ->where('id', $provider->id)
                                   ->update(['next_available_slot' => null]);
                } 
            }
        
            echo "Next available slot cummand run successfully!";
        }
    }


    /*public function handle()
    {
        $providers = Provider::where([
                    'status'=>'active',
                    'register_status'=>'completed',
                    'register_step'=>5])->get();
       
        if($providers->count())
        {
            $day  = date('l', strtotime(date('Y-m-d')));
            $appDate = date('Y-m-d');
            foreach($providers as $key => $provider) 
            {
                   $availabilityLists = ProviderAvailability::where(['provider_id'=>$provider->id,'day'=>$day])->get(); 
                   if(count($availabilityLists)){
                        $slot = []; 
                        foreach ($availabilityLists as $key => $value) 
                        {
                            $startTime = strtotime($value->start_time);
                            $endTime    = strtotime($value->end_time);
 
                            for($i = $startTime; $i < $endTime; $i+=2700) {  // 3600 second => 1 hour
                                $time = gmdate("H:i", $i);
                                $appointment = Appointment::where(['provider_id'=>$provider->id,'date'=>date('Y-m-d'),'time'=> $time])->first();
                                if(empty($appointment)){
                                    $slot[] = $time;
                                }
                            }
                        }

                        $leaveSlot = [];
                        foreach ($slot as $key => $slotTime) {
                            $providerTimeOff = DB::table('provider_time_off')->where(['provider_id' => $provider->id])->get();
                            foreach ($providerTimeOff as $key => $value) {
                                $startDate = $value->start_date;
                                $endDate = $value->end_date;
                        
                                if(($appDate >= $startDate)  && ($appDate <= $endDate))
                                {
                                    if((strtotime($appDate." ".$slotTime) >= strtotime($appDate." ".$value->start_time) && strtotime($appDate." ".$slotTime) < strtotime($appDate." ".$value->end_time)))
                                    {
                                        $leaveSlot[] =  $slotTime;
                                    }
                                }
                            }
                        }

                        $result = array_values(array_diff($slot,$leaveSlot));
                        //dd($result[0]);
                        if(count($result)){
                            foreach($result as $v){
                                if($v >= date('H:i')){
                                    $nexAvailableSlot = $v;
                                    $providerData = Provider::find($provider->id);
                                    $providerData->next_available_slot = $nexAvailableSlot;
                                    $providerData->save();
                                }else{
                                    $providerData = Provider::find($provider->id);
                                    $providerData->next_available_slot = null;
                                    $providerData->save();
                                }
                            }
                        }else{
                            $providerData = Provider::find($provider->id);
                            $providerData->next_available_slot = null;
                            $providerData->save();
                        }
                   }else{
                        $providerData = Provider::find($provider->id);
                        $providerData->next_available_slot = null;
                        $providerData->save();
                   }   
            }
            echo "Next available slot cummand run successfully!";
        }
    }*/
}
