<?php

namespace App\Http\Controllers\Business;
use Illuminate\Support\Facades\Schema;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\App;
use App\Models\Business\Business;
use App\Models\Business\BusinessDomain;
use App\Models\Business\BusinessAccessCode;
use App\Models\Business\BusinessAccessCodeUserMap;
use Illuminate\Support\Arr;
use App\Models\Settings\Setting;
use App\Models\Customers\Oauth;
use Illuminate\Support\Facades\DB;
use App\Jobs\CustomerAccessCodeJob;
use App\Models\EmailTemplate\EmailTemplate;
use App\Models\Customers\Customer;
use Validator;
use Exception;
use Throwable;
use Carbon\Carbon;
use App\Exports\AccessCodeExport;
use Maatwebsite\Excel\Facades\Excel;
use App\Jobs\GenerateAccessCodeJob;


class BusinessController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */


    protected $business;
    protected $oauth_user;
    protected $user_id;
    protected $role_fk;

    public function __construct(Business $business, Oauth $oauth_user,  Request $request)
    {
        $this->business = $business;
        $this->oauth_user = $oauth_user;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        
        date_default_timezone_set('UTC');
    }

    public function index(){
        $business=$this->business::with('business_access_code')->first();
       
    }

    public function store(Request $request){
       
         $message = array(
            'business_name.required'   => config('service.BUSINESS_API.VALIDATION.BUSINESS_NAME_REQUIRED'),
            'business_id.required'    => config('service.BUSINESS_API.VALIDATION.BUSINESS_ID'),
            'access_code.required'       => config('service.BUSINESS_API.VALIDATION.ACCESS_CODE'),
            'membership.required'        =>config('service.BUSINESS_API.VALIDATION.MEMBERSHIP'),
            'business_domain.required'        =>config('service.BUSINESS_API.VALIDATION.BUSINESS_DOMAIN'), 
        );
        $validator = Validator::make($request->all(), [
            'business_name' => 'required|max:100',
            'business_id' => 'required|unique:businesses',
            'access_code' => 'required',
            'membership' =>'required',
            'business_domain'=>'required',
            
            
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }


        try {

            DB::beginTransaction();
            // $input = $request->all();
        
            $input['business_name']=$request->business_name;
            $input['business_id']=strtoupper($request->business_id);
            $input['membership_plan']=$request->membership;
            $input['access_code_count']=$request->access_code;
            $input['status']=true;
          
            
            $business =  Business::create($input);
            $businessId=$business->id;
            DB::commit();
            if($business){
                $business_domain=explode(',',$request->business_domain);
                $business_domain=array_unique($business_domain);
                foreach(array_filter($business_domain) as $domain){
                    
                    $data['business_id']=$businessId;
                    $data['name']=$domain;
                    $data['status']=true;
                           
                    $business=BusinessDomain::create($data);
                }
               
                if($request->access_code){
                    $generateAccessCode = new GenerateAccessCodeJob($request->access_code, $request->business_id,$businessId);
                    // $this->dispatchNow($generateAccessCode);
                  
                    // for($i=0;$i<$request->access_code;$i++){
                    //    $access_code= $request->business_id.'-'.rand(1000,9999).'-'.rand(1000,9999).'-'.rand(1000,9999);                       
                    //     $code['business_id']=$businessId;
                    //     $code['access_code']=$access_code;
                    //     $code['status']=true;
                    //     $access_code=BusinessAccessCode::create($code);
                    // }
                    
                }
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STORE'), 'records' => $business]);
            }
           

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
        // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function update(Request $request){
        $message = array(
            'business_name.required'   => config('service.BUSINESS_API.VALIDATION.BUSINESS_NAME_REQUIRED'),
            'business_id.required'    => config('service.BUSINESS_API.VALIDATION.BUSINESS_ID'),
            'access_code.required'       => config('service.BUSINESS_API.VALIDATION.ACCESS_CODE'),
            'membership.required'        =>config('service.BUSINESS_API.VALIDATION.MEMBERSHIP'),
            'business_domain.required'        =>config('service.BUSINESS_API.VALIDATION.BUSINESS_DOMAIN'), 
        );
        $validator = Validator::make($request->all(), [
            'business_name' => 'required|max:100',
            'business_id' =>'required',
            'access_code' => 'required',
            'membership' =>'required',
            'business_domain'=>'required',
            
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }


        try {

            DB::beginTransaction();
            // $input = $request->all();

            $business = Business::find($request->update_id);
       
            if ($business->access_code_count > $request->access_code) {
                return response()->json(['success' => false,'message' => 'Please go to business details to
                decrease/reduce access codes'],404);
       
            }
            $totalAccessCode=$request->access_code-$business->access_code_count;
 
            $input['business_name']=$request->business_name;
            $input['business_id']=strtoupper($request->business_id);
            $input['membership_plan']=$request->membership;
            $input['access_code_count']=$request->access_code;
            $input['status']=true;

            if ($business) {
                // Update the attributes as needed
                $business->fill($input); // where $data is an array of new values
                
                // Save the updated record
                $business->save();
            }
        
            $businessId=$business->id;
            DB::commit();
            if($business){
                BusinessDomain::where('business_id',$business->id)->delete();
                $business_domain=explode(',',$request->business_domain);
                $business_domain=array_unique($business_domain);
                foreach(array_filter($business_domain) as $domain){
                    
                    $data['business_id']=$businessId;
                    $data['name']=$domain;
                    $data['status']=true;
                           
                    $business=BusinessDomain::create($data);
                }
               
                if($request->access_code && $totalAccessCode>0){
                    
                    $generateAccessCode = new GenerateAccessCodeJob($totalAccessCode, $request->business_id,$businessId);
                    // $this->dispatchNow($generateAccessCode);
                    
                }
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.BUSINESS_API.MESSAGE.SUCCESS_UPDATE'), 'records' => $business]);
            }
           

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
        // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function checkDomain(Request $request){
        
        $message = array(
            'business_domain.required'        =>config('service.BUSINESS_API.VALIDATION.BUSINESS_DOMAIN'), 
        );
        
        $validator = Validator::make($request->all(), [
            'business_domain'=>'required',
            
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        
        
        try {

              $domains = explode(',',$request->business_domain);
              
              $existDomain = array();

              foreach($domains as $domain){
                
                $dataDomain = DB::table('business_domains')->where('name',trim($domain))->count(); 
                
                if($dataDomain > 0){
                    array_push($existDomain,trim($domain));
                }
              }

              if(count($existDomain) > 0){
                $existDomains = implode(',',$existDomain);
                return response()->json(['success' => false,'message' => $existDomains.' '.'already exist.'],200);
              }else{
                return response()->json(['success' => true,'message' => 'success'],200);
              }

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
        // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function getBusinessList(Request $request){


        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            
            /*if ($limit > env('MAX_PER_PAGE')) {
            $limit = 2;
            }*/
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }

            $tableName = 'businesses';

            // Get the column names of the 'users' table
            $columnNames = Schema::getColumnListing($tableName);
          

          
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $columnNames)) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

        
    // Build the query
            $query = Business::select('businesses.*', 'plans.type as type', 'plan_translations.title')
                ->join('plans', function ($join) {
                    $join->on(\DB::raw("FIND_IN_SET(plans.id, businesses.membership_plan)"), '>', \DB::raw("'0'"));
                })
                ->join('plan_translations', 'plans.id', '=', 'plan_translations.plan_id')
                ->where(function ($q) use ($request) {
                    if($request->filled('business_name')) {
                        $business_name = $request->business_name;
                        
                        $q->where('business_name', 'like', '%'.$business_name.'%');
                    
                    }
                    if($request->filled('business_id')) {
                        $business_id = $request->business_id;
                        
                        $q->where('business_id', 'like', '%'.$business_id.'%');
                    
                    }
                    if ($request->filled('plan_translations_title')) {
                        $plan_translations_title = $request->plan_translations_title;
                        $q->where('plan_translations.title', 'like', '%' . $plan_translations_title . '%');
                    }
                    
                    
                    if($request->filled('business_status')) {
                        $business_status = $request->business_status;
                        
                        $q->where('businesses.status', 'like', '%'.$business_status.'%');
                    
                    }
                })
                ->where('plan_translations.locale', 'en')
                ->orderBy($order_by_column_name, $order);

    // Paginate the results
                $businesses = $query->paginate($limit);

    // Process and format the results
                $result = [
                    'data' => [],
                    'total' => $businesses->total(),
                    'next_page_url' => $businesses->nextPageUrl(),
                    'path' => $businesses->path(),
                    'per_page' => $businesses->perPage(),
                    'prev_page_url' => $businesses->previousPageUrl(),
                    'to' => $businesses->perPage(),
                ];

            
            $arrId=[];
            $membershipData=[];
            foreach ($businesses as $row) {
                $id=$row->id;
                if(!in_array($id,$arrId)){
                    array_push($arrId,$id);
                    $membershipData[$id]=$row->title.' ('.$row->type.')';
                }else{
                    $membershipData[$id].=','.$row->title.' ('.$row->type.')';
                }            
             }
             $arrData=[];
             foreach ($businesses as $row) {
                $id=$row->id;
                if(!in_array($id,$arrData)){
                    
                    array_push($arrData,$id);
                    $result['data'][] = [
                        'id' => $row->id,
                        'business_name' => $row->business_name,
                        'business_id' => $row->business_id,
                        'access_code_count' => $row->access_code_count,
                        'status' => $row->status,
                        'created_at' => $row->created_at,
                        'updated_at' => $row->updated_at,
                        'memebership_plan'=>$membershipData[$id],
                        
                        
                    ];
                }
            }

            if(empty($result)){
                return response()->json(['success' => true,'message' => 'success', 'record' => $businesses],200);
             
            }
            
            return response()->json(['success' => true,'message' => 'success', 'record' => $result],200);

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }

    }

    public function geAccessCodeList(Request $request,$id){
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
           
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $tableName = 'business_access_codes';

            // Get the column names of the 'users' table
            $columnNames = Schema::getColumnListing($tableName);
            // print_r($columnNames);exit;
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $columnNames)) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $mapAccessId=BusinessAccessCodeUserMap::where('business_id',$id)->pluck('access_id');
            $acccess_code = BusinessAccessCode::select('business_access_codes.*','business_access_code_user_maps.email_id as email','customers.first_name','customers.last_name','business_access_code_user_maps.status as map_status','business_access_code_user_maps.join_datetime as join_date')
                            ->leftjoin('business_access_code_user_maps','business_access_codes.id','business_access_code_user_maps.access_id')
                            ->leftjoin('customers','business_access_code_user_maps.user_id','customers.id')
            ->where(function ($q) use ($request,$mapAccessId) {
                
                //if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    // if (!empty($this->getColumnName())) {
                    //     foreach ($this->getColumnName() as $key => $col) {
                    //         if ($key == 0) {
                    //             $q->where($col, 'like', '%' . encryptString($request->search) . '%');
                            
                    //         } else {
                    //             $pieces = explode(" ", $request->search);
                              
                    //         }
                    //     }
                    // }
                //}
                // filters
                if($request->filled('access_code_status')) {
                    $access_code_status = $request->access_code_status;
                    if($access_code_status!=0){
                        $q->where('business_access_code_user_maps.status', 'like', '%'.$access_code_status.'%');
                    }else{
                        $q->whereNotIn('business_access_codes.id', $mapAccessId);
                        
                    }
                }
                if($request->filled('joining_date')) {
                    $joining_date = date('Y-m-d',strtotime($request->joining_date));
                    
                    $q->where('business_access_code_user_maps.join_datetime','like','%'.$joining_date.'%');
                   
                }
                
            })
            
            ->orderBy($order_by_column_name, $order)
            ->where('business_access_codes.business_id',$id)
            ->paginate($limit);

            if (empty($acccess_code)) {
                throw new Exception(config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'));
            }
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.CUSTOMER_API.MESSAGE.CUSTOMER_LISTING'), 'records' => $acccess_code]);
            }
            catch (Exception $e) {
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
            catch (Throwable $e) {
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }

    public function getBusinessDetails($id){
        try
        {
            $business_details=Business::with('businessDomain')->where('id',$id)->first();
            $business_map=BusinessAccessCode::where('business_id',$id)->count();
            $assign=BusinessAccessCodeUserMap::where('business_id',$id)->where('status',1)->count();
            $inProgress=BusinessAccessCodeUserMap::where('business_id',$id)->where('status',2)->count();
            $unassign=$business_map-($assign+$inProgress);
            $data=[
                   'business_details'=>$business_details,
                   'assign'=>$assign,
                   'unassign'=>$unassign,
                   'inProgress'=>$inProgress,
            ];

        if (empty($business_details)) {
            throw new Exception(config('service.EVERGREEN_API.MESSAGE.CUSTOMER_NOT_FOUND'));
        }
        return response()->json(['status' => config('service.EVERGREEN_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.CUSTOMER_API.MESSAGE.CUSTOMER_LISTING'),'business_details'=>$data]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.EVERGREEN_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.EVERGREEN_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
 
    public function requestAccessCode(Request $request){
        
        $validator = Validator::make($request->all(), [
            'email_id' => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {

            $businessEmailID = $request->email_id;

            $CheckActivePlan = DB::table('business_access_code_user_maps')
                                                ->where('user_id',$this->user_id)
                                                ->where('status','1')
                                                ->first();
            if(!empty($CheckActivePlan)){
                return response()->json(['success' => false,'message' => 'You already have an active plan.'],404);
            }

            $emailIdMaped = DB::table('business_access_code_user_maps')
                                                ->where('email_id',trim($businessEmailID))
                                                ->where('user_id','!=',$this->user_id)
                                                ->first();
            if(!empty($emailIdMaped)){
                return response()->json(['success' => false,'message' => 'This business domain email address is already in use.'],404);
            }

            $CheckAccessCodeWithUserID = DB::table('business_access_code_user_maps')
                ->where('user_id',$this->user_id)
                ->where('status','2')
                ->first();

               
            if(!empty($CheckAccessCodeWithUserID) && $CheckAccessCodeWithUserID->email_id != $request->email_id){
                $accssCode = DB::table('business_access_code_user_maps')
                                    ->where('id',$CheckAccessCodeWithUserID->id)
                                    ->delete();   
                
            }

            $customer = Customer::findOrFail($this->user_id);
            $customerName = decryptString($customer->first_name)." ".decryptString($customer->last_name);
            $customerEmail = decryptString($customer->email);

            $businessDomainName = "";
            $businessDomain = explode('@',$request->email_id);
            if(!empty($businessDomain[1])){
                $businessDomainName = '@'.$businessDomain[1];
            }else{
                return response()->json(['success' => false,'message' => 'The business is not valid.'],404);
            }
           
            if(!empty($businessDomainName)){
                
                ///check if access code already assign to user
                $CheckAccessCodeWithUserEmailID = DB::table('business_access_code_user_maps')
                                                    ->where('email_id',trim($businessEmailID))
                                                    ->where('status','2')
                                                    ->first();
                                                    
                
                
                $dataDomain = DB::table('business_domains')->where('name',trim($businessDomainName))->first();  

                if(!empty($dataDomain)){
                    $BusinessDataCheck = DB::table('businesses')
                                            ->where('id',$dataDomain->business_id)
                                            ->first();

                    if($BusinessDataCheck){ 
                        $planDataCheck = DB::table('plans')
                                                // ->whereRaw('FIND_IN_SET("id", ?)', [$BusinessDataCheck->membership_plan])
                                                ->whereIn('id', explode(',',$BusinessDataCheck->membership_plan))
                                                ->first();
                                                
                        if(empty($planDataCheck)){
                            return response()->json(['success' => false,'message' => 'This business does not have any membership plan.'],404);
                        }

                    }else{
                        return response()->json(['success' => false,'message' => 'The business is not valid.'],404);
                    }

                }
                                                    
                if($CheckAccessCodeWithUserEmailID){
                    $accessCodeId = $CheckAccessCodeWithUserEmailID->access_id;
                    $accessCodeNumberGet = DB::table('business_access_codes')
                                            ->where('id',trim($accessCodeId))
                                            ->where('status','1')
                                            ->first();
                    $accessCodeNumberOld   =   $accessCodeNumberGet->access_code;


                    /*  customer send access code on mail ID */
                   
                    $templateName = "Access Code";
                    $templateData = EmailTemplate::where('name',$templateName)->first();
                    $greeting = 'Hi ' . $customerName.' ,';
                    // $body     = 'We are excited to welcome you to our exclusive membership program! As a valued member, we are delighted to provide you with your unique access code.';
                    $body     = $templateData->text;
                    $accessCodeNumbertext     = ' Membership Access Code: '.$accessCodeNumberOld;
                    $type     = $templateData->title;
                    $gratitude     = config('constant.gratitude_english');
                    $regard     = config('constant.regard_english');
                    $footer     = config('constant.footer_english');
                    // $sendMail = new CustomerAccessCodeJob($businessEmailID, $greeting, $body,$accessCodeNumbertext, $type, $gratitude, $regard, $footer);
                    // $this->dispatchNow($sendMail);
                    /* End customer send access code on mail ID*/
                    $moengageData =  [
                        "type" => "event",
                        "customer_id" => $customerEmail,
                        "actions" => [
                            [
                                "action" => "request_access_code",
                                "attributes" => [
                                    "business_email_address" => $businessEmailID,
                                    "user_access_code" => $accessCodeNumberOld
                                ],
                                "platform" => "web"
                            ]
                        ]
                    ];
            
                   moengageCallCustomEventApiRequest($moengageData);
                 
                    $business_id = $dataDomain->business_id; 
                    return response()->json(['success' => true,'message' => 'Thank you for your request. Please check your company email address. If you have any questions, please contact us at support@pmnowapp.com','business_id' => $business_id ],200);
                    
                }
                ///check if access code already assign to user
               
                if($dataDomain){
                   
                    $business_id = $dataDomain->business_id;                    
                    $accessData = DB::table('business_access_codes')->where('business_id',$business_id)
                                    ->where(function ($query) use ($businessEmailID) {
                                        if($businessEmailID != '') {
                                            $access_id_get = DB::table('business_access_code_user_maps')->select('access_id')->where('email_id','!=','')->get();
                                            $accessCodeIdsArr = array();
                                            foreach($access_id_get as $accessId){
                                                array_push($accessCodeIdsArr,$accessId->access_id);
                                            }
                                            $query->whereNotIn('id',$accessCodeIdsArr);
                                        }
                                    })
                                    ->orderBy('id', 'ASC')
                                    ->first();  
                    if(empty($accessData)){
                        return response()->json(['success' => false,'message' => 'This business does not have an access code.'],404);
                    }else{
                        
                        $assignAccessCodeID  =  $accessData->id;
                        $assignAccessCode    =  $accessData->access_code;
                       
                                    
                        DB::beginTransaction();
                        $input = $request->all();
                    
                        $input['business_id']=$business_id;
                        $input['access_id']=$assignAccessCodeID;
                        $input['user_id']=$this->user_id;
                        $input['email_id']=$businessEmailID;
                        $input['status']=2;
                    
                    
                        $AccessCodeMap =  DB::table('business_access_code_user_maps')->insert($input);
                        DB::commit();
                        if($AccessCodeMap){
                            /*  customer send access code on mail ID */
                            $templateName = "Access Code";
                            $templateData = EmailTemplate::where('name',$templateName)->first();
                            $greeting = 'Hi ' . $customerName.' ,';
                            // $body     = 'We are excited to welcome you to our exclusive membership program! As a valued member, we are delighted to provide you with your unique access code.';
                            $body     = $templateData->text;
                            $accessCodeNumber     = ' Membership Access Code: '.$assignAccessCode;
                            $type     = $templateData->title;
                            $gratitude     = config('constant.gratitude_english');
                            $regard     = config('constant.regard_english');
                            $footer     = config('constant.footer_english');
                            // $sendMail = new CustomerAccessCodeJob($businessEmailID, $greeting, $body,$accessCodeNumber, $type, $gratitude, $regard, $footer);
                            // $this->dispatchNow($sendMail);

                            $moengageData =  [
                                "type" => "event",
                                "customer_id" => $customerEmail,
                                "actions" => [
                                    [
                                        "action" => "request_access_code",
                                        "attributes" => [
                                            "business_email_address" => $businessEmailID,
                                            "user_access_code" => $assignAccessCode
                                        ],
                                        "platform" => "web"
                                    ]
                                ]
                            ];
                    
                            moengageCallCustomEventApiRequest($moengageData);

                            /* End customer send access code on mail ID*/
                            $business_id = $business_id;
                            return response()->json(['success' => true,'message' => 'Thank you for your request. Please check your company email address. If you have any questions, please contact us at support@pmnowapp.com','business_id'=>$business_id],200);
                            
                        }

                    }

                }else{
                    return response()->json(['success' => false,'message' => 'The domain does not exist.'],404);
                }    

            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'records' => []]);
        }
    }
   
    
    public function accessCodeDelete(Request $request){
        try {
        
            DB::beginTransaction();

            $accss_code = BusinessAccessCode::find($request->id);
            
            if($accss_code){
                BusinessAccessCodeUserMap::where('access_id', $request->id)->delete();
                $business=Business::where('id', $accss_code->business_id)->first();
                $remainAceessCode=$business->access_code_count-1;
                Business::where('id', $accss_code->business_id)->update([
                                        'access_code_count' => $remainAceessCode
                                    ]);
            }
            $accss_code->delete();
            
            DB::commit();

            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Access Code Deleted Successfully']);
        }
        catch (Exception $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }

    }
    public function multipleAccessCodeDelete(Request $request){
        try {
        
            DB::beginTransaction();

            $idArray = explode(',', $request->id);
            $accss_code=BusinessAccessCode::where('id',$idArray[0])->first();
            if($accss_code){                
                $business=Business::where('id', $accss_code->business_id)->first();
                $remainAceessCode=$business->access_code_count-count($idArray);
               
                Business::where('id', $accss_code->business_id)->update([
                                        'access_code_count' => $remainAceessCode
                                    ]);
            }
            BusinessAccessCodeUserMap::whereIn('access_id', $idArray)->delete();
            BusinessAccessCode::whereIn('id', $idArray)->delete();
            
            
            
            DB::commit();

            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Access Code Deleted Successfully']);
        }
        catch (Exception $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }

    public function accessCodeRemove(Request $request,$id){
        try {
        
            DB::beginTransaction();

            $accssCode = BusinessAccessCodeUserMap::where('access_id',$request->id)->delete();
            
           
            DB::commit();

            if($accssCode){
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Access code has been removed successfully.']);
            }else{
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => 'Assigned and Unassigned status not removed ']);
            }

            
        }
        catch (Exception $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }        
    }

    public function export() 
    {
        return Excel::download(new AccessCodeExport, 'AccessCodeExport.xlsx');
    }

    public function multipuleAccessCodeRemove(Request $request){
        try {
        
            $ids = explode(',',$request->id);
            
            $accssCodeCheck = BusinessAccessCodeUserMap::whereIn('access_id',$ids)->where('status','1')->count();        
            if($accssCodeCheck == 0){
                $accssCode = BusinessAccessCodeUserMap::whereIn('access_id',$ids)->delete();
                if($accssCode){
                    return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Access code has been removed successfully.']);
                }else{
                    return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => 'Assigned or Unassigned access code cannot be removed.']);
                }
            }else{
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => 'Assigned or Unassigned access code cannot be removed.']);
            }
           
            
        }
        catch (Exception $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }        
    }

    public function exportAccessCode($id){
        
        $business=Business::where('id', $id)->first();

        $acccess_code = BusinessAccessCode::select('business_access_codes.*','business_access_code_user_maps.email_id as email','customers.first_name','customers.last_name','business_access_code_user_maps.status as map_status','business_access_code_user_maps.join_datetime as join_date')
                ->leftjoin('business_access_code_user_maps','business_access_codes.id','business_access_code_user_maps.access_id')
                ->leftjoin('customers','business_access_code_user_maps.user_id','customers.id')
                ->where('business_access_codes.business_id',$id)->get();

        $data[] = array('','',$business->business_name.' | '.$business->business_id,'','');
        $data[] = array('Access code', 'Customer Name', 'Customer Email', 'Joined Date', 'Status');
        foreach($acccess_code as $code){

            $status = 'Unassigned';
            switch ($code->map_status) {
            case "2":
               $status = 'In-Progress';
            case "1":               
                $status = 'Assigned';
            default:                              
                $status = 'Unassigned';
            }

            
            $data[] = array(
                $code->access_code,
                $code->first_name.' '.$code->last_name,
                $code->email,
                $code->join_date,
                $status
            );
        }
              
        // File name for the CSV file
        $filename = $business->business_name.'-'.$business->business_id.'.csv';
        
        // Create a file pointer
        $file = fopen($filename, 'w');
        
        // Loop through the data and write it to the CSV file
        foreach ($data as $row) {
            fputcsv($file, $row);
        }
        
        // Close the file pointer
        fclose($file);
        
        // Set headers to force download
        header('Content-Type: application/csv');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        
        // Output the file to the browser
        readfile($filename);
        
        // Delete the file after it's sent to the browser (optional)
        unlink($filename);

    }

    
    public function exportAccessCodeData($id){
        
        $business=Business::where('id', $id)->first();

        $acccess_code = BusinessAccessCode::select('business_access_codes.*','business_access_code_user_maps.email_id as email','customers.first_name','customers.last_name','business_access_code_user_maps.status as map_status','business_access_code_user_maps.join_datetime as join_date')
                ->leftjoin('business_access_code_user_maps','business_access_codes.id','business_access_code_user_maps.access_id')
                ->leftjoin('customers','business_access_code_user_maps.user_id','customers.id')
                ->where('business_access_codes.business_id',$id)->get();

        $data[] = array('','',$business->business_name.' | '.$business->business_id,'','');
        $data[] = array('Access code', 'Customer Name', 'Customer Email', 'Joined Date', 'Status');
        foreach($acccess_code as $code){

            $status = 'Unassigned';
            if($code->map_status=='1')
            {
                $status ='Assigned';
            }
            elseif($code->map_status=='2')
            {
                $status ='In-Progress';
            }
            else{
                $status ='Unassigned';
            }
            
            $data[] = array(
                $code->access_code,
                decryptString($code->first_name).' '.decryptString($code->last_name),
                $code->email,
                $code->join_date,
                $status
            );
        }
        
              
        // File name for the CSV file
        $filename = $business->business_name.'-'.$business->business_id.'.csv';
        
        $result['access_data'] = $data;        
        $result['filename'] = $filename;
        return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Access code has been removed successfully.','data' => $result]);
        

    }


    public function exportPdf(Request $request, $id)
    {
        $result = []; 
        $startDate = $request->from_date;
        $endDate = $request->to_date;
        // DB::statement('SET SESSION sql_mode=""');
        $totalActivePlanAmount = DB::table('payments')
                                        ->selectRaw('COUNT(plans.id) AS quantity, plans.price, plan_translations.title, plan_translations.description, payments.*')
                                        ->leftJoin('plans', 'plans.id', '=', 'payments.plan_id')
                                        ->leftJoin('business_access_code_user_maps', 'business_access_code_user_maps.user_id', '=', 'payments.customer_id')
                                        ->leftJoin('plan_translations', 'plan_translations.plan_id', '=', 'plans.id')
                                        ->whereBetween('payments.purchased_date', [$startDate, $endDate])
                                        ->where('plans.type', 'B2B')
                                        ->where('plan_translations.locale', 'en')
                                        ->where('business_access_code_user_maps.status', 1)
                                        ->where('business_access_code_user_maps.business_id', $id)
                                        ->groupBy('plans.id')
                                        ->get();

      

        $totalRenewPlanAmount = DB::table('recurring_payments')
                                    ->selectRaw('COUNT(plans.id) AS quantity, plans.price, plan_translations.title, plan_translations.description, recurring_payments.*')
                                    ->leftJoin('customers', 'customers.id', '=', DB::raw("JSON_UNQUOTE(JSON_EXTRACT(recurring_payments.payment_response, '$.customer_id'))"))
                                    ->leftJoin('business_access_code_user_maps', 'business_access_code_user_maps.user_id', '=', 'customers.id')
                                    ->leftJoin('plans', 'plans.id', '=', DB::raw("JSON_UNQUOTE(JSON_EXTRACT(recurring_payments.payment_response, '$.plan_id'))"))
                                    ->leftJoin('plan_translations', 'plan_translations.plan_id', '=', 'plans.id')
                                    ->where('recurring_payments.plan_type', 'B2B')
                                    ->whereBetween('recurring_payments.created_at', [$startDate, $endDate])
                                    ->where('business_access_code_user_maps.business_id', $id)
                                    ->where('business_access_code_user_maps.status', 1)
                                    ->where('plan_translations.locale', 'en')
                                    ->groupBy('plans.id')
                                    ->get();
       
        if(count($totalActivePlanAmount) > 0 || count($totalRenewPlanAmount) > 0 ){    
                                    

            $PDFData = DB::table('pdf')
                    ->where('from_date', '=', $startDate)
                    ->where('to_date', '=', $endDate)
                    ->where('business_id', $id)
                    ->first();
                        
            if(!empty($PDFData->pdf_url)){
                $result['pdf_url'] = $PDFData->pdf_url;
            } 


            $BusinessData = DB::table('businesses')->where('id', $id)->first();              
            $invoiceId  = $BusinessData->business_id.rand(10,99).time();        
            $business_map=BusinessAccessCode::where('business_id',$id)->count();
            $assign=BusinessAccessCodeUserMap::where('business_id',$id)->where('status',1)->count();
            $inProgress=BusinessAccessCodeUserMap::where('business_id',$id)->where('status',2)->count();
            $unassign=$business_map-($assign+$inProgress);


            $result['invoice_id']           =    $invoiceId;              
            $result['invoice_date']         =    date('m/d/Y');             
            $result['invoice_due_date']     =    date('m/d/Y');      
            $result['file_name']            =    $BusinessData->business_name.'-'.$BusinessData->business_id.'-'.date('Y-m-d-H-i-s');  
            $result['new_plan']           =    $totalActivePlanAmount; 
            $result['renew_plan']         =    $totalRenewPlanAmount; 
            $result['business_name']      =    $BusinessData->business_name; 
            $result['business_code']      =    $BusinessData->business_id;  
            $result['assign']             =    $assign; 
            $result['inProgress']         =    $inProgress; 
            $result['unassign']           =    $unassign; 
            $result['total_access_code']  =    $business_map;
        }else{
            $result = []; 
        }
      
          
        

        return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Export data fetch successfully.','data' => $result]);
    }

    public function exportPdfInsert(Request $request){

        $data = [
            'business_id'       =>          $request->business_id,
            'from_date'         =>          $request->from_date,
            'to_date'           =>          $request->to_date,
            'pdf_genrated_date' =>          $request->pdf_genrated_date,
            'pdf_url'           =>          $request->pdf_url,
            'status'            =>          1,
        ];
    
        $result = DB::table('pdf')->insert($data);
        if($result){
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Export data fetch successfully.','data' => $result]);
        }else{
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => 'Please try again.','data' => $result]);
        }

    }

    public function getLastPdf($id){
        try
        {
            $result = DB::table('pdf')
                            ->where('status', 1)
                            ->where('business_id', $id)
                            ->orderBy('id', 'desc') 
                            ->limit(3)
                            ->get();
            if($result){
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Export data fetch successfully.','data' => $result]);
            }else{
                return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => 'Please try again.','data' => '']);
            }
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.BUSINESS_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }

        
    }

}
