<?php

namespace App\Http\Controllers\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Models\Contractors\ReviewAndRating;

class ReviewAndRatingController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $review_rating;
    protected  $user_id;

    public function __construct(ReviewAndRating $review_rating, Request $request) {
        $this->review_rating = $review_rating;
        $this->user_id = $request->user_id;
    }

    /**
     * @ review rating list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $review_rating = $this->review_rating::where('user_id', '=', $this->user_id)
            ->where(function($q) use($request) {
                // Global searching...
                if(isset($request->search) && !empty($request->search) && $request->search!="") {
                    if(!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if($key==0) {
                                $q->where($col, 'like', '%' .$request->search. '%');
                            } else {
                                $q->orWhere($col, 'like', '%' .$request->search. '%');
                            }
                        }
                    }
                }
            })
            ->where(function($q) use($request) {
                // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_NOT_LISTING'), 'records' => $review_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * store booking api.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'contractor_id' => 'required|numeric',
            'rating' => 'required|in:1,2,3,4,5',
            'review' => 'required|max:256',
        ]);

        try {
            $input = $request->all();
            $input['user_id'] = $this->user_id;
            $review_rating = $this->review_rating::create($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_NOT_STORE'), 'records' => $review_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ contact details api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        
        try
        {
            $review_rating = $this->review_rating::with(['user' => function($query) {
                $query->with('address')->where('status', '=', 1);
            }, 'contractor' => function($query) {
                $query->with(['rating' => function($query) {
                    $query->select('contractor_id', DB::raw('AVG(rating) as avarage_rating'))->groupBy('contractor_id');
                }])
                ->select(['id', 'first_name', 'last_name', 'email', 'phone', 'exprence'])
                ->where('status', '=', 1);
            }])
            ->find($id);

            if( empty($review_rating) ) {
                
                throw new Exception(config('service.USER_API.MESSAGE.REVIEW_RATING_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_INFO'), 'records' => $review_rating]);
        } 
        catch (Exception $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch(Throwable $e) {
            
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified contact us query from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete(Request $request, $id)
    {
        try {
            $review_rating = $this->review_rating::find($id)->delete();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.REVIEW_RATING_NOT_DELETED'), 'records' => $review_rating]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col = $this->review_rating::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }

        return ['id'];
    }
}
