<?php

namespace App\Http\Controllers\Customers;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Models\Providers\Provider;
use App\Models\Providers\Language;
use App\Models\Customers\Customer;
use App\Notifications\Providers\Notification;
use App\Notifications\Customers\Notification as CustomerNotification;
use App\Models\Notifications\NotificationSetting;
use Validator;

class CustomerNotificationController extends Controller
{
    
    public function register($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Inscribirse";
                $message = "Querida ".decryptString($customer->first_name)." ".decryptString($customer->last_name).", Estamos felices de verte en PM Ahora.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Sign Up";
                $message = "Dear ".decryptString($customer->first_name)." ".decryptString($customer->last_name).", We are happy to see you at PM Now.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function changePassword($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Restablecimiento de contraseña";
                $message = "tu contraseña ha sido cambiada exitosamente.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Password Reset";
                $message = "Your password has been changed successfully.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function forgotPassword($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Contraseña olvidada";
                $message = "Su contraseña ha sido cambiada recientemente usando olvidar contraseña.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Forget Password";
                $message = "Your password has been recently changed using forget password.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function updateCustomerProfile($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            // if($language->code == "es"){
            //     /*  push notification   */
            //     $deviceToken = $customer->device_token;
            //     $title = "Editar perfil";
            //     $message = "Ha actualizado correctamente su cuenta.";
            //     sendCustomerPushNotification($deviceToken,$title,$message);
            //     /* end push notification   */

            //     /*  database notification  */
            //     $details = [
            //         'title'    => $title,
            //         'message'    => $message,
            //     ];
            //     $customer->notify(new CustomerNotification($details));
            //     /*  end database */
            // }else{
            //     /*  push notification   */
            //     $deviceToken = $customer->device_token;
            //     $title = "Edit Profile";
            //     $message = "You have successfully updated your account.";
            //     sendCustomerPushNotification($deviceToken,$title,$message);
            //     /* end push notification   */

            //     /*  database notification  */
            //     $details = [
            //         'title'    => $title,
            //         'message'    => $message,
            //     ];
            //     $customer->notify(new CustomerNotification($details));
            //     /*  end database */
            // }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerAddAppointment($customer_id,$provider_id)
    {
        try
        {
            $notificationSetting = NotificationSetting::where('customer_id',$customer_id)->first();
            if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                // send to customer
                $customer = Customer::findOrFail($customer_id);
                $language = Language::findOrFail($customer->language_id);
                // if($language->code == "es"){
                //     /*  push notification   */
                //     $deviceToken = $customer->device_token;
                //     $title = "Cita Reservada";
                //     $message = "Su cita ha sido programada exitosamente.";
                //     sendCustomerPushNotification($deviceToken,$title,$message);
                //     /* end push notification   */

                //     /*  database notification  */
                //     $details = [
                //         'title'    => $title,
                //         'message'    => $message,
                //     ];
                //     $customer->notify(new CustomerNotification($details));
                //     /*  end database */
                // }else{
                //     /*  push notification   */
                //     $deviceToken = $customer->device_token;
                //     $title = "Appointment Booked";
                //     $message = "Your appointment has been booked successfully.";
                //     sendCustomerPushNotification($deviceToken,$title,$message);
                //     /* end push notification   */

                //     /*  database notification  */
                //     $details = [
                //         'title'    => $title,
                //         'message'    => $message,
                //     ];
                //     $customer->notify(new CustomerNotification($details));
                //     /*  end database */
                // }
            }

            $notificationSetting = NotificationSetting::where('provider_id',$provider_id)->first();
            if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                // send to provider
                $provider = Provider::findOrFail($provider_id);
            
                /*  push notification   */
                $deviceToken = $provider->device_token;
                $title = "New Appointment";
                $message = "You have a new appointment.";
                sendPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $provider->notify(new Notification($details));
                /*  end database */
            }


        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerRescheduleAppointment($customer_id,$provider_id)
    {
        try
        {
            $notificationSetting = NotificationSetting::where('customer_id',$customer_id)->first();
            if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                // send to customer
                $customer = Customer::findOrFail($customer_id);
                $language = Language::findOrFail($customer->language_id);
                if($language->code == "es"){
                    /*  push notification   */
                    $deviceToken = $customer->device_token;
                    $title = "Cita reprogramada";
                    $message = "Su cita ha sido reprogramada.";
                    sendCustomerPushNotification($deviceToken,$title,$message);
                    /* end push notification   */

                    /*  database notification  */
                    $details = [
                        'title'    => $title,
                        'message'    => $message,
                    ];
                    $customer->notify(new CustomerNotification($details));
                    /*  end database */
                }else{
                    /*  push notification   */
                    $deviceToken = $customer->device_token;
                    $title = "Appointment rescheduled";
                    $message = "Your appointment has been rescheduled.";
                    sendCustomerPushNotification($deviceToken,$title,$message);
                    /* end push notification   */

                    /*  database notification  */
                    $details = [
                        'title'    => $title,
                        'message'    => $message,
                    ];
                    $customer->notify(new CustomerNotification($details));
                    /*  end database */
                }
            }

            $notificationSetting = NotificationSetting::where('provider_id',$provider_id)->first();
            if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                // send to provider
                $provider = Provider::findOrFail($provider_id);
            
                /*  push notification   */
                $deviceToken = $provider->device_token;
                $title = "Appointment rescheduled";
                $message = "Appointment is rescheduled.";
                sendPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $provider->notify(new Notification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerCancelAppointment($customer_id,$provider_id)
    {
        try
        {
            $notificationSetting = NotificationSetting::where('customer_id',$customer_id)->first();
            if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                // send to customer
                $customer = Customer::findOrFail($customer_id);
                $language = Language::findOrFail($customer->language_id);
                if($language->code == "es"){
                    /*  push notification   */
                    $deviceToken = $customer->device_token;
                    $title = "Cita cancelada";
                    $message = "Su cita ha sido cancelada.";
                    sendCustomerPushNotification($deviceToken,$title,$message);
                    /* end push notification   */

                    /*  database notification  */
                    $details = [
                        'title'    => $title,
                        'message'    => $message,
                    ];
                    $customer->notify(new CustomerNotification($details));
                    /*  end database */
                }else{
                    /*  push notification   */
                    $deviceToken = $customer->device_token;
                    $title = "Appointment Cancelled";
                    $message = "Your appointment has been cancelled.";
                    sendCustomerPushNotification($deviceToken,$title,$message);
                    /* end push notification   */

                    /*  database notification  */
                    $details = [
                        'title'    => $title,
                        'message'    => $message,
                    ];
                    $customer->notify(new CustomerNotification($details));
                    /*  end database */
                }
            }

            $notificationSetting = NotificationSetting::where('provider_id',$provider_id)->first();
            if($notificationSetting->show_notification == 'active' && $notificationSetting->appointment_reminder == 'active'){
                // send to provider
                $provider = Provider::findOrFail($provider_id);
            
                /*  push notification   */
                $deviceToken = $provider->device_token;
                $title = "Appointment Cancelled";
                $message = "One of your appointment is cancelled.";
                sendPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $provider->notify(new Notification($details));
            }
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerChangeProvider($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Solicitud de cambio de proveedor";
                $message = "Ha solicitado exitosamente el cambio de proveedor. Se le notificará una vez que el administrador apruebe la solicitud.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Provider changed request";
                $message = "You have successfully requested to change the provider. You will be notified once the admin approve the request.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }
        
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerChangeProviderRequestApproved($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Solicitud de cambio de proveedor";
                $message = "Tu solicitud de cambio de proveedor ha sido aprobada exitosamente.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Provider changed request";
                $message = "Your request to change provider has been successfully approved.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }
        
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerJournalCreate($customer_id,$provider_id)
    {
        try
        {
            $notificationSetting = NotificationSetting::where('customer_id',$customer_id)->first();
            if($notificationSetting->show_notification == 'active'){
                // send to customer
                $customer = Customer::findOrFail($customer_id);
                $language = Language::findOrFail($customer->language_id);
                if($language->code == "es"){
                    /*  push notification   */
                    $deviceToken = $customer->device_token;
                    $title = "Diario añadido";
                    $message = "El diario ha sido actualizado exitosamente.";
                    sendCustomerPushNotification($deviceToken,$title,$message);
                    /* end push notification   */

                    /*  database notification  */
                    $details = [
                        'title'    => $title,
                        'message'    => $message,
                    ];
                    $customer->notify(new CustomerNotification($details));
                    /*  end database */
                }else{
                    /*  push notification   */
                    $deviceToken = $customer->device_token;
                    $title = "Journal Added";
                    $message = "A new journal added.";
                    sendCustomerPushNotification($deviceToken,$title,$message);
                    /* end push notification   */

                    /*  database notification  */
                    $details = [
                        'title'    => $title,
                        'message'    => $message,
                    ];
                    $customer->notify(new CustomerNotification($details));
                    /*  end database */
                }
            }

            $notificationSetting = NotificationSetting::where('provider_id',$provider_id)->first();
            if($notificationSetting->show_notification == 'active'){
                // send to provider
                $provider = Provider::findOrFail($provider_id);
            
                /*  push notification   */
                $deviceToken = $provider->device_token;
                $title = "New Journal";
                $message = "Consumer has added a new journal.";
                sendPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $provider->notify(new Notification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerSubscriptionPayment($customer_id)
    {
        try
        {

            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                // plan payment
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Pago hecho";
                $message = "Has realizado un pago exitosamente.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */


                // plan activated
                /*  push notification   */
                $title = "Plan activado";
                $message = "Tu plan ya está activados.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                // plan payment
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Payment Done";
                $message = "You have made a payment successfully.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */


                // plan activated
                /*  push notification   */
                $title = "Plan Activated";
                $message = "Your plan is now activated.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerUpdateEmergencyContact($customer_id)
    {
        try
        {
            $customer = Customer::findOrFail($customer_id);
            $language = Language::findOrFail($customer->language_id);
            if($language->code == "es"){
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Contacto de emergencia agregado";
                $message = "Tu contacto de emergencia ha sido guardado.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }else{
                /*  push notification   */
                $deviceToken = $customer->device_token;
                $title = "Emergency Contact added";
                $message = "Your emergency contact has been saved.";
                sendCustomerPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                /*  database notification  */
                $details = [
                    'title'    => $title,
                    'message'    => $message,
                ];
                $customer->notify(new CustomerNotification($details));
                /*  end database */
            }
    
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerSendMessage($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "New Message";
            $message = "Consumer has sent a new message.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */


        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerJoinSession($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Call Joined";
            $message = "Consumer has joined and will wait for 10 minutes.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function customerJoinSessionAgain($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Call Joined";
            $message = "Consumer has joined and waiting for you.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

    public function providerDidNotJoinSession($provider_id)
    {
        try
        {
            $provider = Provider::findOrFail($provider_id);
        
            /*  push notification   */
            $deviceToken = $provider->device_token;
            $title = "Appointment";
            $message = "It seems you did not join the session.";
            sendPushNotification($deviceToken,$title,$message);
            /* end push notification   */

            /*  database notification  */
            $details = [
                'title'    => $title,
                'message'    => $message,
            ];
            $provider->notify(new Notification($details));
            /*  end database */

        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }

}
