<?php

namespace App\Http\Controllers\Services;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use App\Models\Services\Service;

class ServiceController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $service;

    public function __construct(Service $service) {
        $this->service = $service;
    }

    /**
     * @ Service List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $services = $this->service::where('status', '=', 'active')
                        ->where(function($q) use($request) {
                            // Global searching...
                            if(isset($request->search) && !empty($request->search) && $request->search!="") {
                                if(!empty($this->getColumnName())) {
                                    foreach ($this->getColumnName() as $key => $col) {
                                        if($key==0) {
                                            $q->where($col, 'like', '%' .$request->search. '%');
                                        } else {
                                            $q->orWhere($col, 'like', '%' .$request->search. '%');
                                        }
                                    }
                                }
                            }
                        })
                        ->where(function($q) use($request) {
                            // Column wise searching filter ...
                            $allowableColumn = $this->getColumnName();
                            if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                                $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                            }
                        })
                        ->orderBy($order_by_column_name, $order)
                        ->paginate($limit);
            if( empty($services) ) {
                throw new Exception(config('service.USER_API.MESSAGE.SERVICE_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.SERVICE_LIST'), 'records' => $services]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Service List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try {
            $services = $this->service::with('category')->find($id);
            if( empty($services) ) {
                throw new Exception(config('service.USER_API.MESSAGE.SERVICE_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.SERVICE_INFO'), 'records' => $services]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function totalServices()
    {
        try {
            $service = $this->service::count();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.SERVICE_TOTAL'), 'records' => $service]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col = $this->service::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }

        return ['id'];
    }
}
