<?php
namespace App\Http\Controllers\Vendors;

use Exception;
use Throwable;

use Illuminate\Support\Str;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;

use App\Models\Vendors\Vendor;
use App\Models\Vendors\Oauth;

use App\Notifications\Vendors\Notification;

use App\Jobs\SendMailJob;

use Carbon\Carbon;

class ForgotPasswordController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Password Reset Controller
    |--------------------------------------------------------------------------
    |
    | This controller is responsible for handling password reset emails and
    | includes a trait which assists in sending these notifications from
    | your application to your users. Feel free to explore this trait.
    |
    */

    /**
     * Send a reset link to the given user.
     *
     * @param Request $request
     * @return Response
     */
    public function sendResetPasswordLink(Request $request)
    {   
        $message = array(
            'email.exists'   => config('service.USER_API.MESSAGE.NOTEXSIST_EMAIL'),
            'email.email'   => config('service.USER_API.MESSAGE.VALID_EMAIL')
        );

        $this->validate($request, 
            ['email' => 'required|email|exists:vendors', 
        ],$message);

        try
        {
            // create token
            $token = Str::random(60);
            $upd['token'] = $token;
            DB::table('vendors')->where(['email' => $request->email])->update($upd);

            $user = Vendor::select('first_name')->where('email', $request->email)->first();

            $data['name'] = $user->first_name;

             // Genereate link for reset password
            $data['link'] =  env('FRONT_END') . '/' . 'vendor/change_password' . '/' . $token;
            
            
            // Trigger mail
            Mail::send('emails.Users.reset_password',['data'=>$data], function($message) use ($request) {
                $message->to($request->email);
                $message->subject('Reset Password');
            });

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.USER_API.MESSAGE.MAIL_SENT') ]);
        }
        catch(Exception $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
    }

    public function updatePassword(Request $request, $token)
    {  
        $message = array(
            'password.regex' => config('service.USER_API.MESSAGE.PASSWORD_REGEX'),
        );

        $this->validate($request, [
            'password' => 'required|string|min:8|regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/',
            'password_confirmation' => 'required|same:password', 
        ],$message);

        try
        {

            $vendor = Vendor::select('id','first_name','email')->where('token', $token)->first();

            if(!$vendor) {
                return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => config('service.USER_API.MESSAGE.INVALID_TOKEN') ]);
            }

            $save_data['password'] = sha1($request->password);
            $result = Oauth::where('user_fk', $vendor->id)->where('role_fk', '2')->update($save_data);
            
            $greeting = 'Hi ' . $vendor->first_name . '!';
            $body     = 'Your password to login as vendor pannel has been recently changed';
            $type     = 'Password Updated';

            $details = [
                'greeting' => $greeting,
                'body' => $body,
                'type' => $type,
                'notification_message' => 'You have change your password successfully.',
                'thanks' => 'Regards, Thanks',
            ];
            
            $role = '3';
            $vendor->notify(new Notification($details));
            $upd['role'] = $role;
            $notif_id = $vendor->notifications[0]->id;
            $vendor->notifications()->find($notif_id)->update($upd);

            $sendMail = new SendMailJob($vendor->email, $greeting, $body, $type);
            dispatch($sendMail);
        
           

            Vendor::where('token', $token)->update(['token' => null]);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.USER_API.MESSAGE.PASSWORD_UPDATED') ]);
        }
        catch(Exception $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
        catch(Throwable $e)
        {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage() , 'records' => []]);
        }
    }

}

