<?php

namespace App\Http\Controllers\v3\Auth;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Laravel\Lumen\Routing\Controller as BaseController;
use App\Models\Auth\Oauth;
use App\Models\LoginActivity\LoginActivity;
use App\Models\Auth\Otp;
use App\Models\Users\User as Admin;
use App\Models\Providers\Provider;
use App\Models\Customers\Customer;
use App\Models\Settings\Setting;
use App\Models\IpLoginAttempt\IpLoginAttempt;
use App\Jobs\CustomerMailJob;
use Carbon\Carbon;
use Validator;
use Illuminate\Support\Facades\Mail;
use App\Jobs\SendTwoFactorJob;
use App\Models\EmailTemplate\EmailTemplate;
use App\Jobs\ProviderConfirmEmailJob;
use App\Jobs\CustomerConfirmEmailJob;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\App;
use App\Models\ChangeProvider\ChangeProvider;
use DB;

class OauthServerController extends BaseController
{
    var $server = null;
    var $storage = null;
    var $config = null;
    var $userid = null; // A value on your server that identifies the user.

    public function __construct() {
        \OAuth2\Autoloader::register();
        //$this->setAccessLifetime(900); // 86400 = 24 hours.
        $this->setAccessLifetime(86400); // 86400 = 24 hours.
        $this->oauth2Connect();
    }


    public function oauth2Connect() {
        $this->storage = new \OAuth2\Storage\Pdo(array(
            'dsn' => 'mysql:dbname='.getenv('DB_DATABASE').';host='.getenv('DB_HOST'),
            'username' => getenv('DB_USERNAME'),
            'password' => getenv('DB_PASSWORD'),
        ), ['user_table' => 'oauth_users']);

        $this->server = new \OAuth2\Server($this->storage, $this->config);
       
        $options = array('always_issue_new_refresh_token' => true, 'use_jwt_access_tokens' => true);
        
        // any grant type you like!
        $this->server->addGrantType(new \OAuth2\GrantType\ClientCredentials($this->storage, $options));
        $this->server->addGrantType(new \OAuth2\GrantType\AuthorizationCode($this->storage));
        $this->server->addGrantType(new \OAuth2\GrantType\UserCredentials($this->storage));
        $this->server->addGrantType(new \OAuth2\GrantType\RefreshToken($this->storage, $options));
    }
    
    
    public function getOTP(Request $request)
    {   
        $this->validate($request, [ 
            'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|exists:customers,phone',
        ]);

        $customer = Oauth::select('user_id','username As first_name','username As last_name','email')->where('mobile', $request->phone)->first();

        $request = \OAuth2\Request::createFromGlobals();
        $response = new \OAuth2\Response();

        // validate the authorize request
        if (!$this->server->validateAuthorizeRequest($request, $response)) {
            pre($response);
            die;
        }
        
        // print the authorization code if the user has authorized your client
        $is_authorized = true;

        if ($is_authorized) {
          
            $this->server->handleAuthorizeRequest($request, $response, $is_authorized, $customer->id);
                
            // this is only here so that you get to see your code in the cURL request. Otherwise, we'd redirect back to the client
            $code = substr($response->getHttpHeader('Location'), strpos($response->getHttpHeader('Location'), 'code=')+5, 4);
           
            $greeting = 'Dear ' . $customer->first_name . ',';
            $body     = 'Your login OTP is : ' . $code . '.';
            $type     =  'Login OTP';

            //customer mail
            // $sendMail = new customerMailJob($customer->email, $greeting, $body, $type);
            // dispatch($sendMail);

            return response()->json([
                'status'  => 'success',
                'message' => 'OTP successfully sent on your registered email id.'
             ]);
        }
       

        $response->send();
    }
    
    
    public function authRedirected() {
        die('authRedirected');
    }


    private function createUser() {
        // $username = 'seanrock1';
        // $password = 'Sean@1234';
        // $firstName = 'Sean1';
        // $lastName = 'Rock';
        // $this->storage->setUser($username, $password, $firstName, $lastName);
    }

    
    private function handleRefreshToken() {
        // Handle a request for an OAuth2.0 Access Token and send the response to the client
        $token_obj = $this->server->handleTokenRequest(\OAuth2\Request::createFromGlobals());

        $tokenData = $token_obj->getParameters();

        if (isset($tokenData['error'])) {
           return $tokenData;
        }

        list($header, $payload, $signature) = explode('.', $tokenData['access_token']);
        
        //In your case you need only $payload but it's a copy paste from my code testI
        $payloadAsString = base64_decode($payload);
        $payloadAsObject = json_decode($payloadAsString);
        
        $oauth = Oauth::select('user_id','role_fk','user_fk','email','username')->where('user_id',$payloadAsObject->sub)->first();
        
        return $this->prepareTokenResponse($tokenData,$oauth);
    }
    
    public function handleAccessTokenBruteForce()
    {
        $obj = Oauth::where('email', request()->username)->first();


        if (request()->user_type=="provider" && empty($obj)) {
            $msg = request()->language == 'es' ? 'No ha introducido credenciales registradas. ¡Pruebe con uno diferente o regístrese ahora!' : 'You have not entered a registered credentials. Please try a different one or Sign up now!' ;
            
            return response()->json([
                'status'  => 'error',
                'message' => $msg
            ]);
        }

        if (request()->user_type=="customer" && empty($obj)) {

            $header = request()->header('locale');
            if($header == 'en'){
                $msg = 'You have not entered a registered credentials. Please try a different one or Sign up now!';
            }else{
                $msg = 'No ha ingresado una credencial registrada. Intente con uno diferente o regístrese ahora!';
            }
            return response()->json([
                'status'  => 'error',
                'message' => $msg
            ]);
        }

        if (request()->user_type=="admin" && empty($obj)) {
            $loginActivity = new LoginActivity();
            $loginActivity->email = request()->username;
            $loginActivity->login_status = "failed";
            $loginActivity->description = "Entered wrong email";
            $loginActivity->save();
            return response()->json([
                'error'  => 'invalid_grant',
                'error_description' => 'Dear Admin, You have not entered a registered email. Please enter the registered email and try again.'
            ]);
        }


        //$this->validateRoles($obj);

        // provider
        /*if(request()->user_type=="provider" && !$this->isProviderApproved($obj)) {

            return response()->json([
               'error'  => 'invalid_grant',
               'error_description' => 'Your acccount approval is pending.'
            ]);
        }*/

        // provider.
        if(request()->user_type=="provider") {
             $provider = Provider::where('email',request()->username)->first();
             if($provider)
             {
                if($provider->status == "inactive"){
                    return response()->json([
                        'status'  => 'error',
                        'message' => 'Your account has been inactive. Please contact to administrator.'
                    ]);
                 }else if($provider->email_confirmed == "no"){
                    
                        if($provider->email_confirm_token == '' ){
                            $token   = Str::random(60);
                            DB::table('providers')
                            ->where('id', $provider->id)
                            ->update(['email_confirm_token' => $token]);
                        }else{                            
                            $token   = $provider->email_confirm_token;
                        }

                        $templateName = "Provider account verification";
                        $templateData = EmailTemplate::where('name',$templateName)->first();
                        if($templateData)
                        {
                            /* Confirm email */
                            $greeting = 'Hi ' . decryptString($provider->first_name)." ".decryptString($provider->last_name).",";
                            $body     = $templateData->text;
                            $type     = 'PM Now | Reverify Your Email';
                            $link     = env('FRONT_END') . '/' . 'admin/provider-confirm-email/'.$token;
                            // $sendMail = new ProviderConfirmEmailJob(decryptString($provider->email), $greeting, $body, $link, $type);
                            // $this->dispatchNow($sendMail);
                            /* End confirm email  */
                        }
                        
                        $link     = env('FRONT_END') . '/' . 'admin/provider-confirm-email/'.$token;
                        $moengageData =  [
                            "type" => "event",
                            "customer_id" => decryptString($provider->email),
                            "actions" => [
                                [
                                    "action" => "re_account_verification",
                                    "attributes" => [
                                        "provider_email_address" => decryptString($provider->email),
                                        "verify_email_link" => $link
                                    ],
                                    "platform" => "web"
                                ]
                            ]
                        ];

                        moengageCallCustomEventProviderApiRequest($moengageData);

                        return response()->json([
                            'status'  => 'error',
                            'message'=>'Your email is not verified. We have sent you an email again to verify your email address.',
                            //'message' => 'Your account has been inactive. Please contact to administrator.'
                        ]);
                        
                 }
                 $update = array();
                 $update['device_id']    = request()->device_id;
                 $update['device_token'] = request()->device_token;
                 $update['device_type']  = request()->device_type;
                 Provider::where('id',$obj->user_fk)->update($update);
             }
             else
             {
                
                //return response()->json($obj);

                return response()->json([

                    'status'  => 'error',
                    'message' => 'You have not entered a registered credentials. Please try a different one or Sign up now!'
                ]);
             }
        }

        // customer.
        if(request()->user_type=="customer") {
             $customer = Customer::where('email',request()->username)->first();
            if($customer)
             {

               // return response()->json($customer->status);
                if($customer->status == "inactive"){

                    if($customer->email_confirm_token == '' ){
                        $token   = Str::random(60);
                        DB::table('customers')
                        ->where('id', $customer->id)
                        ->update(['email_confirm_token' => $token]);
                    }else{                            
                        $token   = $customer->email_confirm_token;
                    }

                    $greeting =  'Hi ' . decryptString($customer->first_name).' '.decryptString($customer->last_name).',';

                    // echo "hii";exit;
                    if(App::getLocale() == "es"){
                        $gratitude     = config('constant.gratitude_spanish');
                        $regard     = config('constant.regard_spanish');
                        $footer     = config('constant.footer_spanish');
                    }else{
                        $gratitude     = config('constant.gratitude_english');
                        $regard     = config('constant.regard_english');
                        $footer     = config('constant.footer_english');
                    }

                    $templateName = "Consumer account verification";
                    $templateData = EmailTemplate::where('name',$templateName)->first();
                    if($templateData)
                    {
                        /* Confirm email */
                            $body     = $templateData->text;
                            $type     = $templateData->title;
                            $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $token;
                            // $sendMail = new CustomerConfirmEmailJob(decryptString($customer->email), $greeting, $body, $link, $type, $gratitude, $regard, $footer);
                            // $this->dispatchNow($sendMail);
                        /* End confirm email  */
                    }


                    $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $token;
                    $moengageData =  [
                        "type" => "event",
                        "customer_id" => decryptString($customer->email),
                        "actions" => [
                            [
                                "action" => "user_account_re_verification",
                                "attributes" => [
                                    "user_email_address" => decryptString($customer->email),
                                    "verify_email_link" => $link
                                ],
                                "platform" => "web"
                            ]
                        ]
                    ];

                    moengageCallCustomEventApiRequest($moengageData);

                    return response()->json([
                        'status'  => 'error',
                        'message'=>'Your account has been inactive. Please check your email and verify .',
                        //'message' => 'Your account has been inactive. Please contact to administrator.'
                    ]);
                }else if($customer->email_confirmed == "no" ){

                    if($customer->email_confirm_token == '' ){
                        $token   = Str::random(60);
                        DB::table('customers')
                        ->where('id', $customer->id)
                        ->update(['email_confirm_token' => $token]);
                    }else{                            
                        $token   = $customer->email_confirm_token;
                    }

                    /** Re Send consumer verifcation mail ***/
                    $greeting = 'Hi ' . decryptString($customer->first_name).' '.decryptString($customer->last_name).',';

                    // echo "hii";exit;
                    if(App::getLocale() == "es"){
                        $gratitude     = config('constant.gratitude_spanish');
                        $regard     = config('constant.regard_spanish');
                        $footer     = config('constant.footer_spanish');
                    }else{
                        $gratitude     = config('constant.gratitude_english');
                        $regard     = config('constant.regard_english');
                        $footer     = config('constant.footer_english');
                    }

                    $templateName = "Consumer account verification";
                    $templateData = EmailTemplate::where('name',$templateName)->first();
                    if($templateData)
                    {
                    /* Confirm email */
                        $body     = $templateData->text;
                        // $type     = $templateData->title;
                        $type     = 'PM Now | Reverify Your Email';
                        $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $customer->email_confirm_token;
                        // $sendMail = new CustomerConfirmEmailJob(decryptString($customer->email), $greeting, $body, $link, $type, $gratitude, $regard, $footer);
                        // $this->dispatchNow($sendMail);
                    /* End confirm email  */
                    }

                    

                    $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $token;
                    $moengageData =  [
                        "type" => "event",
                        "customer_id" => decryptString($customer->email),
                        "actions" => [
                            [
                                "action" => "user_account_re_verification",
                                "attributes" => [
                                    "user_email_address" => decryptString($customer->email),
                                    "verify_email_link" => $link
                                ],
                                "platform" => "web"
                            ]
                        ]
                    ];
                    moengageCallCustomEventApiRequest($moengageData);

                    return response()->json([
                        'status'  => 'error',
                        'message'=>'Your email is not verified. We have sent you an email again to verify your email address.',
                        //'message' => 'Your account has been inactive. Please contact to administrator.'
                    ]);
                    
                    /** Re Send consumer verifcation mail ***/
                }
                $update = array();
                $update['device_id']    = request()->device_id;
                $update['device_token'] = request()->device_token;
                $update['device_type']  = request()->device_type;
                Customer::where('id',$obj->user_fk)->update($update);
             }
             else
             {
                return response()->json([
                    'status'  => 'error',
                    'message' => 'You have not entered a registered credentials. Please try a different one or Sign up now!'
                ]);
             }
        }

        // admin
        if(request()->user_type=="admin") {
             $admin = Admin::where('email',request()->username)->first();
             if(empty($admin))
             {
                return response()->json([
                    'error'  => 'invalid_grant',
                    'error_description' => 'Dear Admin, You have entered a wrong email. Please enter the correct email and try again.'
                ]);
             }
             if($admin->status == "1"){
                 return response()->json([
                     'error'  => 'invalid_grant',
                     'error_description' => 'Your account has been inactive. Please contact to administrator.'
                 ]);
             }
        }


        if (isset($obj->is_block) && $obj->is_block == 1) {
            header("HTTP/1.1 401");
            header('Content-Type: application/json');
            return response()->json([
                'error'  => 'invalid_grant',
                'error_description' => 'Your account has been blocked. If you want to activate it contact to administration!'
            ]);
        }
        
        
        $form = \OAuth2\Request::createFromGlobals();
        $form->request['username'] = $obj->username;
        
        // Handle a request for an OAuth2.0 Access Token and send the response to the client
        $token_obj = $this->server->handleTokenRequest($form);
        $tokenData = $token_obj->getParameters();

        // return response()->json([
        //     'token_obj'  => $token_obj,
        //     'tokenData'  => $form,
        // ]);
        
        return $this->bruteForce($tokenData, $obj);
    }
    
    
    private function handleAccessToken()
    {
        // Handle a request for an OAuth2.0 Access Token and send the response to the client
        $token_obj = $this->server->handleTokenRequest(\OAuth2\Request::createFromGlobals());
        
        $tokenData = $token_obj->getParameters(); 
      
        if(isset($tokenData['access_token'])) {
            list($header, $payload, $signature) = explode('.', $tokenData['access_token']);
        
            //In your case you need only $payload but it's a copy paste from my code testI
            $payloadAsString = base64_decode($payload);
            $payloadAsObject = json_decode($payloadAsString);

            //update device token
            $update = array();
            $update['device_id']    = request()->device_id;
            $update['device_token'] = request()->device_token;
            $update['device_type']  = request()->device_type;
            Customer::where('id',$payloadAsObject->sub)->update($update);
            
            $oauth = Oauth::select('user_id','role_fk','user_fk')->where('user_fk',$payloadAsObject->sub)->first();
            $tokenData['login_id']  = (int) $oauth->user_id;
            $tokenData['role_fk']   = (int) $oauth->role_fk;
            $tokenData['user_fk']   = (int) $oauth->user_fk;
            return $tokenData;
        }else{
            return response()->json([
                'error'  => 'invalid_grant',
                'error_description' => 'Something went wrong. Please try again.'
            ]);
        }
        
        
    }
    

    public function token(Request $request)
    {   
        if (request()->grant_type === 'authorization_code') {
            $this->handleRefreshTokenWithOtp();
        }
        else if(request()->grant_type === 'refresh_token') {
            //return $this->handleRefreshToken();
        }        
        else if (request()->grant_type === 'password') {
            
            // if provider
            if(isset($request->user_type) && $request->user_type=="provider" || $request->user_type=="customer") {
                $this->validate($request, [
                    'device_id'    => 'required',
                    'device_token' => 'required',
                    'device_type'  => 'required|in:android,ios'
                ]);
            }

            
            return $this->handleAccessTokenBruteForce();            
        }
        else {


            return $this->handleAccessToken();
        }
    }
    
    
    public function bruteForce($tokenData, $obj)
    {

        $success = isset($tokenData['error']) ? false : true;
        
        $apc_key = $obj->user_id.":{$_SERVER['SERVER_NAME']}~wallet:app:login:{$_SERVER['REMOTE_ADDR']}";
        $apc_blocked_key = $obj->user_id."{$_SERVER['SERVER_NAME']}~wallet:app:login-blocked:{$_SERVER['REMOTE_ADDR']}";

        $tries = (int) \Cache::store('file')->get($apc_key);

        //if ($tries >= 5 && ($obj->role_fk != 3)) {
        $attempt =  Setting::where('name','no_of_attempt')->value('value');
        if(request()->user_type=="admin") {
            if ($tries >= $attempt) {
                header("HTTP/1.1 429 Too Many Requests");
                header('Content-Type: application/json');

                $error = "You are blocked as you've exceeded the number of login attempts. You can login back in 2 hours.";

        
                $admin = Admin::find($obj->user_fk);
                if(isset($admin->id)) {
                    $greeting = 'Dear ' . $admin->first_name . ',';
                    $body     = $error;
                    $type     =  'Account Blocked Mail';
                    $sendMail = new customerMailJob($admin->email, $greeting, $body, $type);
                    $x = dispatch($sendMail);
                }
                
                return response()->json([
                    'error'  => 'ip_blocked',
                    'error_description' => $error,
                    'a' => @$admin,
                    'sendMail' => @$sendMail,
                    'xx' => @$x,
                    'aa' => env('MAIL_FROM_ADDRESS')
                ]);
                
            }
        }
        
        if (!$success) {

            $blocked = (int) \Cache::store('file')->get($apc_blocked_key);

            # store tries for 2^(x+1) minutes: 2, 4, 8, 16, ...
            \Cache::store('file')->put($apc_key, $tries+1, pow(2, $blocked+1)*60);

            # store number of times blocked for 2 hours
            \Cache::store('file')->put($apc_blocked_key, $blocked+1, 7200);
            

            if(request()->user_type =="customer"){
                /* login attempts  */
                $user = Customer::find($obj->user_fk);

                $ipAddress = request()->ip();
                $userloginAttempt = $user->login_attempt;
                $attemptCount = $userloginAttempt+1;
                if($attemptCount ==  config('service.LOGIN_ATTEMPT'))
                {
                    $user->is_blocked = "yes";
                    $user->login_attempt = $attemptCount;
                    $user->login_attempt_time = strtotime(date("Y-m-d H:i:s"));
                    $user->save();

                    return response()->json([
                        'status'  => 'error',
                        'message' => "Too many login attempts. Your account has been suspended for 5 minutes."
                    ]);
                }
                if ($attemptCount ==  config('service.LOGIN_ATTEMPT_IP_COUNT')) 
                {
                    $user->login_attempt = $attemptCount;
                    $user->save();
                    $ipLoginAttempt = new IpLoginAttempt();
                    $ipLoginAttempt->email = $user->email;
                    $ipLoginAttempt->ip_address = $ipAddress;
                    $ipLoginAttempt->save();

                    return response()->json([
                        'status'  => 'error',
                        'message' => "Too many login attempts. Your Ip address has been suspended, please contact to admin."
                    ]);
                }
                if($attemptCount <= config('service.LOGIN_ATTEMPT_IP_COUNT'))
                {
                    $user->login_attempt = $attemptCount;
                    $user->save();
                }
                $checkIp = IpLoginAttempt::where(['ip_address'=>$ipAddress,'is_blocked'=>"yes"])->first();
                if($checkIp)
                {
                    if($user->login_attempt == config('service.LOGIN_ATTEMPT_IP_COUNT') && $user->is_blocked == "yes")
                    { 
                        return response()->json([
                            'status'  => 'error',
                            'message' => "Too many login attempts. Your Ip address has been suspended, please contact to admin."
                        ]);
                    } 
                }
                if($user->login_attempt >= config('service.LOGIN_ATTEMPT') && $user->is_blocked == "yes")
                { 
                    return response()->json([
                            'status'  => 'error',
                            'message' => "Too many login attempts. Your account has been suspended for 5 minutes."
                        ]);
                } 
                /* login attempts */ 

                $header = request()->header('locale');
                if($header == 'en'){
                    $msg = "The password you entered is wrong. Please try another";
                }else{
                    $msg = "La contraseña que ingresaste es incorrecta. Por favor, intente con otro";
                }
                return response()->json([
                    'status'  => 'error',
                    'message' => $msg
                ]);
            }
            if(request()->user_type =="provider"){
                /* login attempts  */
                $user = Provider::find($obj->user_fk);

                $ipAddress = request()->ip();
                $userloginAttempt = $user->login_attempt;
                // echo "<pre>";
                // print_r($userloginAttempt);
                // exit;
                if($userloginAttempt == ''){
                    $userloginAttempt = 0;
                    $attemptCount = $userloginAttempt+1;
                }else{
                    $attemptCount = $userloginAttempt+1;
                }
                
                if($attemptCount ==  config('service.LOGIN_ATTEMPT'))
                {
                    $user->is_blocked = "yes";
                    $user->login_attempt = $attemptCount;
                    $user->login_attempt_time = strtotime(date("Y-m-d H:i:s"));
                    $user->save();

                    return response()->json([
                        'status'  => 'error',
                        'message' => "Too many login attempts. Your account has been suspended for 5 minutes."
                    ]);
                }
                if ($attemptCount ==  config('service.LOGIN_ATTEMPT_IP_COUNT')) 
                {
                    $user->login_attempt = $attemptCount;
                    $user->save();
                    $ipLoginAttempt = new IpLoginAttempt();
                    $ipLoginAttempt->email = $user->email;
                    $ipLoginAttempt->ip_address = $ipAddress;
                    $ipLoginAttempt->save();

                    return response()->json([
                        'status'  => 'error',
                        'message' => "Too many login attempts. Your Ip address has been suspended, please contact to admin."
                    ]);
                }
                if($attemptCount <= config('service.LOGIN_ATTEMPT_IP_COUNT'))
                {
                    $user->login_attempt = $attemptCount;
                    $user->save();
                }
                $checkIp = IpLoginAttempt::where(['ip_address'=>$ipAddress,'is_blocked'=>"yes"])->first();
                if($checkIp)
                {
                    if($user->login_attempt == config('service.LOGIN_ATTEMPT_IP_COUNT') && $user->is_blocked == "yes")
                    { 
                        return response()->json([
                            'status'  => 'error',
                            'message' => "Too many login attempts. Your Ip address has been suspended, please contact to admin."
                        ]);
                    } 
                }
                if($user->login_attempt >= config('service.LOGIN_ATTEMPT') && $user->is_blocked == "yes")
                { 
                    return response()->json([
                            'status'  => 'error',
                            'message' => "Too many login attempts. Your account has been suspended for 5 minutes."
                        ]);
                }
                /* login attempts */ 
                return response()->json([
                    'status'  => 'error',
                    'message' => "The password you entered is wrong. Please try another"
                ]); 
            }
            return $tokenData;            
        }
        else 
        {
            \Cache::store('file')->forget($apc_key);
            \Cache::store('file')->forget($apc_blocked_key);

            if(isset($obj) && request()->user_type=="provider")
            {
                /* To check user is blocked or not   */
                    $user = Provider::find($obj->user_fk);
                    $validityTime = strtotime(date("Y-m-d H:i:s"));
                    $login_attempt_time = $user->login_attempt_time;
                    if($login_attempt_time == ''){
                        $login_attempt_time = 0;
                    }
                    $blockTime = $validityTime - $login_attempt_time;
                    if($user->login_attempt >= config('service.LOGIN_ATTEMPT') && $blockTime < config('service.BLOCK_TIME') && $user->is_blocked == "yes")
                    { 
                        return response()->json([
                            'status'  => 'error',
                            'message' => "Too many login attempts. Your account has been suspended for 5 minutes."
                        ]);
                    }
                /*  To check user is blocked or not   */ 

                $provider = Provider::find($obj->user_fk);
                if($provider->two_factor == "enable")
                {
                   $provider->generateTwoFactorCode();

                   return response()->json([
                           'status' => "success",
                           'message' => 'Two factor code has been sent on your email successfully.'
                       ],220);
                }
            }
            if(isset($obj) && request()->user_type=="customer")
            {
                
                /* To check user is blocked or not   */
                    $user = Customer::find($obj->user_fk);
                    
                    $validityTime = strtotime(date("Y-m-d H:i:s"));
                    $login_attempt_time = $user->login_attempt_time;
                    if($login_attempt_time == ''){
                        $login_attempt_time = 0;
                        $blockTime = $validityTime - $login_attempt_time;
                    }else{
                        $blockTime = $validityTime - 0;
                    }
                    // $blockTime = $validityTime - $login_attempt_time;
                    if($user->login_attempt >= config('service.LOGIN_ATTEMPT') && $blockTime < config('service.BLOCK_TIME') && $user->is_blocked == "yes")
                    { 
                        return response()->json([
                            'status'  => 'error',
                            'message' => "Too many login attempts. Your account has been suspended for 5 minutes."
                        ]);
                    }
                /*  To check user is blocked or not   */ 

                $customer = Customer::find($obj->user_fk);
                if($customer->two_factor == "enable")
                {
                   $customer->generateTwoFactorCode();

                   $header = request()->header('locale');
                   if($header == 'en'){
                        $msg = 'Two factor code has been sent on your email successfully.';
                   }else{
                        $msg = 'El código de dos factores se ha enviado a su correo electrónico con éxito.';
                   }
                   return response()->json([
                           'status' => "success",
                           'message' => $msg
                       ],220);
                }
            }

            return $this->prepareTokenResponse($tokenData, $obj);
        }
    }
    
    
    private function prepareTokenResponse($tokenData, $obj = null)
    {
        if (isset($tokenData['error'])) {
            return $tokenData;
        }
        
        list($header, $payload, $signature) = explode('.', $tokenData['access_token']);
        
        //In your case you need only $payload but it's a copy paste from my code testI
        $payloadAsString = base64_decode($payload);
        $payloadAsObject = json_decode($payloadAsString);
       
        
        $tokenData['login_id'] = (int) $payloadAsObject->sub ; // Add a new property to your $tokenData object
        $tokenData['user_fk']  = (int) $obj->user_fk;

        // if(request()->user_type=="admin"){

        //     //$command = 'node -e "console.log(Intl.DateTimeFormat().resolvedOptions().timeZone);"';
        //     $ip = file_get_contents("http://ipecho.net/plain");
        //     $ipInfo = file_get_contents('http://ip-api.com/json/' . $ip);
        //     $ipInfo = json_decode($ipInfo);
        //     $timezone = $ipInfo->timezone;
        //     date_default_timezone_set($timezone);
        //     $output = date_default_timezone_get();
           
        //     $Data['time_zone'] = $output;
            
        //     $updateCode = Oauth::where('email', $obj->email)->update($Data);
        //     if(isset($output)){
        //         $tokenData['time_zone']  = $output;
        //     }else{
        //         $tokenData['time_zone']  = $obj->time_zone;
        //     }
        // }

        if(request()->user_type=="admin"){
            $ip = file_get_contents("http://ipecho.net/plain");
            $ipInfo = file_get_contents('http://ip-api.com/json/' . $ip);
            $ipInfo = json_decode($ipInfo);
            $timezone = $ipInfo->timezone;
            $tokenData['time_zone']=$timezone;
        }

        // if(request()->user_type=="admin"){

        //     // $command = 'node -e "console.log(Intl.DateTimeFormat().resolvedOptions().timeZone);"';
        //     // $output = exec($command);
        //     $command = 'node -e "console.log(new Date().getTimezoneOffset());"';
        //     $timezoneOffset = exec($command);
        //     $timezoneSeconds = abs($timezoneOffset) * 60;
        //     $output = timezone_name_from_abbr('', $timezoneSeconds, 0);
        //     $tokenData['time_zone']  = $output;
        //     $Data['time_zone']  = $output;
        //     $updateCode = Oauth::where('email', $obj->email)->update($Data);
        //     if(isset($output)){
        //         $tokenData['time_zone']  = $output;
        //     }else{
        //         $tokenData['time_zone']  = $obj->time_zone;
        //     }
        // }

        /* for admin */
        if (isset($obj->role_fk) && request()->user_type=="admin" )
        {
            $twoFactor = siteConfig('two_factor');
            if($twoFactor == "yes")
            { 
                $user = Admin::find($obj->user_fk);
                 
                $digits = 4;
                $toEmail = $user->email;
                $name = $user->first_name." ".$user->last_name;
                $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);

                $sendMail =  new SendTwoFactorJob($name,$toEmail,$otpCode); 
                dispatch($sendMail);

                $fData = [
                    'two_factor_code' => $otpCode,
                    'two_factor_expires_at' => strtotime(date("Y-m-d H:i:s"))
                ];
                $updateCode = Admin::where('email', $obj->email)->update($fData);
                $adminData = Admin::where('email', $obj->email)->first();

                $tokenData['two_factor']  = $twoFactor;
                $tokenData['two_factor_code']  = $adminData->two_factor_code;
            }
            $tokenData['role_fk']  = (int) $obj->role_fk;  
        }
        /* for admin */

        /* for provider */
        elseif(isset($obj) && request()->user_type=="provider")
        {
            $token   = Str::random(60);
            $provider = Provider::with(['providerAvailability'])->find($obj->user_fk);
            $provider->is_blocked = "no";
            $provider->login_attempt = null;
            $provider->login_attempt_time = null;
            $provider->email_confirm_token = $token;
            $provider->save();

            if($provider->email_confirmed == "no"){
                /* Check provider email  verified*/
                    /* Confirm email */
                    $greeting = 'Hi ' . decryptString($provider->first_name)." ".decryptString($provider->last_name);
                    $body     = "Press this button and you are on your way to confirm your email.";
                    $type     = "PM Now | Verify Your Email";
                    $link     = env('FRONT_END') . '/' . 'admin/provider-confirm-email/'. $token;
                    $sendMail = new ProviderConfirmEmailJob(decryptString($provider->email), $greeting, $body, $link, $type);
                    dispatch($sendMail);
                    /* End confirm email  */
                    
                /* end check provider email  */
            }
            
            unset($tokenData['scope']);

            $tokenData['register_status']   = $provider->register_status;
            $tokenData['register_step']     = $provider->register_step;
            $tokenData['email_confirmed']   = $provider->email_confirmed;
            if($provider->email_confirmed !== "yes"){
                $tokenData['email_confirmed_message']   = "Please verify your email address.";
            }
            $tokenData['application_status']   = $provider->application_status;
            if($provider->application_status !== "Approved"){
                $tokenData['application_status_message']   = "Your account is not approved, Please contact to admin.";
            }
            $tokenData['two_factor']   = $provider->two_factor;
            $tokenData['set_availability']  = count($provider->providerAvailability) ? "yes": "no";

            if($provider->pin == ''){                
                $tokenData['pin'] = false;
            }else{                
                $tokenData['pin'] = true;
            }
            
        }
        /* for provider */

        /* for customer */
        elseif(isset($obj) && request()->user_type=="customer")
        {
            $token   = Str::random(60);
            $customer = Customer::with(['plan'])->find($obj->user_fk);
            $customer->is_blocked = "no";
            $customer->login_attempt = null;
            $customer->login_attempt_time = null;
            $customer->last_viewed_at = date("Y-m-d H:i:s");
            $customer->email_confirm_token = $token;
            $customer->save();

            if($customer->email_confirmed == "no"){
                /* Confirm email */
                $header = request()->header('locale');
                App::setLocale($header);
                $templateName = "Consumer account verification";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                $greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name).",";
                if($templateData)
                {
                    /* Confirm email */
                    if(App::getLocale() == "es"){
                        $regard     = config('constant.regard_spanish');
                        $footer     = config('constant.footer_spanish');
                    }else{
                        $regard     = config('constant.regard_english');
                        $footer     = config('constant.footer_english');
                    }

                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $token;
                    // $sendMail = new CustomerConfirmEmailJob(decryptString($customer->email), $greeting, $body, $link, $type, $regard,$footer);
                    // dispatch($sendMail);
                    /* End confirm email  */
                }

                /*$greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name).",";
                $body     = "Please click the below link to verify your email address.";
                $type     = "PM Now | Verify Your Email";
                $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $token;
                $sendMail = new CustomerConfirmEmailJob(decryptString($customer->email), $greeting, $body, $link, $type);
                dispatch($sendMail);*/
                /* End confirm email  */
            }
            
            /* Check change provider request  */
                $checkRequest = ChangeProvider::where(['customer_id' => $customer->id, 'status' => 'pending'])->first();
                if(!empty($checkRequest)){
                    $tokenData['change_provider_request'] = "yes";
                }else{
                    $tokenData['change_provider_request'] = "no";
                }
            /* End check provider request  */

            unset($tokenData['scope']);

            $tokenData['register_status']   = $customer->register_status;
            $tokenData['register_step']     = $customer->register_step;
            $tokenData['email_confirmed']   = $customer->email_confirmed;
            $tokenData['first_name']        = $customer->first_name;
            $tokenData['last_name']         = $customer->last_name;
            $tokenData['email']             = $customer->email;
            $tokenData['image']             = $customer->image;
            $tokenData['two_factor']        = $customer->two_factor;
            $tokenData['plan']              = $customer->plan;
            $tokenData['current_provider_id'] = $customer->current_provider_id;
            if($customer->pin == ''){                
                $tokenData['pin'] = false;
            }else{                
                $tokenData['pin'] = true;
            }
        }
        /* for provider */

        /* activity logs */
            $adminData = Oauth::where('email', $obj->email)->first();
            activity('Login')
            ->causedBy($adminData)
            ->log('Login Successful');
        /* activity logs  */


        /*  login logs */
             $loginActivity = new LoginActivity();
             $loginActivity->user_id = $adminData->user_fk;
             $loginActivity->email = $adminData->email;
             $loginActivity->login_status = "success";
             $loginActivity->description = null;
             $loginActivity->save();
        /*  login logs */

        if(request()->user_type=="provider" || request()->user_type=="customer"){
            return response()->json(['status' => 'success','message' =>'Login successfully.','records'=>$tokenData]); 
        }
        return $tokenData;
    }
        
    
    private function validateRoles($obj)
    {
        if (isset(request()->role) && count(request()->role) > 0) {
           if (!isset($obj->role_fk) || !in_array($obj->role_fk, request()->role)) {
                return response()->json([
                    'error'  => 'invalid_grant',
                    'error_description' => "The username and password doesn't match. Please try again."
                ]);
           }           
        }
    }


    public function authorizationCode()
    {
        $request = \OAuth2\Request::createFromGlobals();
        $response = new \OAuth2\Response();

        // validate the authorize request
        if (!$this->server->validateAuthorizeRequest($request, $response)) {

            $response->send();
            die;
        }
        // display an authorization form
        if (empty($_POST)) {

          exit('<form method="post">
              <label>Do You Authorize TestClient?</label><br />
              <input type="submit" name="authorized" value="yes">
              <input type="submit" name="authorized" value="no">
            </form>');
        }

        // print the authorization code if the user has authorized your client
        $is_authorized = ($_POST['authorized'] === 'yes');

        $this->server->handleAuthorizeRequest($request, $response, $is_authorized, $this->userid);

        if ($is_authorized) {
          // this is only here so that you get to see your code in the cURL request. Otherwise, we'd redirect back to the client
          $code = substr($response->getHttpHeader('Location'), strpos($response->getHttpHeader('Location'), 'code=')+5, 40);
          exit("SUCCESS! Authorization Code: $code");
        }

        $response->send();
    }


    public function resource()
    {
        if (!$this->server->verifyResourceRequest(\OAuth2\Request::createFromGlobals())) {

            $this->server->getResponse()->send();
            die;
        }

        $token = $this->server->getAccessTokenData(\OAuth2\Request::createFromGlobals());

        echo json_encode(array(
            "success" => true,
            "error_description" => "You accessed my APIs!, User ID associated with this token is {$token['user_id']}"
        ));
    }


    protected function setAccessLifetime($seconds)
    {
        $this->config = [
            'access_lifetime' => $seconds,
            'id_lifetime' => $seconds,
            'use_jwt_access_tokens' => true,
            'auth_code_lifetime' => 86400
            //'auth_code_lifetime' => 300
        ];
    }

    public function logout(Request $request){
        return response()->json([
            'status'  => 'success',
            'message' => 'Logout successfully.'
        ]);
    }

    // verify authentication token and then otp verify & return response
    private function handleRefreshTokenWithOtp() 
    {
        $token_obj = $this->server->handleTokenRequest(\OAuth2\Request::createFromGlobals());
        $tokenData = $token_obj->getParameters();

        echo "<pre>";
        print_r(request()->all());
        print_r($tokenData);
        die;
        

        if(isset($tokenData['access_token'])) 
        {
            list($header, $payload, $signature) = explode('.', $tokenData['access_token']);

            $payloadAsString = base64_decode($payload);
            $payloadAsObject = json_decode($payloadAsString);
            
            $otp = request()->otp;
            $oauth = Oauth::select('user_id','role_fk','user_fk')->where('user_id',$payloadAsObject->sub)->first();
            if(true) 
            {
                if($oauth->role_fk == 1) {
                    $whr = ['admin_id' => $oauth->role_fk];
                } 
                elseif($oauth->role_fk == 2) {
                    $whr = ['provider_id' => $oauth->role_fk];
                } else {
                    $whr = ['user_id' => $oauth->role_fk];
                }

                $where = ['otp' => $otp, 'tries' => '0', 'phone_email' => request()->phone_email];
                $whrArr = array_merge($whr, $where);
                pre($whrArr);
                die;

                $otp_check = Otp::where($whrArr);
                
               
                return $this->prepareTokenResponse($tokenData, $oauth);   
            }
        }

        print_r([
            'error'  => 'invalid_grant',
            'error_description' => "Invalid token."
        ]);
    }

    private function isProviderApproved($obj)
    {            
        $provider = Provider::select('application_status')->where('id', $obj->user_fk)->first();

        if(isset($provider->application_status) && $provider->application_status == "Pending") {
            
            return true;
        }
        
    }

}
