<?php

namespace App\Http\Controllers\v3\CMS;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Models\CMS\Banner;

class BannerController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */

    public function index(Request $request) {
        has_scope('cms.banners.read');
        try {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;

            if(isset($request->order) && strtolower($request->order) == "asc") {   
                $order  = $request->order;  
            }

            if($request->language == 'en') {

                $result = Banner::select('id','title_en as title','image','type','start_date','end_date','created_at','updated_at'); 
            }else if($request->language == 'es') {

                $result = Banner::select('id','title_es as title','image','type','start_date','end_date','created_at','updated_at'); 
            }else{

                $result = Banner::select('*');
            }

            $date =  date("Y-m-d H:i:s");

            $result = $result->where(function ($query) use($date) {
                            $query->where('end_date', '>=',$date)
                                ->orWhere('end_date', '=', NULL);
                        })->orderBy('id',$order)->paginate($limit);
            
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.BANNER_LIST'), 'records' => $result]);
        } catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }    
    }
    
    public function store(Request $request) {
        has_scope('cms.banners.write');
        //dd($request->all());

        $service = array(
            'image.uploaded' => config('service.USER_API.MESSAGE.BANNER_IMAGE'),
            'image.max' => config('service.USER_API.MESSAGE.BANNER_IMAGE'),
        );

        $this->validate($request, [
            'image'    => 'required|mimes:jpeg,png,jpg,svg,gif|max:1024',
            'title_en' => 'required|string|unique:banners',
            'title_es' => 'required|string|unique:banners',
            'type'     => 'required|in:Web Banner,App Banner,Web App landing page',
        ], $service);

        try {
             //image upload
            $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();          
            request()->image->move(storage_path('banner'),$imageName);
                    

            $banner = new Banner;
            $banner->title_en = $request->title_en;
            $banner->title_es = $request->title_es;
            $banner->start_date = $request->start_date ? date('Y-m-d H:i:s', strtotime($request->start_date)) : NULL;
            $banner->end_date   = $request->end_date   ? date('Y-m-d H:i:s', strtotime($request->end_date))   : NULL;
            $banner->type       = $request->type;
            $banner->image      = $imageName;
            $banner->save();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.BANNER_ADDED'), 'records' => $banner]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }      
    }

    public function show(Request $request,$id) {
        has_scope('cms.banners.read');

        $banner = Banner::findOrFail($id);

        try {

            if($request->language == 'en') {
                $result = Banner::select('id','title_en as title','image','type','start_date','end_date','created_at','updated_at'); 
            }else if($request->language == 'es') {
                $result = Banner::select('id','title_es as title','image','type','start_date','end_date','created_at','updated_at'); 
            }else{
                $result = Banner::where('id',$id);
            }

            $result = $result->first(); 
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.SINGLE_BANNER'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }    
    }

    public function update(Request $request, $id) {
        has_scope('cms.banners.read');

        $banner = Banner::findOrFail($id);

        $service = array(
            'image:uploaded' => config('service.USER_API.MESSAGE.CATEGORY_IMAGE'),
        );

       
        $this->validate($request, [
            'title_en' => 'required|string|unique:banners,title_en,'.$id,
            'title_es' => 'required|string|unique:banners,title_es,'.$id,
            'type'     => 'required|in:Web Banner,App Banner,Web App landing page',
            'image'    => 'image|mimes:jpeg,png,jpg,gif,svg|max:1024',
        ],  $service);
    
        try {
            $data = array();

            if ($request->has('image')) {
                //image upload
                $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();          
                request()->image->move(storage_path('banner'),$imageName);
                $data['image'] = $imageName; 
            }

            $data['title_en'] = $request->title_en;
            $data['title_es'] = $request->title_es;
            $data['type']     = $request->type;
            $data['start_date']   = $request->start_date ? date('Y-m-d H:i:s', strtotime($request->start_date)) : NULL;
            $data['end_date']     = $request->end_date   ? date('Y-m-d H:i:s', strtotime($request->end_date))   : NULL;

            $banner = Banner::where('id',$id)->update($data);
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.BANNER_UPDATED'), 'records' => $banner]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }    
    }

    public function delete($id) {
        has_scope('cms.banners.delete');

        $banner = Banner::findOrFail($id);

        try {
            $banner = $banner->delete();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.BANNER_DELETED'), 'records' => $banner]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
}
