<?php

namespace App\Http\Controllers\v3\CMS;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use Intervention\Image\ImageManagerStatic as Image;

use App\Models\CMS\Country;

class CountryController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */

    public function index(Request $request) {
        try
        {
             $order = "asc";
             $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
         
             $obj = Country::whereNotNull('id');
 
             if(isset($request->order) && strtolower($request->order) == "desc") {
               
                $order  = $request->order; 
             }
 
             $result = $obj->orderBy('phonecode', $order)->groupBy('phonecode')->paginate($limit);
 
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function count() {
        try {
            
            $country = Country::count();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => "Total number of Countries.", 'count' => $country]);
        } 
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } 
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function search(Request $request) {
        try
        {
             $order = "desc";
             $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
         
             $obj = Country::whereNotNull('id');
 
             if(isset($request->order) && strtolower($request->order) == "asc") {
               
                $order  = $request->order; 
             }

             if(isset($request->keyword) && $request->keyword!='') {
                $obj = $obj->where('country_name', 'like', '%' . $request->keyword . '%');
             }
 
             $result = $obj->orderBy('id', $order)->paginate($limit);
 
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function store(Request  $request) {
        has_scope('cms.country.write');

        $this->validate($request, [
            'country_code' => 'required',
            'country_name' => 'required|numeric',
            'phonecode' => 'required|numeric',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048'
        ]);

        $imageName = rand(10000, 9999999) . '-' . time() . '.' . $picture->getClientOriginalExtension();
        $image = Image::make($picture);
                
        //thumb 
        Image::make($image)->resize(150,150)->save(storage_path('product/thumb/').$imageName);
    }

}
