<?php

namespace App\Http\Controllers\v3\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use App\Models\Contractors\Contractor;
use App\Models\Contractors\ContractorCategory;
use App\Models\Contractors\ContractorService;
use App\Models\Contractors\Oauth;
use App\Models\Contractors\Service;

class ContractorServicesController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected $contractor;
    protected $contractor_services;
    protected $oauth_user;
    protected $services;
    protected $user_id;
    protected $role_fk;
    protected $service_image;

    public function __construct(Contractor $contractor, ContractorService $contractor_services, Oauth $oauth_user, Service $services, Request $request) {
        $this->contractor = $contractor;
        $this->oauth_user = $oauth_user;
        $this->contractor_services = $contractor_services;
        $this->services = $services;

        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->service_image = str_replace('SIS941-API', 'SIS941', url('uploads/image'));
    }

    /**
     * @ Contractor's services list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request, $id) {
        try
        {
            // echo $id; echo "<pre>"; print_r($request->all()); die;
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if($limit > 500) {
                $limit = 25;
            }
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            // echo"<pre>"; print_r($this->getColumnName()); die;
            // echo $this->user_id; die;
            $id = $id ? $id : $this->user_id;
            $services = $this->contractor_services::select(['contractor_service.*', 'services.service_name', 'services.image'])
                ->join('services', 'services.id', '=', 'contractor_service.service_id')
                ->where([
                    ['contractor_service.contractor_id', '=', $id],
                    ['contractor_service.status', '=', 'active'],
                ])
                ->where(function($q) use($request) {
                    // Global searching...
                    if(isset($request->search) && !empty($request->search) && $request->search!="") {
                        if(!empty($this->getColumnName())) {
                            foreach ($this->getColumnName() as $key => $col) {
                                if($key==0) {
                                    $q->where($col, 'like', '%' .$request->search. '%');
                                } else {
                                    $q->orWhere($col, 'like', '%' .$request->search. '%');
                                }
                            }
                        }
                    }
                })
                ->where(function($q) use($request) {
                    // Column wise searching filter ...
                    $allowableColumn = $this->getColumnName();
                    if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                        $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);
            if( empty($services) ) {
                throw new Exception(config('service.USER_API.MESSAGE.SERVICE_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.SERVICE_LIST'), 'records' => $services, 'service_image_path'=>$this->service_image]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Contractor List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($contractor_id, $service_id) {
        try {
            $con_services = $this->contractor_services::select(['contractor_service.*', 'services.service_name', 'services.image'])
            ->join('services', 'services.id', '=', 'contractor_service.service_id')
            ->with(['contractor_service_category'=> function($query) use($service_id) {
                $query->select(['contractor_sub_category.*', 'service-category.sub_category_name'])->join('service-category', 'service-category.id', 'contractor_sub_category.service_cat_id')->where('contractor_service_id', '=', $service_id);
            }])
            ->where([
                ['contractor_service.id', '=', $service_id],
                ['contractor_service.contractor_id', '=', $contractor_id],
                ['contractor_service.status', '=', 'active']
            ])->get();
            if( empty($con_services) ) {
                throw new Exception(config('service.CONTRACTOR_API.MESSAGE.SERVICE_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.SERVICE_INFO'), 'records' => $con_services, 'service_image_path'=>$this->service_image]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete($contractor_id, $service_id)
    {
        die('need to work on it');
        try {
            $upd['status'] = 'inactive';
            $contractor = $this->contractor_services::where([
                ['contractor_id', '=', $contractor_id],
                ['status', '=', 'active']
            ])->update($upd);

            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CONTRACTOR_API.MESSAGE.SERVICE_DELETED'), 'records' => $contractor]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CONTRACTOR_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $services_col = [];
        $services = $this->services::first();
        if(!empty($services) && $services!="") {
            foreach (array_keys($services->toArray()) as $key => $value) {
               $services_col[] = 'services.'.$value;
            }
            return $services_col;
        }

        return ['services.id'];
    }

    private function hashPassword($password) {
        return sha1($password);
    }
}
