<?php

namespace App\Http\Controllers\v3\TwoFactor;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Controller;
use App\Models\Auth\Oauth;
use App\Jobs\SendTwoFactorJob;
use App\Models\Users\User as Admin;

class TwoFactorController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Renderable
     */
    /*public function __construct()
    {
        $this->middleware('auth:admin'); 
 
    }*/

    public function index()
    {
       
    }

    /**
     * Show the form for creating a new resource.
     * @return Renderable
     */
    public function create()
    {
        
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Renderable
     */
    public function store(Request $request)
    {
        $this->validate($request, [
           'two_factor_code' => 'integer|required',
        ]);

        try {
            $data = $request->all();
            //$adminUser = Oauth::where('user_fk',$data['id'])->first();
            $adminUser = Admin::find($data['id']);

            $validityTime = strtotime(date("Y-m-d H:i:s"));
            if(($validityTime - $adminUser->two_factor_expires_at) > 300)
            {
                $adminUser->two_factor_code = null;
                $adminUser->two_factor_expires_at = null;
                $adminUser->save();
                //Oauth::where('user_fk',$data['id'])->update(['two_factor_code'=>null,'two_factor_expires_at'=>null]);
    
                return response()->json(['status' => 'expire', 'message' => 'The two factor code has expired. Please login again.']);

            }

            if($data['two_factor_code'] == $adminUser->two_factor_code)
            {
                $adminUser->two_factor_code = null;
                $adminUser->two_factor_expires_at = null;
                $adminUser->save();
                //Oauth::where('user_fk',$data['id'])->update(['two_factor_code'=>null,'two_factor_expires_at'=>null]);

                return response()->json(['status' => 'success', 'message' => 'Welcome! You are successfully logged in as Admin']);
            } 

            return response()->json(['status' => 'not match', 'message' => 'The two factor code you have entered does not match']);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function resend(Request $request)
    {
        $data = $request->all();
        //$adminUser = Oauth::where('user_fk',$data['id'])->first();
        $adminUser = Admin::find($data['id']);
        if($adminUser)
        {
            $email = $adminUser->email;
            $name = $adminUser->first_name." ".$adminUser->last_name;
            /*$fData = [
                    'two_factor_code' => $data['two_factor_code'],
                    'two_factor_expires_at' => strtotime(date("Y-m-d H:i:s"))
                ];
             */
            $adminUser->two_factor_code = $data['two_factor_code'];   
            $adminUser->two_factor_expires_at = strtotime(date("Y-m-d H:i:s"));
            $adminUser->save();
            //$updateCode = Oauth::where('email', $email)->update($fData);

            $sendMail =  new SendTwoFactorJob($name,$email,$data['two_factor_code']); 
            dispatch($sendMail);
               
            return response()->json(['status' => 'success', 'message' => 'The two factor code has been sent again']);    
        }
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function show($id)
    {
        return view('twofactor::show');
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Renderable
     */
    public function edit($id)
    {
        return view('twofactor::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Renderable
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Renderable
     */
    public function destroy($id)
    {
        //
    }
}
