<?php
namespace App\Http\Controllers\v3\Users;
use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Notifications\CreateNewUserNotification;
use App\Models\Users\User;
use App\Models\Users\Oauth;
use App\Models\Users\Jobs;
use App\Models\Roles\Role;

class RoleController extends Controller {
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected $users;
    protected $oauth_user;

    public function __construct(User $users, Oauth $oauth_user)
    {
        $this->users = $users;
        $this->oauth_user = $oauth_user;
        //has_scope('roles.read');
    }

    /**
    * @ user list List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function index(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $roles = Role::with(['countUser'])
            ->where(function ($q) use ($request) {
            // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    if (!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if ($key == 0) {
                                $q->where($col, 'like', '%' . $request->search . '%');
                            } else {
                                $q->orWhere($col, 'like', '%' . $request->search . '%');
                            }
                        }
                    }
                }
            // filters
                if($request->filled('designation')) {
                    $q->where('name', 'like', '%'.$request->designation.'%');
                }
                if($request->filled('date_from') && $request->filled('date_to')) {
                    $date_from = date("Y-m-d", strtotime($request->date_from));
                    $date_to = date("Y-m-d", strtotime($request->date_to));
                    $q->whereBetween('created_at', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                }
            })
            ->where(function ($q) use ($request) {
            // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles List', 'records' => $roles]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function allRoles(Request $request)
    {
        try {
            $roles = Role::orderBy('id', 'desc')->get();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles List', 'records' => $roles]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
    * @ Register user api.
    * @return \Illuminate\Http\Response
    */
    public function store(Request $req)
    {
    //$this->validateInputs($req);
        $this->validate($req, [
            'name' => 'required|max:50|unique:roles',
            'scopes' => 'required|array',
        ]);
        try {
            $input = $req->all();
            $scopes = !empty($input['scopes']) ? implode(' ', $input['scopes']) : null;
            $role = new Role();
            $role->name = $input['name'];
            $role->status = $input['status'];
            $role->scope = $scopes;
            $role->created_at = convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
            $role->save();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Role has been created successfully!', 'result' => $role]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }

    /**
    * @ Service List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function show($id)
    {
        try {
            $users = Role::where('id', $id)->first();
            $users->scope = isset($users->scope) ? explode(' ', $users->scope) : [];
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles Details', 'records' => $users]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function validateInputs(Request $request, $id = null)
    {
        $messages = [];
        $rules = [
            'name' => 'required|max:50|unique:roles',
    //'description' => 'required|max:255',
            'status' => 'required',
        ];
        if ($id > 0) {
            $rules['name'] = 'required|unique:roles,name,' . $id;
        }
        return $this->validate($request, $rules, $messages);
    }
    /**
    * @ update user api .
    * @return \Illuminate\Http\Response
    */
    public function update(Request $req, $id)
    {
        $this->validate($req, [
            'name' => 'required|max:50|unique:roles,name,'.$id,
            'scopes' => 'required|array',
        ]);

        try {
            //$this->validateInputs($req, $id);
            DB::beginTransaction();
            $input = $req->all();
            $scopes = !empty($input['scopes']) ? implode(' ', $input['scopes']) : null;
            $role = Role::find($id);
            $role->name = $input['name'];
            $role->scope = $scopes;
            $role->save();

            if ($role) {
                $data = ['scope'=>$scopes];
                $result_user = Oauth::where('role_fk', $id)->update($data);
            }
            DB::commit();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles updated successfully!', 'result' => $role]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    /**
    * Remove the specified resource from storage.
    *
    * @param  int  $id
    * @return \Illuminate\Http\Response
    */
    public function delete(Request $request, $id) 
    {


        try {
            $result = Role::where('id', $id)->delete();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles Details', 'result' => $result]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function getUsers($role_id)
    {
        try {
            $users = DB::table('oauth_users')
            ->select('oauth_users.*')
            ->join('roles', 'roles.id', '=', 'oauth_users.role_fk')
            ->where('roles.id', $role_id)
            ->get();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles Details', 'records' => $users]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function getCount()
    {
        try {
    //$users = $this->users::count();
            $users = DB::table('roles')->count();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Roles Count', 'count' => $users]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'count' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'count' => []]);
        }
    }
    public function getScopes($role_id) 
    {

        try {
            $result = DB::table('roles')->select(array('id', 'scope'))->where('id', $role_id)->first();

            if (trim($result->scope) != '') {

                $result->scope = isset($result->scope) ? explode(' ', $result->scope) : [];
            }
            else {

                $result->scope = array();
            }

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Scopes List', 'result' => $result]);
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function setScopes(Request $req, $role_id) 
    {

        DB::beginTransaction();

        try {
            $input_data = $req->input('scopes');
            $save_data['scope'] = (isset($input_data) && count($input_data)) ? implode(' ', $input_data) : null;

            $result = Role::where('id', $role_id)->update($save_data);

            if ($result) {
                $result_user = Oauth::where('role_fk', $role_id)->update($save_data);
                DB::commit();
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => 'Scopes List', 'result' => $result]);
        }
        catch (Exception $e) {

            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {

            DB::rollBack();
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function activeInactive(Request $request, $id) 
    {

        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['status'] = $request->status;
            $result = Role::where('id', $id)->update($save_data);

            if($request->status=="active")
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ROLE_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ROLE_STATUS_INACTIVE'),'action'=>$request->status]);
            }
        }
        catch (Exception $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {

            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }
    private function getColumnName()
    {
        $address = Role::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }
}