<?php

namespace App\Http\Controllers\v4\AppointmentManagement;
use Exception;
use Throwable;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use App\Models\AppointmentManagement\Appointment;
use App\Models\Customers\AppointmentCancelled;
use App\Models\AppointmentManagement\NoShow;
use App\Models\Users\User;
use Illuminate\Support\Facades\App;

class AppointmentManagementController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected $user_id;
    protected $role_fk;
    protected $appointments;
    protected $appointmentCancelled;
    protected $noShow;
    public function __construct(Appointment $appointments, AppointmentCancelled $appointmentCancelled, NoShow $noShow,Request $request)
    {
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        $this->appointments = $appointments;
        $this->appointmentCancelled = $appointmentCancelled;
        $this->noShow = $noShow;
    }

    /**
    * @ user list List api using pagination.
    * @return \Illuminate\Http\Response
    */

    public function upcomingAppointments(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $appointmentIds = AppointmentCancelled::pluck('appointment_id')->toArray();
            $query = $this->appointments
                    ->where(function ($qry) use ($request) {
                            $qry->whereHas('customer', function ($qry) use ($request) {
                                if($request->filled('search')) {
                                    $pieces = explode(" ", $request->search);
                                    if(count($pieces)){
                                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                    }
                                    $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                                }
                            })->orwhereHas('provider', function ($qry) use ($request) {
                                if($request->filled('search')) {
                                    $pieces = explode(" ", $request->search);
                                    if(count($pieces)){
                                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                    }
                                    $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                    $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                                }
                            });
                        })
                        ->whereHas("provider", function($q) use ($request) {
                            if($request->filled('provider_name')) {
                                $pieces = explode(" ", $request->provider_name);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->provider_name; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->provider_name; 
                                }
                                $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                                $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                                $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        })
                        ->whereHas("customer", function($q) use ($request) {
                            if($request->filled('consumer_name')) {
                                $pieces = explode(" ", $request->consumer_name);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->consumer_name; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->consumer_name; 
                                }
                                $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                                $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                                $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        })

                        ->with(['customer','provider'])
                        ->where(function ($q) use ($request) {
                            if($request->filled('date_from') && $request->filled('date_to')) {
                                $date_from = date("Y-m-d", strtotime($request->date_from));
                                $date_to   = date("Y-m-d", strtotime($request->date_to));
                                $q->whereBetween('date', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                            }

                        })
                        ->where('date_time', '>=', strtotime(date("Y-m-d H:i")))
                        ->whereNotIn('id', $appointmentIds)
                        ->orderBy($order_by_column_name, $order)
                        ->paginate($limit);
            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.UPCOMING_APPOINTMENT_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.UPCOMING_APPOINTMENT_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function completedAppointments(Request $request)
    {
       
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $appointmentIds = AppointmentCancelled::pluck('appointment_id')->toArray();
            $noShowAppointments = $this->noShow::whereNull('customer_id')->orwhereNull('provider_id')->pluck('appointment_id')->toArray();

            $query = $this->appointments
                    ->where(function ($qry) use ($request) {
                        $qry->whereHas('customer', function ($qry) use ($request) {
                            if($request->filled('search')) {
                                $pieces = explode(" ", $request->search);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                }
                                $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        })->orwhereHas('provider', function ($qry) use ($request) {
                            if($request->filled('search')) {
                                $pieces = explode(" ", $request->search);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                }
                                $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        });
                    })
                    ->whereHas("provider", function($q) use ($request) {
                        if($request->filled('provider_name')) {
                            $pieces = explode(" ", $request->provider_name);
                            if(count($pieces)){
                                $fName = !empty($pieces[0]) ? $pieces[0] : $request->provider_name; 
                                $lName = !empty($pieces[1]) ? $pieces[1] : $request->provider_name; 
                            }
                            $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                            $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                            $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                        }
                    })
                    ->whereHas("customer", function($q) use ($request) {
                        if($request->filled('consumer_name')) {
                            $pieces = explode(" ", $request->consumer_name);
                            if(count($pieces)){
                                $fName = !empty($pieces[0]) ? $pieces[0] : $request->consumer_name; 
                                $lName = !empty($pieces[1]) ? $pieces[1] : $request->consumer_name; 
                            }
                            $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                            $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                            $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                        }
                    })
                   
                 ->with(['customer','provider'])
                 ->where(function ($q) use ($request) {
                    if($request->filled('date_from') && $request->filled('date_to')) {
                        $date_from = date("Y-m-d", strtotime($request->date_from));
                        $date_to   = date("Y-m-d", strtotime($request->date_to));
                        $q->whereBetween('date', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                    }

                    })
                 ->where('date_time', '<=', strtotime(date("Y-m-d H:i")))
                 ->whereNotIn('id', $appointmentIds)
                 ->whereNotIn('id', $noShowAppointments)
                 ->orderBy($order_by_column_name, $order)
                 ->paginate($limit);
            
            if(empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.COMPLETED_APPOINTMENT_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.COMPLETED_APPOINTMENT_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function cancelledAppointments(Request $request)
    {
        try{
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;

            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getAppointmentCancelledColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $query = $this->appointmentCancelled
                     ->where(function ($qry) use ($request) {
                         $qry->whereHas('customer', function ($qry) use ($request) {
                             if($request->filled('search')) {
                                 $pieces = explode(" ", $request->search);
                                 if(count($pieces)){
                                     $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                     $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                 }
                                 $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                             }
                         })->orwhereHas('provider', function ($qry) use ($request) {
                             if($request->filled('search')) {
                                 $pieces = explode(" ", $request->search);
                                 if(count($pieces)){
                                     $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                     $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                 }
                                 $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                             }
                         });
                     })
                     ->whereHas("provider", function($q) use ($request) {
                         if($request->filled('provider_name')) {
                             $pieces = explode(" ", $request->provider_name);
                             if(count($pieces)){
                                 $fName = !empty($pieces[0]) ? $pieces[0] : $request->provider_name; 
                                 $lName = !empty($pieces[1]) ? $pieces[1] : $request->provider_name; 
                             }
                             $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                             $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->provider_name)))."%");
                             $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                         }
                     })
                     ->whereHas("customer", function($q) use ($request) {
                         if($request->filled('consumer_name')) {
                             $pieces = explode(" ", $request->consumer_name);
                             if(count($pieces)){
                                 $fName = !empty($pieces[0]) ? $pieces[0] : $request->consumer_name; 
                                 $lName = !empty($pieces[1]) ? $pieces[1] : $request->consumer_name; 
                             }
                             $q->where("first_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                             $q->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->consumer_name)))."%");
                             $q->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                         }
                     })
                     ->with(['customer','provider','appointment'])
                     ->where(function ($q) use ($request) {
                        if($request->filled('date_from') && $request->filled('date_to')) {
                            $date_from = date("Y-m-d", strtotime($request->date_from));
                            $date_to   = date("Y-m-d", strtotime($request->date_to));
                            $q->whereBetween('date', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                        }

                    })
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);
            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.CANCELLED_APPOINTMENT_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.CANCELLED_APPOINTMENT_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function cancelledAppointmentsShow(Request $request,$id)
    {
        try
        {
            $appointmentCancelled =  AppointmentCancelled::find($id);
            if(empty($appointmentCancelled)) {
                throw new Exception(config('service.USER_API.MESSAGE.CANCELLED_APPOINTMENT_NOT_FOUND'));
            }
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CANCELLED_APPOINTMENT_DETAILS'), 'records' => $appointmentCancelled]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function deleteUpcomingAppointment(Request $request, $id) {
        //has_scope('roles.delete');
        $appointment = Appointment::findOrFail($id);
        try {
            $appointment  = $appointment->delete();
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.APPOINTMENT_DELETED'), 'result' => $appointment]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }
    public function latestNoShow(Request $request)
    {
        try {
            // $pag_no = $request->has('page') ? $request->get('page') : 1;
            // $per_page = 2;
            // $skip = ($pag_no - 1) * $per_page ;
            $query = NoShow::with(['customer','provider','appointment.customer','appointment.provider'])
            ->whereNull('customer_id')
            ->orwhereNull('provider_id')
            ->orderBy('id', 'DESC')
            //->skip($skip)
            //->take($per_page)
            ->take(10)
            ->get();

            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.NO_SHOW_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.NO_SHOW_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    public function appointmentsNoShow(Request $request)
    {
        try{
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;

            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getAppointmentNoShowColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $query = $this->noShow
                     ->where(function ($qry) use ($request) {
                         $qry->whereHas('appointment.customer', function ($qry) use ($request) {
                             if($request->filled('search')) {
                                 $pieces = explode(" ", $request->search);
                                 if(count($pieces)){
                                     $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                     $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                 }
                                 $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                             }
                         })->orwhereHas('appointment.provider', function ($qry) use ($request) {
                             if($request->filled('search')) {
                                 $pieces = explode(" ", $request->search);
                                 if(count($pieces)){
                                     $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                     $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                 }
                                 $qry->where("first_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere("last_name","like","%".encryptString(ucfirst(strtolower($request->search)))."%");
                                 $qry->orWhere(DB::raw('CONCAT(first_name," ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName))).' '.encryptString(ucfirst(strtolower($lName))).'%');
                             }
                         });
                     })
                     
                    ->with(['customer','provider','appointment.customer','appointment.provider'])
                    ->whereNull('customer_id')
                    ->orwhereNull('provider_id')
                    ->orderBy($order_by_column_name, $order)
                    ->paginate($limit);
            if (empty($query)) {
                throw new Exception(config('service.USER_API.MESSAGE.NO_SHOW_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.USER_API.MESSAGE.NO_SHOW_LISTING'), 'records' => $query]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
    private function getColumnName()
    {
        $address = Appointment::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }
    private function getAppointmentCancelledColumnName()
    {
        $address = AppointmentCancelled::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }

    private function getAppointmentNoShowColumnName()
    {
        $address = NoShow::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }

    public function providerPaymentPaid($id) 
    {
        try {
            $appointment = Appointment::where(['id' => $id, 'provider_payment_status' => 'Unpaid'])->first();
            if($appointment){
                $appointment->provider_payment_status = "Paid";
                $appointment->save();

                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.PROVIDER_PAYMENT_PAID')]);  
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
}