<?php

namespace App\Http\Controllers\v4\CMS;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Models\CMS\FAQ;

class FAQController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $faqs;
    protected  $user_id;

    public function __construct(FAQ $faqs) {
        $this->faqs = $faqs;

        $this->user_id = 1;
    }

    /**
     * @ cms pages listing api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        has_scope('cms.faq.read');
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            if($request->language == 'en') {
                $obj = $this->faqs::select('id', 'faqs_type','question_en as question', 'answer_en as awnser', 'created_at','updated_at');
            } else if ($request->language == 'es') {
                $obj = $this->faqs::select('id', 'faqs_type','question_es as question', 'answer_es as awnser', 'created_at','updated_at');
            } else {
                $obj = $this->faqs::select('*');
            }

            $faqs =  $obj->where(function($q) use($request) {
                    // Global searching...
                    if(isset($request->search) && !empty($request->search) && $request->search!="") {
                        if(!empty($this->getColumnName())) {
                            foreach ($this->getColumnName() as $key => $col) {
                                if($key==0) {
                                    $q->where($col, 'like', '%' .$request->search. '%');
                                } else {
                                    $q->orWhere($col, 'like', '%' .$request->search. '%');
                                }
                            }
                        }
                    }
                })
                ->where(function($q) use($request) {
                    // Column wise searching filter ...
                    $allowableColumn = $this->getColumnName();
                    if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                        $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_LISTING'), 'records' => $faqs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ Cms view details
     * @return \Illuminate\Http\Response
    */
    public function show(Request $request, $id) {
        has_scope('cms.faq.read');
        try
        {   
            if($request->language == 'en') {
                $obj = $this->faqs::select('id', 'faqs_type','question_en as question', 'answer_en as awnser', 'created_at','updated_at');
            } else if ($request->language == 'es') {
                $obj = $this->faqs::select('id', 'faqs_type','question_es as question', 'answer_es as awnser', 'created_at','updated_at');
            } else {
                $obj = $this->faqs::select('*');
            }

            $faqs = $obj->where('id',$id)->first();
            if( empty($faqs) ) {
                throw new Exception(config('service.USER_API.MESSAGE.FAQ_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_DETAILS'), 'records' => $faqs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col = $this->faqs::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }

        return ['id'];
    }

    public function store(Request $request) {
        has_scope('cms.faq.write');
    
        $this->validate($request,[
         'faqs_type' => 'required|integer',
         'question_en'  => 'required|string',
         'answer_en'  => 'required|string',
         'question_es' => 'required|string',
         'answer_es' => 'required|string',
        ]);

        try {
            $input = $request->all();
            $data  =  new FAQ;
            $save  =  $data->create($input);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_SUCCESS'), 'records' => $save]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function update(Request $request, $id) {
        has_scope('cms.faq.write');

        $faq = FAQ::findOrFail($id);
        if(!$faq) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.USER_API.MESSAGE.NOT_FOUND')]);
        }
    
        $this->validate($request,[
         'faqs_type' => 'required|integer',
         'question_en'  => 'required|string',
         'answer_en'  => 'required|string',
         'question_es' => 'required|string',
         'answer_es' => 'required|string',
        ]);

        try {
            $input = $request->all();
            $save  = $faq->update($input);

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_UPDATED'), 'records' => $save]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function delete(Request $request, $id) {
        has_scope('cms.faq.delete');

        $faq = FAQ::findOrFail($id);
        if(!$faq) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.USER_API.MESSAGE.NOT_FOUND')]);
        }

        try {
            $save  = $faq->delete();

            return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAQ_DELETD'), 'records' => $save]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    
    }

}
