<?php

namespace App\Http\Controllers\v4\Contractors;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Contractors\Booking;
use App\Models\Contractors\Contractor;
use Illuminate\Support\Facades\DB;

use App\Models\Contractors\Jobs;
use App\Models\Contractors\Oauth;
use App\Notifications\JobsNotification;

class JobsController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $jobs;
    protected  $user_id;
    protected  $role_fk;
    protected  $bookings;
    protected  $oauth_user;
    protected  $contractor;

    public function __construct(Jobs $jobs, Request $request, Booking $bookings, Oauth $oauth_user, Contractor $contractor) {
        $this->jobs = $jobs;
        $this->bookings = $bookings;
        $this->user_id = $request->user_id;
        $this->oauth_user = $oauth_user;
        $this->role_fk = $request->role_fk;
        $this->contractor = $contractor;
    }

    /**
     * @ contact details api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try
        {
            $jobs = $this->bookings::with(['user.address' => function ($query) {
                $query->where('user_type', '=', 1);
            }, 'contractor.address' => function ($query) {
                $query->where('user_type', '=', 2);
            }, 'service.category', 'jobs'])
            ->where([
                ['id', '=', $id],
                ['user_id', '=', $this->user_id],
            ])->get();

            if( empty($jobs) ) {
                throw new Exception(config('service.USER_API.MESSAGE.JOBS_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.JOBS_DETAILS'), 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function updateJobStatus(Request $request, $id) {
        $this->validate($request, [
            'job_status' => 'required|in:1,2,3,4,5',
            'specific_reason' => $request->job_status == 5 ? 'required|max:256' : "",
        ]);

        try {
            $input = $request->all();
            $input['status'] = $request->job_status;
            $input['reason'] = $request->specific_reason;
            $jobs = $this->jobs::find($id);
            $jobs->update($input);
            $booking_id = $jobs->booking_id;

            // // also update booking table status
            $this->bookings::find($booking_id)->update($input);

            // database notification
            if($request->job_status==1) {
                $msg = "Jobs has been accepted";
                $body = "Jobs has been accepted";
            }
            if($request->job_status==2) {
                $msg = "Jobs on the way";
                $body = "Jobs on the way";
            }
            if($request->job_status==3) {
                $msg = "Jobs is in the progress";
                $body = "Jobs is in the progress";
            }
            if($request->job_status==4) {
                $msg = "Jobs has been completed";
                $body = "Jobs has been completed";
            }
            if($request->job_status==5) {
                $body =  $request->specific_reason;
                $msg = "Jobs has been decline";
            }
            $user = $this->contractor::find($this->user_id);
            $details = [
                'greeting' => 'Hi '. $user->first_name .'!',
                'body'=> @$body,
                'database_notification_message' => $msg,
                'thanks' => 'Welcome to ZION!!',
            ];
            $role = 'contractor';
            $user->notify(new JobsNotification($role, $details));
            $upd['user_type'] = $role;
            $notif_id = $user->notifications[0]->id;
            $user->notifications()->find($notif_id)->update($upd);


            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.JOBS_DECLINE_SUCCESS'), 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function startEndJob(Request $request, $id) {
        $this->validate($request, [
            'job_started_datetime' => isset($request->job_started_datetime) && $request->job_started_datetime ? 'required' : "",
            'job_end_datetime' => isset($request->job_end_datetime) && $request->job_end_datetime ? 'required' : "",
        ]);

        // echo "<pre>"; print_r($request->all()); die;

        try {
            $msg = '';
            $input = $request->all();
            if(isset($request->job_started_datetime) && $request->job_started_datetime){
                $input['job_started'] = date('Y-m-d H:i:s', strtotime($request->job_started_datetime));
                $msg = config('service.USER_API.MESSAGE.JOBS_START');
            }
            if(isset($request->job_end_datetime) && $request->job_end_datetime){
                $input['job_end'] = date('Y-m-d H:i:s', strtotime($request->job_end_datetime));
                $msg = config('service.USER_API.MESSAGE.JOBS_END');
            }
            $jobs = $this->jobs::find($id);
            $jobs->update($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $msg, 'records' => $jobs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col_name = [];
        $col = $this->bookings::first();
        if(!empty($col) && $col!="") {
            // return array_keys($col->toArray());
            foreach (array_keys($col->toArray()) as $key => $value) {
                $col_name[] = 'booking.'.$value;
            }
            $extraCol = ['service-category.sub_category_name'];
            return array_merge($col_name, $extraCol);
        }

        return ['id'];
    }
}
