<?php
namespace App\Http\Controllers\v4\DidYouKnow;
use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use App\Models\DidYouKnow\DidYouKnow;
use App\Jobs\SendMailJob;

class DidYouKnowController extends Controller
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected  $didYouKnow;
    protected  $user_id;
    public function __construct(DidYouKnow $didYouKnow, Request $request) 
    {
        $this->didYouKnow = $didYouKnow;
        $this->user_id = $request->user_id;
    }
    /**
    * @ cms pages listing api using pagination.
    * @return \Illuminate\Http\Response
    */


    public function index(Request $request) 
    {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if($request->language == 'es') {
                app()->setLocale('es');
                $didYouKnow = $this->didYouKnow::where(function ($q) use ($request) {
                    // Global searching...
                    if (isset($request->search) && !empty($request->search) && $request->search != "") {
                        $q->whereTranslationLike("title","%".$request->search."%");         
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate($limit);
            }
            $didYouKnow = $this->didYouKnow::where(function ($q) use ($request) {
                // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    $q->whereTranslationLike("title","%".$request->search."%");
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW_LISTING'), 'records' => $didYouKnow]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
    * @ Cms view details
    * @return \Illuminate\Http\Response
    */
    public function show(Request $request,$id) 
    {
        try
        {
            $didYouKnow =  $this->didYouKnow::find($id);
            if( empty($didYouKnow) ) {
                throw new Exception(config('service.USER_API.MESSAGE.DID_YOU_KNOW_NOT_FOUND'));
            }
            $didYouKnowEn = [
                'id' => $didYouKnow->id,
                'title' => $didYouKnow->title
            ];
            app()->setLocale('es');
            $didYouKnowEs =  $this->didYouKnow::find($id);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW_DETAILS'), 'recordEn' => $didYouKnowEn,'recordEs' => $didYouKnowEs]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function getColumnName() 
    {
        $col = $this->cmspage::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }
        return ['id'];
    }


    public function store(Request $request) 
    {
        $this->validate($request, [
            'en_title' => 'required|max:100',
            'es_title' => 'required|max:100',
        ]);
        try {
            $input = $request->all();
            $data = [
                'en' => [
                    'title'  => $input['en_title']
                ],
                'es' => [
                    'title'  => $input['es_title']
                ],
            ];
            $result  = DidYouKnow::create($data);
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW_ADDED'), 'records' => $result]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function update(Request $request, $id) 
    {
        $this->validate($request, [
            'en_title' => 'required|max:100',
            'es_title' => 'required|max:100',
        ]);
        try {
                $didYouKnow = DidYouKnow::findOrFail($id);
                $input = $request->all();
                $data = [
                    'en' => [
                        'title'  => $input['en_title'],
                    ],
                    'es' => [
                        'title'  => $input['es_title'],
                    ],
                ];
                $data  = $didYouKnow->update($data);
                
                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW_UPDATED'), 'records' => $data]);
            } catch (Exception $e) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            } catch(Throwable $e) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }


    public function delete($id) 
    {
        try {
                $page = DidYouKnow::findOrFail($id);
                $data  = $page->delete();
            
                return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.CMS_PAGE_DELETED'), 'records' => $data]);
            } catch (Exception $e) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            } catch(Throwable $e) {
                return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }


    public function activeInactive(Request $request, $id) 
    {
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $save_data['status'] = $request->status;
            $result = DidYouKnow::where('id', $id)->update($save_data);

            if($request->status=="active")
            {
                return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                 return response()->json(['status' => config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW_STATUS_INACTIVE'),'action'=>$request->status]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }


    public function didYouKnow() 
    {
        try
        {
            $didYouKnow =  DidYouKnow::where(['status' => 'active'])->get();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.DID_YOU_KNOW'), 'records' => $didYouKnow]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function publishAction(Request $request, $id) 
    {
        has_scope('cms.pages.write');
        $page = StaticPage::find($id);
        if(!$page) {
            return response()->json(['status' => config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.USER_API.MESSAGE.NOT_FOUND')]);
        }
        $this->validate($request, [
            'action' => 'required|in:publish,unpublish',
        ]);
        try {
            $status  = $request->action == 'publish' ? 1 : 0;
            $message = $request->action == 'publish' ? config('service.USER_API.MESSAGE.CMS_PAGE_PUBLISHED') : config('service.USER_API.MESSAGE.CMS_PAGE_UNPUBLISHED');
            $page->status = $status;
            $data = $page->update();
            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => $message, 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }
}