<?php

namespace App\Http\Controllers\v4\Favourites;

use Exception;
use Throwable;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;

use App\Models\Favourite;

class FavouriteController extends Controller
{
    /**
     * The user repository implementation.
     *
     * @var UserRepository
     */
    protected  $favourites;
    protected  $user_id;
    protected  $role_fk;

    public function __construct(Favourite $favourites, Request $request) {
        $this->favourites = $favourites;

        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
    }

    /**
     * @ booking list list api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function index(Request $request) {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) && $request->limit > 500 ? 25 : $request->limit;
            if(isset($request->order) && strtolower($request->order)=="asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if(isset($request->order_by_column_name)) {
                if(in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $id = $this->user_id;
            $bookings = $this->favourites::with(['contractor' => function($query) {
                $query->with(['rating' => function($query) {
                    $query->select('contractor_id', DB::raw('AVG(rating) as avarage_rating'))->groupBy('contractor_id');
                }])->select(['id', 'first_name', 'last_name', 'email', 'phone', 'exprence', 'image'])->where('status', '=', 1);
            }])
            ->where('user_id', '=', $id)
            ->where(function($q) use($request) {
                // Global searching...
                if(isset($request->search) && !empty($request->search) && $request->search!="") {
                    if(!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if($key==0) {
                                $q->where($col, 'like', '%' .$request->search. '%');
                            } else {
                                $q->orWhere($col, 'like', '%' .$request->search. '%');
                            }
                        }
                    }
                }
            })
            ->where(function($q) use($request) {
                // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                }
            })
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            $contractor_image_path =  url('/assets/images/contractors');

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FABOURITE_LIST'), 'records' => $bookings, 'contractor_image_path'=>$contractor_image_path]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    /**
     * store booking api.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'contractor_id' => 'required|numeric',
        ]);

        try {

            if(strtolower($this->role_fk)=='admin' || strtolower($this->role_fk)=='contractor') {
                throw new Exception("Sorry you have no access to perform this action. Please login as a user for this.");
            }

            $check_exists = $this->favourites::where([
                ['user_id', '=', $this->user_id],
                ['contractor_id', '=', $request->contractor_id],
            ])->exists();
            if($check_exists) {
                throw new Exception(config('service.USER_API.MESSAGE.FAVOURITES_ALREADY_IN_FAV_LIST'), 1);
            }

            $input = $request->all();
            $input['user_id'] = $this->user_id;
            $favourites = $this->favourites::create($input);

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAVOURITES_STORE'), 'records' => $favourites]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * @ fav List api using pagination.
     * @return \Illuminate\Http\Response
    */
    public function show($id) {
        try
        {
            if(strtolower($this->role_fk)=='admin' || strtolower($this->role_fk)=='contractor') {
                throw new Exception("Sorry you have no access to perform this action. Please login as a user for this.");
            }

            $bookings = $this->favourites::with(['user' => function($query) {
                $query->with('address')->where('status', '=', 1);
            }, 'contractor' => function($query) {
                $query->with(['rating' => function($query) {
                    $query->select('contractor_id', DB::raw('AVG(rating) as avarage_rating'))->groupBy('contractor_id');
                }])
                ->select(['id', 'first_name', 'last_name', 'email', 'phone', 'exprence'])
                ->where('status', '=', 1);
            }])
            ->find($id);

            if( empty($bookings) ) {
                throw new Exception(config('service.USER_API.MESSAGE.FAVOURITES_NOT_FOUND'));
            }

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.FAVOURITES_INFO'), 'records' => $bookings]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    /**
     * Remove the specified addres from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function delete($id)
    {
        try {
            if(strtolower($this->role_fk)=='admin' || strtolower($this->role_fk)=='contractor') {
                throw new Exception("Sorry you have no access to perform this action. Please login as a user for this.");
            }

            $fav = $this->favourites::find($id)->delete();

            return response()->json(['status'=> config('service.USER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.USER_API.MESSAGE.ADDRESS_DELETED'), 'records' => $fav]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.USER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function getColumnName() {
        $col = $this->favourites::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }

        return ['id'];
    }
}
