<?php

namespace App\Models\Customers;

use Illuminate\Database\Eloquent\Model;
use App\Models\Providers\Provider;
use Carbon\Carbon;


class ProviderRatingReview extends Model
{
    protected $table = 'provider_rating_reviews';

    protected $fillable = [
        'appointment_id ', 
        'customer_id', 
        'provider_id', 
        'environment_rating',
        'environment_rating_reason',
        'session_rating', 
        'session_rating_reason', 
        'recommend_service_rating',
        'review',
        'published',
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];


    public function customer(){        
        return $this->belongsTo('App\Models\Customers\Customer', 'customer_id','id');
    }

    public function provider() {  
        return $this->belongsTo(Provider::class, 'provider_id');
    }

    public function getCreatedAtAttribute ($value) 
    { 
        $timezone = !empty(request()->admin_time_zone) ? request()->admin_time_zone : "America/New_York";
        $dateTime = Carbon::parse($value)->timezone($timezone);
        return date('m/d/Y h:i A',strtotime($dateTime));
    }

}
