<?php

namespace App\Models\Products;

use Illuminate\Database\Eloquent\Model;
use App\Models\Products\Category;

class Category extends Model
{
    protected $fillable = [ 
        'id',   
        'category_en',
        'category_es',
        'parent_id',
        'icon',
        'status'
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    protected $hidden = [
        'created_at', 'updated_at'
    ];

    public function product() {
        return $this->hasMany('App\Models\Products\ProductCategory');
    }

    public function parent()
    {
        return $this->belongsTo('App\Models\Products\Category', 'parent_id',);
    }
    
    public function children()
    {
        return $this->hasMany('App\Models\Products\Category','parent_id','id');
    }

    public function getIconAttribute($value) {
        $target = "../storage/category/".$value;
        
        return image_to_base64($target);
    }

    public function ecom_categories() {
        return $this->hasMany('App\Models\Products\EcomCategory', 'category_id', 'id');
    }

    public function get_amazon_categories() {
        return $this->hasMany('App\Models\Products\EcomCategory', 'category_id', 'id')->whereNotNull('amazon_cat_id');
    }

    public function get_ebay_categories() {
        return $this->hasMany('App\Models\Products\EcomCategory', 'category_id', 'id')->whereNotNull('ebay_cat_id');
    }

    public function amzon_categories() {
        return $this->belongsToMany('App\Models\Products\Category','ecom_categories','category_id','amazon_cat_id');
    }

    public function ebay_categories() {
        return $this->belongsToMany('App\Models\Products\Category','ecom_categories','category_id','ebay_cat_id');
    }
}
