<?php

namespace App\Models\Providers;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use App\Models\Customers\Address;
use App\Models\Customers\Jobs;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\Activitylog\Traits\LogsActivity;
use Spatie\Activitylog\LogOptions;
use App\Jobs\SendTwoFactorJob;

class Customer extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable, SoftDeletes,LogsActivity;

    protected $table = 'customers';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
         'last_name', 
         'birth_date',
         'gender',
         'other_gender',
         'email',
         'password',
         'phone_code',
         'phone', 
         'address',
         'apartment',
         'suite_number',
         'image',
         'race_id',
         'ethnicity_id',
         'religious_id',
         'relationship_id',
         'register_step',
         'register_status',
         'email_confirmed',
         'language_id',
         'device_id',
         'device_token',
         'device_type',
         'two_factor',
         'two_factor_code',
         'two_factor_expires_at',
         'otp',
         'otp_validity',
         'height_feet',
         'height_inch',
         'last_updated_by',
         'last_updated_at',
         'latitude',
         'longitude',
         'timezone',
         'city',
         'country',
         'state',
         'zipcode',
         'token',
         'country',
         'city',
         'verified'
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    public function generateTwoFactorCode()
    {
        $digits = 4;
        //$toEmail = decryptString($this->email);
        $toEmail = $this->email;
        $toName = $this->first_name;
        $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);

       /* $data = array('otp'=>$otpCode);
        Mail::send('emails.Providers.login-otp', ['data'=>$data],function($message) use ($toEmail) {
        $message->to($toEmail)
        ->subject('OTP for login');
        }); */

        $sendMail =  new SendTwoFactorJob($toName,$toEmail,$otpCode); 
        dispatch($sendMail);

        $this->timestamps = false;
        $this->two_factor_code = $otpCode;
        $this->two_factor_expires_at = strtotime(date("Y-m-d H:i:s"));
        $this->save();  

    }

    public function resetTwoFactorCode()
    {
        $this->timestamps = false;
        $this->two_factor_code = null;
        $this->two_factor_expires_at = null;
        $this->save();
    }
    
    public function relationshipStatus(){ 
        return $this->belongsTo('App\Models\Customers\Relationship','relationship_id','id');       
    }

    public function adminUser(){ 
        return $this->belongsTo('App\Models\Users\Admin','last_updated_by','id');       
    }

    public function adminUserView(){ 
        return $this->belongsTo('App\Models\Users\Admin','last_viewed_by','id');       
    }

    public function race(){ 
        return $this->belongsTo('App\Models\Customers\Race','race_id','id');       
    }

    public function ethnicity(){ 
        return $this->belongsTo('App\Models\Customers\Ethnicity','ethnicity_id','id');       
    }

    public function religious(){ 
        return $this->belongsTo('App\Models\Customers\Religious','religious_id','id');       
    }

    public function emergencyContact()
    {
        return $this->hasOne('App\Models\Customers\EmergencyContact','customer_id','id');
    }

    public function plan(){
       return $this->belongsTo('App\Models\MembershipPlan\Plan','plan_id','id');
    }
    /**
     * Get customer reports
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function flagged(){        
    	return $this->hasMany('App\Models\Reports\Report', 'node_id','id')->where('type','customer')->with('flagged_by');
    }

    public function getImageAttribute ($value) 
    {
        $directoryName = "customerProfileImages";
        return  $providerProfilePresignedUrl = presignedUrl($directoryName,$value);
        
    }


    /*public function setFirstNameAttribute($value)
    {
        $this->attributes['first_name'] = encryptString($value);
    }*/

    /*public function setLastNameAttribute($value)
    {
        $this->attributes['last_name'] = encryptString($value);
    }

    public function setEmailAttribute($value)
    {
        $this->attributes['email'] = encryptString($value);
    }

    public function setPhoneAttribute($value)
    {
        $this->attributes['phone'] = encryptString($value);
    }

    public function setAddressAttribute($value)
    {
        $this->attributes['address'] = encryptString($value);
    }*/

    /*public function getFirstNameAttribute($value)
    {
        return decryptString($value);
    }*/

    public function getActivitylogOptions(): LogOptions
    {
        return LogOptions::defaults()
        ->useLogName('Customer')
        ->logOnlyDirty()
        ->logOnly(['first_name','last_name','email','password','phone','address','birth_date','gender','race_id','ethnicity_id','religious_id','relationship_id','image','status','provider_preference','language_id','plan_id','purchased_date','register_status','register_step','email_confirmed','timezone','height_feet','height_inch','weight'])
        ->setDescriptionForEvent(fn(string $eventName) => "This customer has been {$eventName}");
    }
}
