<?php

namespace App\Models\Users;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use App\Models\Users\Address;
use App\Models\Users\Oauth;
use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class User extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable;

    protected $table = 'admins';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name', 
        'last_name', 
        'username', 
        'email', 
        'password', 
        'image',
        'role_id', 
        'phone',
        'phone_code',
        'two_factor_code',
        'two_factor_expires_at',       
    ];

    protected $casts = [
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    /**
     * Get all user addresses
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */


    public function userRole(){        
        return $this->belongsTo('App\Models\Roles\Role', 'role_id','id');
    }

    public function userData() {
        return $this->hasOne('App\Models\Users\Oauth', 'user_fk','id');
    }


    public function address() {
        return $this->hasMany(Address::class, 'user_id', 'id');
    }

    public function oauth(){        
    	return $this->belongsTo('App\Models\Users\Oauth', 'id','user_fk')->where('role_fk','1');
    }

    public function getImageAttribute($value) {
        $target = "../storage/userProfile/".$value;
        return image_to_base64($target);
    }

   //  public function getCreatedAtAttribute ($value) 
   // { 
   //      // $timezone = !empty(request()->admin_time_zone) ? request()->admin_time_zone : "America/New_York";
   //      // $dateTime = Carbon::parse($value)->timezone($timezone);
   //      // return date('m/d/Y h:i A',strtotime($dateTime));
   //      return $value;
   // }

    public function getCreatedAtAttribute ($created_at) 
   { 
      return convertTimeUtcToLocal($created_at);
     
   }

   public function getUpdatedAtAttribute ($updated_at) 
   { 
      return convertTimeUtcToLocal($updated_at);
   }

   public function setCreatedAtAttribute () 
   { 
      $this->attributes['created_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));

   }
   
   public function setUpdatedAtAttribute () 
   { 
      $this->attributes['updated_at']=convertcurrenttimetoUtc(date('Y-m-d H:i:s'));
   } 

}
