<?php

namespace App\Models\Vendors;

use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Laravel\Lumen\Auth\Authorizable;

use Illuminate\Notifications\Notifiable;
use App\Models\Vendors\Address;
use Illuminate\Database\Eloquent\SoftDeletes;

class Vendor extends Model implements AuthenticatableContract, AuthorizableContract
{
    use Authenticatable, Authorizable, HasFactory, Notifiable;

    protected $table = 'vendors';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
        'last_name', 
        'email', 
        'password',
        'phone_code',
        'phone', 
        'image', 
        'otp',
        'shop_no',
        'categories',
        'latitude',
        'longitude',
        'city',
        'country',
        'state',
        'zipcode',
        'language',
        'token',
        'status',
        'commission_type',
        'commission'
    ];

    /**
     * The attributes excluded from the model's JSON form.
     *
     * @var array
     */
    protected $hidden = [
        'password',
    ];

    protected $casts = [
        'categories' => 'string',
        'created_at' => 'datetime:Y-m-d H:i:s',
        'updated_at' => 'datetime:Y-m-d H:i:s',
    ];


    /**
     * Get all user addresses
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function address() {
        return $this->hasMany(Address::class, 'user_id', 'id');
    }

    public function oauth(){        
    	return $this->belongsTo('App\Models\Users\Oauth', 'id','user_fk')->where('role_fk','2');
    }

    public function getImageAttribute ($value) 
    {
        
        $target = $value!=NULL ?  "../storage/userProfile/".$value :  "../storage/default/default.png";
        
        return image_to_base64($target);
    }

}
