<?php

/** @var \Laravel\Lumen\Routing\Router $router */

/*
|--------------------------------------------------------------------------
| Application Routes
|--------------------------------------------------------------------------
|
| Here is where you can register all of the routes for an application.
| It is a breeze. Simply tell Lumen the URIs it should respond to
| and give it the Closure to call when that URI is requested.
|
*/

// Contractor ReviewAndRatings API
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    
    $path = "Contractors";
    
    $router->get('contractor/reviews', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@index']);
    $router->get('contractor/reviews/search', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@index']);
    $router->post('contractor/reviews', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@store']);
    $router->get('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@show']);
    $router->put('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@update']);
    $router->delete('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@delete']);
});

// Contractor API
$router->group(['prefix' => 'api/v2/'], function () use ($router) {
    
    $path = "Contractors";
    
    $router->post('contractors', ['as' => 'contractors', 'uses' => $path.'\ContractorController@store']);
    
    $router->group(['middleware' => 'auth'], function () use ($router, $path) {
        $router->get('contractors', ['as' => 'contractors', 'uses' => $path.'\ContractorController@index']);
        $router->get('contractors/search', ['as' => 'contractors', 'uses' => $path.'\ContractorController@index']);
        $router->get('contractors/count', ['as' => 'users', 'uses' => $path.'\ContractorController@contractorCount']);
        $router->post('contractors/is_email_available', ['as' => 'users', 'uses' => $path.'\ContractorController@isEmailAvailable']);
        $router->get('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@show']);
        $router->put('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@update']);
        $router->delete('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@delete']);

        // My services api routes
        $router->get('contractors/{id}/services', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@index']);
        $router->get('contractors/{id}/services/search', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@index']);
        $router->get('contractors/{contractor_id}/services/{service_id}', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@show']);
        $router->delete('contractors/{contractor_id}/services/{service_id}', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@delete']);

        // my jobs
        $router->get('contractors/{id}/jobs', ['as' => 'jobs', 'uses' => $path.'\ContractorController@allContractorJobs']);
    });
});

// Contractors Address API
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "Contractors";
    $router->get('contractor/{contractor_id}/addresses', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@index']);
    $router->get('contractor/{contractor_id}/addresses/search', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@index']);
    $router->get('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@show']);
    $router->post('contractor/{contractor_id}/addresses', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@store']);
    $router->put('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@update']);
    $router->put('contractor/{contractor_id}/addresses/{address_id}/default', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@setDefaultAddress']);
    $router->delete('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@delete']);
});

// my services api
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "Users";
    // $router->get('my-services', ['as' => 'my-services', 'uses' => $path.'\MyServicesController@index']);
    // $router->get('my-services/search', ['as' => 'my-services', 'uses' => $path.'\MyServicesController@index']);
    // $router->get('contractors/services/{id}', ['as' => 'services', 'uses' => $path.'\MyServicesController@show']);
    // $router->delete('contractors/services/{id}', ['as' => 'services', 'uses' => $path.'\MyServicesController@delete']);
});

// subscription or membership master plan
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "SubscriptionPlan";
    $router->get('memberships/plans', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@index']);
    $router->get('memberships/plans/search', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@index']);

    // $router->post('memberships/plans', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@store']);
    // $router->get('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@show']);
    // $router->put('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@update']);
    // $router->delete('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@delete']);
});

// purchage a plane or membership api
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "SubscriptionPlan";

    $router->get('memberships/members', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@index']);
    $router->get('memberships/members/search', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@index']);
    $router->post('memberships/members', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@store']);
    $router->get('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@show']);
    $router->put('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@update']);
    $router->delete('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@delete']);
});

// contractor availibility api
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "Availability";
    $router->get('availability-type', ['as' => 'availability', 'uses' => $path.'\AvailabilityTypeController@index']);

    $router->get('availability', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@index']);
    $router->post('set-availability', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@store']);
    $router->get('availability/{id}', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@show']);
    $router->put('availability/{id}', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@update']);
});


// job api
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "Users";
    $router->get('jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@show']);
    $router->put('decline-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@updateJobStatus']);
    $router->put('accept-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@updateJobStatus']);
    $router->put('start-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@startEndJob']);
    $router->put('end-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@startEndJob']);
});

// settings api
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {
    $path = "Users";
    // $router->get('contractors/settings', ['as' => 'settings', 'uses' => $path.'\ContractorAppSettingController@index']);
    // $router->put('contractors/settings', ['as' => 'settings', 'uses' => $path.'\ContractorAppSettingController@update']);
});

//**** Subcodevs v3 api ******/


$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    
    $path = "v3\Contractors";
    
    $router->get('contractor/reviews', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@index']);
    $router->get('contractor/reviews/search', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@index']);
    $router->post('contractor/reviews', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@store']);
    $router->get('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@show']);
    $router->put('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@update']);
    $router->delete('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@delete']);
});

// Contractor API
$router->group(['prefix' => 'api/v3/'], function () use ($router) {
    
    $path = "v3\Contractors";
    
    $router->post('contractors', ['as' => 'contractors', 'uses' => $path.'\ContractorController@store']);
    
    $router->group(['middleware' => 'auth'], function () use ($router, $path) {
        $router->get('contractors', ['as' => 'contractors', 'uses' => $path.'\ContractorController@index']);
        $router->get('contractors/search', ['as' => 'contractors', 'uses' => $path.'\ContractorController@index']);
        $router->get('contractors/count', ['as' => 'users', 'uses' => $path.'\ContractorController@contractorCount']);
        $router->post('contractors/is_email_available', ['as' => 'users', 'uses' => $path.'\ContractorController@isEmailAvailable']);
        $router->get('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@show']);
        $router->put('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@update']);
        $router->delete('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@delete']);

        // My services api routes
        $router->get('contractors/{id}/services', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@index']);
        $router->get('contractors/{id}/services/search', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@index']);
        $router->get('contractors/{contractor_id}/services/{service_id}', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@show']);
        $router->delete('contractors/{contractor_id}/services/{service_id}', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@delete']);

        // my jobs
        $router->get('contractors/{id}/jobs', ['as' => 'jobs', 'uses' => $path.'\ContractorController@allContractorJobs']);
    });
});

// Contractors Address API
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\Contractors";
    $router->get('contractor/{contractor_id}/addresses', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@index']);
    $router->get('contractor/{contractor_id}/addresses/search', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@index']);
    $router->get('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@show']);
    $router->post('contractor/{contractor_id}/addresses', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@store']);
    $router->put('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@update']);
    $router->put('contractor/{contractor_id}/addresses/{address_id}/default', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@setDefaultAddress']);
    $router->delete('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@delete']);
});

// my services api
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\Users";
    // $router->get('my-services', ['as' => 'my-services', 'uses' => $path.'\MyServicesController@index']);
    // $router->get('my-services/search', ['as' => 'my-services', 'uses' => $path.'\MyServicesController@index']);
    // $router->get('contractors/services/{id}', ['as' => 'services', 'uses' => $path.'\MyServicesController@show']);
    // $router->delete('contractors/services/{id}', ['as' => 'services', 'uses' => $path.'\MyServicesController@delete']);
});

// subscription or membership master plan
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\SubscriptionPlan";
    $router->get('memberships/plans', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@index']);
    $router->get('memberships/plans/search', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@index']);

    // $router->post('memberships/plans', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@store']);
    // $router->get('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@show']);
    // $router->put('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@update']);
    // $router->delete('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@delete']);
});

// purchage a plane or membership api
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\SubscriptionPlan";

    $router->get('memberships/members', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@index']);
    $router->get('memberships/members/search', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@index']);
    $router->post('memberships/members', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@store']);
    $router->get('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@show']);
    $router->put('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@update']);
    $router->delete('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@delete']);
});

// contractor availibility api
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\Availability";
    $router->get('availability-type', ['as' => 'availability', 'uses' => $path.'\AvailabilityTypeController@index']);

    $router->get('availability', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@index']);
    $router->post('set-availability', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@store']);
    $router->get('availability/{id}', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@show']);
    $router->put('availability/{id}', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@update']);
});


// job api
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\Users";
    $router->get('jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@show']);
    $router->put('decline-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@updateJobStatus']);
    $router->put('accept-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@updateJobStatus']);
    $router->put('start-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@startEndJob']);
    $router->put('end-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@startEndJob']);
});

// settings api
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v3\Users";
    // $router->get('contractors/settings', ['as' => 'settings', 'uses' => $path.'\ContractorAppSettingController@index']);
    // $router->put('contractors/settings', ['as' => 'settings', 'uses' => $path.'\ContractorAppSettingController@update']);
});


//**** Subcodevs v4 api ******/


$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    
    $path = "v4\Contractors";
    
    $router->get('contractor/reviews', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@index']);
    $router->get('contractor/reviews/search', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@index']);
    $router->post('contractor/reviews', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@store']);
    $router->get('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@show']);
    $router->put('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@update']);
    $router->delete('contractor/reviews/{id}', ['as' => 'reviews', 'uses' => $path . '\ReviewAndRatingController@delete']);
});

// Contractor API
$router->group(['prefix' => 'api/v4/'], function () use ($router) {
    
    $path = "v4\Contractors";
    
    $router->post('contractors', ['as' => 'contractors', 'uses' => $path.'\ContractorController@store']);
    
    $router->group(['middleware' => 'auth'], function () use ($router, $path) {
        $router->get('contractors', ['as' => 'contractors', 'uses' => $path.'\ContractorController@index']);
        $router->get('contractors/search', ['as' => 'contractors', 'uses' => $path.'\ContractorController@index']);
        $router->get('contractors/count', ['as' => 'users', 'uses' => $path.'\ContractorController@contractorCount']);
        $router->post('contractors/is_email_available', ['as' => 'users', 'uses' => $path.'\ContractorController@isEmailAvailable']);
        $router->get('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@show']);
        $router->put('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@update']);
        $router->delete('contractors/{id}', ['as' => 'contractors', 'uses' => $path.'\ContractorController@delete']);

        // My services api routes
        $router->get('contractors/{id}/services', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@index']);
        $router->get('contractors/{id}/services/search', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@index']);
        $router->get('contractors/{contractor_id}/services/{service_id}', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@show']);
        $router->delete('contractors/{contractor_id}/services/{service_id}', ['as' => 'services', 'uses' => $path.'\ContractorServicesController@delete']);

        // my jobs
        $router->get('contractors/{id}/jobs', ['as' => 'jobs', 'uses' => $path.'\ContractorController@allContractorJobs']);
    });
});

// Contractors Address API
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\Contractors";
    $router->get('contractor/{contractor_id}/addresses', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@index']);
    $router->get('contractor/{contractor_id}/addresses/search', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@index']);
    $router->get('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@show']);
    $router->post('contractor/{contractor_id}/addresses', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@store']);
    $router->put('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@update']);
    $router->put('contractor/{contractor_id}/addresses/{address_id}/default', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@setDefaultAddress']);
    $router->delete('contractor/{contractor_id}/addresses/{address_id}', ['as' => 'users', 'uses' => $path.'\ContractorAddressController@delete']);
});

// my services api
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\Users";
    // $router->get('my-services', ['as' => 'my-services', 'uses' => $path.'\MyServicesController@index']);
    // $router->get('my-services/search', ['as' => 'my-services', 'uses' => $path.'\MyServicesController@index']);
    // $router->get('contractors/services/{id}', ['as' => 'services', 'uses' => $path.'\MyServicesController@show']);
    // $router->delete('contractors/services/{id}', ['as' => 'services', 'uses' => $path.'\MyServicesController@delete']);
});

// subscription or membership master plan
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\SubscriptionPlan";
    $router->get('memberships/plans', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@index']);
    $router->get('memberships/plans/search', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@index']);

    // $router->post('memberships/plans', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@store']);
    // $router->get('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@show']);
    // $router->put('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@update']);
    // $router->delete('memberships/plans/{id}', ['as' => 'memberships', 'uses' => $path.'\SubscriptionPlanController@delete']);
});

// purchage a plane or membership api
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\SubscriptionPlan";

    $router->get('memberships/members', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@index']);
    $router->get('memberships/members/search', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@index']);
    $router->post('memberships/members', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@store']);
    $router->get('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@show']);
    $router->put('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@update']);
    $router->delete('memberships/members/{id}', ['as' => 'memberships', 'uses' => $path.'\PurchagePlanController@delete']);
});

// contractor availibility api
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\Availability";
    $router->get('availability-type', ['as' => 'availability', 'uses' => $path.'\AvailabilityTypeController@index']);

    $router->get('availability', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@index']);
    $router->post('set-availability', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@store']);
    $router->get('availability/{id}', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@show']);
    $router->put('availability/{id}', ['as' => 'availability', 'uses' => $path.'\AvailabilityController@update']);
});


// job api
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\Users";
    $router->get('jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@show']);
    $router->put('decline-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@updateJobStatus']);
    $router->put('accept-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@updateJobStatus']);
    $router->put('start-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@startEndJob']);
    $router->put('end-jobs/{id}', ['as' => 'availability', 'uses' => $path.'\JobsController@startEndJob']);
});

// settings api
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {
    $path = "v4\Users";
    // $router->get('contractors/settings', ['as' => 'settings', 'uses' => $path.'\ContractorAppSettingController@index']);
    // $router->put('contractors/settings', ['as' => 'settings', 'uses' => $path.'\ContractorAppSettingController@update']);
});
