<?php

/** @var \Laravel\Lumen\Routing\Router $router */
/*
  |--------------------------------------------------------------------------
  | Application Routes
  |--------------------------------------------------------------------------
  |
  | Here is where you can register all of the routes for an application.
  | It is a breeze. Simply tell Lumen the URIs it should respond to
  | and give it the Closure to call when that URI is requested.
  |
 */

//product category
$router->group(['prefix' => 'api/v2/'], function () use ($router) {

    $path = "Products";
    $router->get('product/categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@index']);
    $router->get('product/ecom-categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@categoriesWithAmazonEbay']);
    $router->post('products/sub_categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@allSubCatShow']);
    $router->get('products/ecommerce_cat', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@ecommerceCategory']);
});

//product variant
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {

  $path = "Products";
  $router->post('product/{id}/variants',['as' => 'variant', 'uses' => $path . '\ProductVariantController@store']);
  $router->get('product/{id}/variants/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@show']);
  $router->get('product/{id}/variants',['as' => 'variant', 'uses' => $path . '\ProductVariantController@index']);
  $router->get('product/{id}/variants/count',['as' => 'variant', 'uses' => $path . '\ProductVariantController@count']);
  $router->post('product/{id}/variants_update/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@update']);
  $router->delete('product/{id}/variants/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@delete']);

});

//product review
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {

  $path = "Products";
  $router->post('product/reviews',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@store']);
  $router->get('product/reviews',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@index']);
  $router->get('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@show']);
  $router->put('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@update']);
  $router->delete('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@delete']);
  $router->put('product/review/approve',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@approve']);
});


$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {

  $path = "Products";
  $router->get('product/categories/count',['as' => 'category', 'uses' => $path . '\ProductCategoryController@count']);
  $router->post('product/categories',['as' => 'category', 'uses' => $path . '\ProductCategoryController@store']);
  $router->post('product/update_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@update']);
  $router->delete('product/categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@delete']);
  $router->get('product/categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@show']);

  $router->post('product/sub_categories',['as' => 'category', 'uses' => $path . '\ProductCategoryController@storeSubCat']);
  $router->post('product/update_sub_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@updateSubCat']);
  $router->delete('product/sub_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@deleteSubCategory']);
  $router->get('product/sub_categories/count',['as' => 'cProductsategory', 'uses' => $path . '\ProductCategoryController@subCatCount']);
  $router->get('product/sub_categories/{id}',['as' => 'cProductsategory', 'uses' => $path . '\ProductCategoryController@subCatShow']);
});

$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {

  $path = "Products";
  $router->get('products', ['as' => 'product', 'uses' => $path . '\ProductController@index']);
  $router->post('products', ['as' => 'product', 'uses' => $path . '\ProductController@store']);
  $router->get('product/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@show']);
  $router->get('products/count', ['as' => 'product', 'uses' => $path . '\ProductController@totalProducts']);
  $router->put('products/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@update']);
  $router->delete('products/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@delete']);
});

//product Images
$router->group(['prefix' => 'api/v2/', 'middleware' => 'auth'], function () use ($router) {

  $path = "Products";
  $router->post('product/{id}/images', ['as' => 'product', 'uses' => $path . '\ProductImageController@store']);
  $router->get('product/{id}/images', ['as' => 'product', 'uses' => $path . '\ProductImageController@show']);
  $router->post('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@update']);
  $router->get('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@single']);
  $router->delete('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@delete']);
  $router->get('product/{id}/images/count', ['as' => 'product', 'uses' => $path . '\ProductImageController@count']);
});


//**** Subcodevs v3 api ******/


//product category
$router->group(['prefix' => 'api/v3/'], function () use ($router) {

  $path = "v3\Products";
  $router->get('product/categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@index']);
  $router->get('product/ecom-categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@categoriesWithAmazonEbay']);
  $router->post('products/sub_categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@allSubCatShow']);
  $router->get('products/ecommerce_cat', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@ecommerceCategory']);
});

//product variant
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {

$path = "v3\Products";
$router->post('product/{id}/variants',['as' => 'variant', 'uses' => $path . '\ProductVariantController@store']);
$router->get('product/{id}/variants/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@show']);
$router->get('product/{id}/variants',['as' => 'variant', 'uses' => $path . '\ProductVariantController@index']);
$router->get('product/{id}/variants/count',['as' => 'variant', 'uses' => $path . '\ProductVariantController@count']);
$router->post('product/{id}/variants_update/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@update']);
$router->delete('product/{id}/variants/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@delete']);

});

//product review
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {

$path = "v3\Products";
$router->post('product/reviews',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@store']);
$router->get('product/reviews',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@index']);
$router->get('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@show']);
$router->put('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@update']);
$router->delete('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@delete']);
$router->put('product/review/approve',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@approve']);
});


$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {

$path = "v3\Products";
$router->get('product/categories/count',['as' => 'category', 'uses' => $path . '\ProductCategoryController@count']);
$router->post('product/categories',['as' => 'category', 'uses' => $path . '\ProductCategoryController@store']);
$router->post('product/update_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@update']);
$router->delete('product/categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@delete']);
$router->get('product/categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@show']);

$router->post('product/sub_categories',['as' => 'category', 'uses' => $path . '\ProductCategoryController@storeSubCat']);
$router->post('product/update_sub_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@updateSubCat']);
$router->delete('product/sub_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@deleteSubCategory']);
$router->get('product/sub_categories/count',['as' => 'cProductsategory', 'uses' => $path . '\ProductCategoryController@subCatCount']);
$router->get('product/sub_categories/{id}',['as' => 'cProductsategory', 'uses' => $path . '\ProductCategoryController@subCatShow']);
});

$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {

$path = "v3\Products";
$router->get('products', ['as' => 'product', 'uses' => $path . '\ProductController@index']);
$router->post('products', ['as' => 'product', 'uses' => $path . '\ProductController@store']);
$router->get('product/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@show']);
$router->get('products/count', ['as' => 'product', 'uses' => $path . '\ProductController@totalProducts']);
$router->put('products/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@update']);
$router->delete('products/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@delete']);
});

//product Images
$router->group(['prefix' => 'api/v3/', 'middleware' => 'auth'], function () use ($router) {

$path = "v3\Products";
$router->post('product/{id}/images', ['as' => 'product', 'uses' => $path . '\ProductImageController@store']);
$router->get('product/{id}/images', ['as' => 'product', 'uses' => $path . '\ProductImageController@show']);
$router->post('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@update']);
$router->get('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@single']);
$router->delete('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@delete']);
$router->get('product/{id}/images/count', ['as' => 'product', 'uses' => $path . '\ProductImageController@count']);
});



//**** Subcodevs v4 api ******/


//product category
$router->group(['prefix' => 'api/v4/'], function () use ($router) {

  $path = "v4\Products";
  $router->get('product/categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@index']);
  $router->get('product/ecom-categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@categoriesWithAmazonEbay']);
  $router->post('products/sub_categories', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@allSubCatShow']);
  $router->get('products/ecommerce_cat', ['as' => 'category', 'uses' => $path . '\ProductCategoryController@ecommerceCategory']);
});

//product variant
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {

$path = "v4\Products";
$router->post('product/{id}/variants',['as' => 'variant', 'uses' => $path . '\ProductVariantController@store']);
$router->get('product/{id}/variants/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@show']);
$router->get('product/{id}/variants',['as' => 'variant', 'uses' => $path . '\ProductVariantController@index']);
$router->get('product/{id}/variants/count',['as' => 'variant', 'uses' => $path . '\ProductVariantController@count']);
$router->post('product/{id}/variants_update/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@update']);
$router->delete('product/{id}/variants/{variant_id}',['as' => 'variant', 'uses' => $path . '\ProductVariantController@delete']);

});

//product review
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {

$path = "v4\Products";
$router->post('product/reviews',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@store']);
$router->get('product/reviews',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@index']);
$router->get('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@show']);
$router->put('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@update']);
$router->delete('product/reviews/{id}',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@delete']);
$router->put('product/review/approve',['as' => 'reviews', 'uses' => $path . '\ProductReviewController@approve']);
});


$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {

$path = "v4\Products";
$router->get('product/categories/count',['as' => 'category', 'uses' => $path . '\ProductCategoryController@count']);
$router->post('product/categories',['as' => 'category', 'uses' => $path . '\ProductCategoryController@store']);
$router->post('product/update_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@update']);
$router->delete('product/categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@delete']);
$router->get('product/categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@show']);

$router->post('product/sub_categories',['as' => 'category', 'uses' => $path . '\ProductCategoryController@storeSubCat']);
$router->post('product/update_sub_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@updateSubCat']);
$router->delete('product/sub_categories/{id}',['as' => 'category', 'uses' => $path . '\ProductCategoryController@deleteSubCategory']);
$router->get('product/sub_categories/count',['as' => 'cProductsategory', 'uses' => $path . '\ProductCategoryController@subCatCount']);
$router->get('product/sub_categories/{id}',['as' => 'cProductsategory', 'uses' => $path . '\ProductCategoryController@subCatShow']);
});

$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {

$path = "v4\Products";
$router->get('products', ['as' => 'product', 'uses' => $path . '\ProductController@index']);
$router->post('products', ['as' => 'product', 'uses' => $path . '\ProductController@store']);
$router->get('product/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@show']);
$router->get('products/count', ['as' => 'product', 'uses' => $path . '\ProductController@totalProducts']);
$router->put('products/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@update']);
$router->delete('products/{id}', ['as' => 'product', 'uses' => $path . '\ProductController@delete']);
});

//product Images
$router->group(['prefix' => 'api/v4/', 'middleware' => 'auth'], function () use ($router) {

$path = "v4\Products";
$router->post('product/{id}/images', ['as' => 'product', 'uses' => $path . '\ProductImageController@store']);
$router->get('product/{id}/images', ['as' => 'product', 'uses' => $path . '\ProductImageController@show']);
$router->post('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@update']);
$router->get('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@single']);
$router->delete('product/images/{id}', ['as' => 'product', 'uses' => $path . '\ProductImageController@delete']);
$router->get('product/{id}/images/count', ['as' => 'product', 'uses' => $path . '\ProductImageController@count']);
});
