<?php

/** @var \Laravel\Lumen\Routing\Router $router */
/*
  |--------------------------------------------------------------------------
  | Application Routes
  |--------------------------------------------------------------------------
  |
  | Here is where you can register all of the routes for an application.
  | It is a breeze. Simply tell Lumen the URIs it should respond to
  | and give it the Closure to call when that URI is requested.
  |
 */


$router->group(['prefix' => 'api/v2/'], function () use ($router) {

    $path = "Users";
    $router->get('users/sendmail', ['as' => 'users', 'uses' => $path . '\UserController@sendMail']);
    
    $router->post('users/password_reset_link', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@sendResetPasswordLink']);

    $router->post('users/change_email_link', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@sendChangeEmailLink']);

    $router->post('users/password_update/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@updatePassword']);

     $router->put('users/reset_password/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@resetPassword']);

     $router->put('users/change_email/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@changeEmail']);

    $router->get('users', ['as' => 'users', 'uses' => $path . '\UserController@index']);
    $router->post('users', ['as' => 'users', 'uses' => $path . '\UserController@store']);
    $router->get('users/count', ['as' => 'users', 'uses' => $path . '\UserController@userAdminCount']);
    $router->get('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@show']);
    $router->get('users-detail/{id}', ['as' => 'users', 'uses' => $path . '\UserController@usersDetail']);
    $router->put('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@update']);

    $router->post('users/update_profile/{id}',['as' => 'users', 'uses' => $path . '\UserController@updateProfile']);

    $router->put('users/{id}/reset_password', ['as' => 'users', 'uses' => $path . '\UserController@resetPassword']);

    $router->post('users/{id}/profile_picture', ['as' => 'users', 'uses' => $path . '\UserController@uploadAvatar']);
    $router->get('users/{id}/profile_picture', ['as' => 'users', 'uses' => $path . '\UserController@viewAvatar']);
    $router->put('users/{id}/block', ['as' => 'users', 'uses' => $path . '\UserController@userBlock']);
    $router->delete('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@delete']);
    $router->put('users/{id}/active_inactive', ['as' => 'users', 'uses' => $path . '\UserController@activeInactive']);

    $router->post('users/{id}/scopes', ['as' => 'users', 'uses' => $path . '\UserController@setScopes']);
    $router->get('users/{id}/scopes', ['as' => 'users', 'uses' => $path . '\UserController@getScopes']);
    
    // Roles
    $router->get('user_roles/{role_id}/users', ['as' => 'users', 'uses' => $path . '\RoleController@getUsers']);
    $router->get('user_roles', ['as' => 'users', 'uses' => $path . '\RoleController@index']);
    $router->get('all-roles', ['as' => 'users', 'uses' => $path . '\RoleController@allRoles']);
    $router->get('user_roles/count', ['as' => 'users', 'uses' => $path . '\RoleController@getCount']);
    $router->get('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@show']);
    $router->post('user_roles', ['as' => 'users', 'uses' => $path . '\RoleController@store']);
    $router->put('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@update']);
    $router->delete('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@delete']);

    $router->post('user_roles/{role_id}/scopes', ['as' => 'users', 'uses' => $path . '\RoleController@setScopes']);
    $router->get('user_roles/{role_id}/scopes', ['as' => 'users', 'uses' => $path . '\RoleController@getScopes']);

    $router->put('user_roles/{id}/active-inactive', ['as' => 'users', 'uses' => $path . '\RoleController@activeInactive']);

    // Scopes
    $router->get('scopes', ['as' => 'users', 'uses' => $path . '\ScopeController@index']);
});


//**** Subcodevs v3 api ******/


$router->group(['prefix' => 'api/v3/'], function () use ($router) {

  $path = "v3\Users";
  $router->get('users/sendmail', ['as' => 'users', 'uses' => $path . '\UserController@sendMail']);
  
  $router->post('users/password_reset_link', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@sendResetPasswordLink']);

  $router->post('users/change_email_link', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@sendChangeEmailLink']);

  $router->post('users/password_update/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@updatePassword']);

   $router->put('users/reset_password/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@resetPassword']);

   $router->put('users/change_email/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@changeEmail']);

  $router->get('users', ['as' => 'users', 'uses' => $path . '\UserController@index']);
  $router->post('users', ['as' => 'users', 'uses' => $path . '\UserController@store']);
  $router->get('users/count', ['as' => 'users', 'uses' => $path . '\UserController@userAdminCount']);
  $router->get('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@show']);
  $router->get('users-detail/{id}', ['as' => 'users', 'uses' => $path . '\UserController@usersDetail']);
  $router->put('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@update']);

  $router->post('users/update_profile/{id}',['as' => 'users', 'uses' => $path . '\UserController@updateProfile']);

  $router->put('users/{id}/reset_password', ['as' => 'users', 'uses' => $path . '\UserController@resetPassword']);

  $router->post('users/{id}/profile_picture', ['as' => 'users', 'uses' => $path . '\UserController@uploadAvatar']);
  $router->get('users/{id}/profile_picture', ['as' => 'users', 'uses' => $path . '\UserController@viewAvatar']);
  $router->put('users/{id}/block', ['as' => 'users', 'uses' => $path . '\UserController@userBlock']);
  $router->delete('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@delete']);
  $router->put('users/{id}/active_inactive', ['as' => 'users', 'uses' => $path . '\UserController@activeInactive']);

  $router->post('users/{id}/scopes', ['as' => 'users', 'uses' => $path . '\UserController@setScopes']);
  $router->get('users/{id}/scopes', ['as' => 'users', 'uses' => $path . '\UserController@getScopes']);
  
  // Roles
  $router->get('user_roles/{role_id}/users', ['as' => 'users', 'uses' => $path . '\RoleController@getUsers']);
  $router->get('user_roles', ['as' => 'users', 'uses' => $path . '\RoleController@index']);
  $router->get('all-roles', ['as' => 'users', 'uses' => $path . '\RoleController@allRoles']);
  $router->get('user_roles/count', ['as' => 'users', 'uses' => $path . '\RoleController@getCount']);
  $router->get('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@show']);
  $router->post('user_roles', ['as' => 'users', 'uses' => $path . '\RoleController@store']);
  $router->put('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@update']);
  $router->delete('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@delete']);

  $router->post('user_roles/{role_id}/scopes', ['as' => 'users', 'uses' => $path . '\RoleController@setScopes']);
  $router->get('user_roles/{role_id}/scopes', ['as' => 'users', 'uses' => $path . '\RoleController@getScopes']);

  $router->put('user_roles/{id}/active-inactive', ['as' => 'users', 'uses' => $path . '\RoleController@activeInactive']);

  // Scopes
  $router->get('scopes', ['as' => 'users', 'uses' => $path . '\ScopeController@index']);
});



//**** Subcodevs v4 api ******/


$router->group(['prefix' => 'api/v4/'], function () use ($router) {

  $path = "v4\Users";
  $router->get('users/sendmail', ['as' => 'users', 'uses' => $path . '\UserController@sendMail']);
  
  $router->post('users/password_reset_link', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@sendResetPasswordLink']);

  $router->post('users/change_email_link', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@sendChangeEmailLink']);

  $router->post('users/password_update/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@updatePassword']);

   $router->put('users/reset_password/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@resetPassword']);

   $router->put('users/change_email/{token}', ['as' => 'users', 'uses' => $path . '\ForgotPasswordController@changeEmail']);

  $router->get('users', ['as' => 'users', 'uses' => $path . '\UserController@index']);
  $router->post('users', ['as' => 'users', 'uses' => $path . '\UserController@store']);
  $router->get('users/count', ['as' => 'users', 'uses' => $path . '\UserController@userAdminCount']);
  $router->get('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@show']);
  $router->get('users-detail/{id}', ['as' => 'users', 'uses' => $path . '\UserController@usersDetail']);
  $router->put('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@update']);

  $router->post('users/update_profile/{id}',['as' => 'users', 'uses' => $path . '\UserController@updateProfile']);

  $router->put('users/{id}/reset_password', ['as' => 'users', 'uses' => $path . '\UserController@resetPassword']);

  $router->post('users/{id}/profile_picture', ['as' => 'users', 'uses' => $path . '\UserController@uploadAvatar']);
  $router->get('users/{id}/profile_picture', ['as' => 'users', 'uses' => $path . '\UserController@viewAvatar']);
  $router->put('users/{id}/block', ['as' => 'users', 'uses' => $path . '\UserController@userBlock']);
  $router->delete('users/{id}', ['as' => 'users', 'uses' => $path . '\UserController@delete']);
  $router->put('users/{id}/active_inactive', ['as' => 'users', 'uses' => $path . '\UserController@activeInactive']);

  $router->post('users/{id}/scopes', ['as' => 'users', 'uses' => $path . '\UserController@setScopes']);
  $router->get('users/{id}/scopes', ['as' => 'users', 'uses' => $path . '\UserController@getScopes']);
  
  // Roles
  $router->get('user_roles/{role_id}/users', ['as' => 'users', 'uses' => $path . '\RoleController@getUsers']);
  $router->get('user_roles', ['as' => 'users', 'uses' => $path . '\RoleController@index']);
  $router->get('all-roles', ['as' => 'users', 'uses' => $path . '\RoleController@allRoles']);
  $router->get('user_roles/count', ['as' => 'users', 'uses' => $path . '\RoleController@getCount']);
  $router->get('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@show']);
  $router->post('user_roles', ['as' => 'users', 'uses' => $path . '\RoleController@store']);
  $router->put('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@update']);
  $router->delete('user_roles/{id}', ['as' => 'users', 'uses' => $path . '\RoleController@delete']);

  $router->post('user_roles/{role_id}/scopes', ['as' => 'users', 'uses' => $path . '\RoleController@setScopes']);
  $router->get('user_roles/{role_id}/scopes', ['as' => 'users', 'uses' => $path . '\RoleController@getScopes']);

  $router->put('user_roles/{id}/active-inactive', ['as' => 'users', 'uses' => $path . '\RoleController@activeInactive']);

  // Scopes
  $router->get('scopes', ['as' => 'users', 'uses' => $path . '\ScopeController@index']);
});
