<?php

namespace App\Http\Controllers\v4\Customers;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Customers\CustomerNotificationController;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;
use App\Models\Customers\Jobs;
use App\Models\Customers\Customer;
use App\Models\Customers\Race;
use App\Models\Customers\Ethnicity;
use App\Models\Customers\Religious;
use App\Models\Customers\Relationship;
use App\Models\Customers\IntakeQuestion;
use App\Models\Customers\CustomerIntakeQuestion;
use App\Models\Customers\CustomerIntakeQuestionAnswer;
use App\Models\Customers\TempCustomer;
use App\Models\Customers\Oauth;
use App\Models\Customers\EmergencyContact;
use App\Models\Customers\IntakeQuestionOptionSpecialization;
use App\Models\Providers\ProviderSpecialization;
use App\Models\Customers\Provider;
use App\Http\Controllers\Auth\OauthServerController;
use App\Models\Users\User;
use App\Models\Settings\Setting;
use App\Models\Customers\Appointment;
use App\Models\Customers\AppointmentCancelled;
use App\Models\Customers\ProviderRatingReview;
use App\Models\Customers\RescheduleAppointment;
use App\Models\Customers\AudioVideoCallHistory;
use App\Models\Customers\MessageRoom;
use App\Models\Customers\MessageRoomChat;
use App\Models\Providers\ProviderAvailability;
use App\Models\Providers\ProviderLanguage;
use App\Models\MembershipPlan\Plan;
use App\Models\ChangeProvider\ChangeProvider;
use App\Models\CustomerPaymentCard\CustomerPaymentCard;
use App\Models\Journals\Journal;
use App\Models\MembershipPlan\Payment;
use App\Models\Notifications\NotificationSetting;
use App\Models\AppointmentManagement\NoShow;
use App\Models\Customers\ProviderTimeOff;
use App\Models\UserPasswordHistory\UserPasswordHistory;
use App\Models\Providers\Language;
use Illuminate\Support\Facades\App;
use App\Notifications\Customers\Notification;
use App\Models\Notifications\Notification as DatabaseNotification;
use Illuminate\Support\Arr;
use App\Jobs\CustomerForgotPasswordMailJob;
use App\Jobs\CustomerMailJob;
use App\Jobs\SendMailJob;
use App\Jobs\CustomerCommonMailJob;
use App\Jobs\CustomerAppointmentMailJob;
use App\Jobs\CustomerConfirmEmailJob;
use App\Jobs\CustomerRegisteredEmailJob;
use App\Jobs\CustomerLinkEmailJob;
use App\Models\EmailTemplate\EmailTemplate;
use App\Models\Providers\ProviderSlot;
use Illuminate\Support\Str;
use Validator;
use Exception;
use Throwable;
use Carbon\Carbon;
use Ramsey\Uuid\Uuid;

class CustomerController extends Controller 
{
    /**
    * The user repository implementation.
    *
    * @var UserRepository
    */
    protected $customer;
    protected $oauth_user;
    protected $jobs;
    protected $user_id;
    protected $role_fk;

    public function __construct(Customer $customer, Oauth $oauth_user, Jobs $jobs, Request $request)
    {
        $this->customer = $customer;
        $this->oauth_user = $oauth_user;
        $this->jobs = $jobs;
        $this->user_id = $request->user_id;
        $this->role_fk = $request->role_fk;
        $this->user_fk = $request->user_fk;
        //has_scope('customers.read');
    }


    /**
    * @ user list List api using pagination.
    * @return \Illuminate\Http\Response
    */
    public function index(Request $request)
    {
        try
        {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            /*if ($limit > env('MAX_PER_PAGE')) {
            $limit = 25;
            }*/
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }
            $customers = $this->customer::with(['plan'])
            ->where(function ($q) use ($request) {
                // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    if (!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if ($key == 0) {
                                $q->where($col, 'like', '%' . encryptString($request->search) . '%');
                                $q->orWhere('email', 'like', '%' . encryptString(strtolower($request->search)) . '%');
                                $q->orWhere('first_name', 'like', '%' . encryptString(ucfirst(strtolower($request->search))) . '%');
                                $q->orWhere('last_name', 'like', '%' . encryptString(ucfirst(strtolower($request->search))) . '%');
                            } else {
                                $pieces = explode(" ", $request->search);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                }
                                $q->orWhere($col, 'like', '%' . encryptString($request->search) . '%');
                                $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%'.encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        }
                    }
                }
                // filters
                if($request->filled('name')) {
                    $pieces = explode(" ", $request->name);
                    if(count($pieces)){
                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->name; 
                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->name; 
                    }
                    $q->where('first_name', 'like', '%'.encryptString(ucfirst(strtolower($request->name))).'%');
                    $q->orwhere("last_name",'like', '%'.encryptString(ucfirst(strtolower($request->name))).'%');
                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' .encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))).'%');
                }
                if($request->filled('email')) {
                    $q->where('email', 'like', '%'.encryptString(strtolower($request->email)).'%');
                }
                if($request->filled('date_from') && $request->filled('date_to')) {
                    $date_from = date("Y-m-d", strtotime($request->date_from));
                    $date_to = date("Y-m-d", strtotime($request->date_to));
                    $q->whereBetween('created_at', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                }
            })
            ->where(function ($q) use ($request) {
                // Column wise searching filter ...
                $allowableColumn = $this->getColumnName();
                if (isset($request->column_name) && !empty($request->column_name) && $request->column_name != "" && in_array($request->column_name, $allowableColumn)) {
                    $q->where($request->column_name, 'like', '%' . $request->column_value . '%');
                }
            })
            ->where(['register_step'=>4,'register_status'=>'completed'])
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            if (empty($customers)) {
                throw new Exception(config('service.CUSTOMER_API.MESSAGE.CUSTOMER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.CUSTOMER_API.MESSAGE.CUSTOMER_LISTING'), 'records' => $customers]);
            }
            catch (Exception $e) {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
            catch (Throwable $e) {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }


    public function archivedCustomers(Request $request)
    {
        try {
            $order = "desc";
            $limit = isset($request->limit) ? $request->limit : 25;
            if (isset($request->order) && strtolower($request->order) == "asc") {
                $order = $request->order;
            }
            $order_by_column_name = "id";
            if ($request->filled('order_by_column_name')) {
                if (in_array($request->order_by_column_name, $this->getColumnName())) {
                    $order_by_column_name = $request->order_by_column_name;
                }
            }

            $customers = $this->customer::onlyTrashed()
            ->where(function ($q) use ($request) {
            // Global searching...
                if (isset($request->search) && !empty($request->search) && $request->search != "") {
                    if (!empty($this->getColumnName())) {
                        foreach ($this->getColumnName() as $key => $col) {
                            if ($key == 0) {
                                $q->where($col, 'like', '%' . encryptString($request->search) . '%');
                                $q->orWhere('first_name', 'like', '%' . encryptString(ucfirst(strtolower($request->search))) . '%');
                                $q->orWhere('last_name', 'like', '%' . encryptString(ucfirst(strtolower($request->search))) . '%');
                            } else {
                                $pieces = explode(" ", $request->search);
                                if(count($pieces)){
                                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->search; 
                                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->search; 
                                }
                                $q->orWhere($col, 'like', '%' . encryptString($request->search) . '%');
                                $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%'.encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))).'%');
                            }
                        }
                    }
                }
                // filters
                if($request->filled('name')) {
                    $pieces = explode(" ", $request->name);
                    if(count($pieces)){
                        $fName = !empty($pieces[0]) ? $pieces[0] : $request->name; 
                        $lName = !empty($pieces[1]) ? $pieces[1] : $request->name; 
                    }
                    $q->where('first_name', 'like', '%'.encryptString(ucfirst(strtolower($request->name))).'%');
                    $q->orwhere("last_name",'like', '%'.encryptString(ucfirst(strtolower($request->name))).'%');
                    $q->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%' . encryptString(ucfirst(strtolower($fName)))." ".encryptString(ucfirst(strtolower($lName))). '%');
                }
                if($request->filled('email')) {
                    $q->where('email', 'like', '%'.encryptString($request->email).'%');
                }
                if($request->filled('date_from') && $request->filled('date_to')) {
                    $date_from = date("Y-m-d", strtotime($request->date_from));
                    $date_to = date("Y-m-d", strtotime($request->date_to));
                    $q->whereBetween('created_at', [$date_from." "."00:00:00", $date_to." "."23:59:59"]);
                }
            })
            ->where(['register_step'=>4,'register_status'=>'completed'])
            ->orderBy($order_by_column_name, $order)
            ->paginate($limit);

            if (empty($customers)) {
                throw new Exception(config('service.CUSTOMER_API.MESSAGE.CUSTOMER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.CUSTOMER_API.MESSAGE.CUSTOMER_LISTING'), 'records' => $customers]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function archiveCustomer(Request $request, $id) 
    {
        //has_scope('roles.delete');
        $customer = Customer::findOrFail($id);
        try {
            $customerName = decryptString($customer->first_name)." ".decryptString($customer->last_name);
            $customerEmail = decryptString($customer->email);
            $customer  = $customer->delete();

            /* delete access token */
                $user =  $this->oauth_user::select('user_id')->where(['user_fk' => $id])->first();
                DB::table('oauth_access_tokens')->where('user_id',$user->user_id)->delete(); 
            /* end access token */
            
            $templateName = "Customer account archive";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                /* Customer archive email */
                    $greeting = 'Hi ' . $customerName.' ,';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $sendMail = new CustomerCommonMailJob($customerEmail, $greeting, $body, $type);
                    dispatch($sendMail);
                /* End customer archive email */
            }

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_ARCHIVE'), 'result' => $customer]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function unarchiveCustomer($id) 
    {
        //has_scope('roles.delete');
        $customer = Customer::withTrashed()->find($id);
        try {
            $customer->deleted_at = Null;
            $customer->save();

            $templateName = "Customer account unarchive";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                /* Customer unarchive email */
                    $greeting = 'Hi ' .decryptString($customer->first_name).' '.decryptString($customer->last_name).' ,';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                /* End customer unarchive email */
            }

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_UNARCHIVE'), 'result' => $customer]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function unblockCustomer($id) 
    {
        //has_scope('roles.delete');
        $customer = Customer::findOrFail($id);
        try {
            $customer->login_attempt = Null;
            $customer->is_blocked = "no";
            $customer->login_attempt_time = Null;
            $customer->save();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_UNBLOCK'), 'result' => $customer]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function deleteCustomer(Request $request, $id) 
    {
        //has_scope('roles.delete');
        $customer = Customer::withTrashed()->find($id);
        try {
            DB::beginTransaction();
            $name = decryptString($customer->first_name)." ".decryptString($customer->last_name);
            $email = $customer->email;
            $customer  = $customer->forceDelete();
            $this->oauth_user::where(['user_fk' => $id,'email'=>$email])->delete();
            DB::commit();

            $templateName = "Customer account deleted";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                /* Customer delete email */
                    $greeting = 'Hi ' .$name.' ,';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $sendMail = new CustomerCommonMailJob(decryptString($email), $greeting, $body, $type);
                    dispatch($sendMail);
                /* End customer delete email */
            }

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_DELETED'), 'result' => $customer]);
        }
        catch (Exception $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function activeInactive(Request $request, $id) 
    {
        //has_scope('cms.pages.write');
        $this->validate($request,[
            'status' => 'required',
        ]);
        try {
            $result = Customer::findOrFail($id);
            $result->status = $request->status;
            $result->save();

            if($request->status=="active")
            {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_STATUS_ACTIVE'),'action'=>$request->status]);
            }
            else
            {
                /* delete access token */
                    $user =  $this->oauth_user::select('user_id')->where(['user_fk' => $id])->first();
                    DB::table('oauth_access_tokens')->where('user_id',$user->user_id)->delete(); 
                /* end access token */

                $templateName = "Customer account inactive";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Customer inactive email */
                        $greeting = 'Hi ' . decryptString($result->first_name)." ".decryptString($result->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new CustomerCommonMailJob(decryptString($result->email), $greeting, $body, $type);
                        dispatch($sendMail);
                    /* End customer inactive email */
                }

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_STATUS_INACTIVE'),'action'=>$request->status]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }


    public function show($id)
    {
        try {
                $customer = Customer::with(['relationshipStatus','race','ethnicity','religious','emergencyContact','adminUser','adminUserView','plan'])->find($id);

                $customer->last_viewed_by = $this->user_id;
                $customer->last_viewed_at = date("Y-m-d H:i:s");
                $customer->save();
            /*$directoryName = "customerProfileImages";
            $customerProfilePresignedUrl = presignedUrl($directoryName,$customer->image);*/
            if (empty($customer)) {

                throw new Exception(config('service.CUSTOMER_API.MESSAGE.CUSTOMER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.CUSTOMER_DETAILS'), 'records' => $customer]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateCustomer(Request $request, $id)
    {
        has_scope('consumer.write');
        $message = ["image.max" => "Image size should be less than 5 mb."];
        $this->validate($request, [
            'first_name' => 'required|max:30',
            'last_name' => 'required|max:30',
            'phone' => "required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:customers,phone,$id",
            //'email' => "required|email|max:100|unique:customers,email,$id",
            //'phone' => "required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10",
            //'email' => "required|max:100",
            'emergency_phone_no' => "regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10",
            'emergency_first_name' => 'max:30',
            'emergency_last_name' => 'max:30',
            'emergency_email' => "email|max:100",
            'emergency_relationship' => "max:30",
            //'address' => 'required|max:2000',
            'image'    => 'image|mimes:jpeg,png,jpg,gif,svg|max:5000',
        ],$message);
        try {
            
                $customer = Customer::findOrFail($id);
                DB::beginTransaction();
                $input = $request->all();
                $input['first_name'] = encryptString(ucfirst(strtolower($input['first_name'])));
                $input['last_name'] = encryptString(ucfirst(strtolower($input['last_name'])));
                //$input['email'] = encryptString($input['email']);
                $input['phone'] = encryptString($input['phone']);
                $input['address'] = encryptString($input['address']);
                if($request->has('image')) {
                    $imageName = rand(10000, 9999999) . '-' . time() . '.' . request()->image->getClientOriginalExtension();
                    $directoryName = "customerProfileImages";
                    uploadFile(request()->image,$directoryName,$imageName);
                    $input['image'] =  $imageName;
                }
                $save  = $customer->update($input);
                    // update
                $emergencyContact = EmergencyContact::where('customer_id',$id)->first();
                if($emergencyContact)
                {
                    $emergencyContact->first_name = !empty($input['emergency_first_name'])? encryptString($input['emergency_first_name']):'';
                    $emergencyContact->last_name = !empty($input['emergency_last_name'])? encryptString($input['emergency_last_name']):'';
                    $emergencyContact->phone_no = !empty($input['emergency_phone_no'])? encryptString($input['emergency_phone_no']):'';
                    $emergencyContact->email = !empty($input['emergency_email'])? encryptString($input['emergency_email']):'';
                    //$emergencyContact->address = !empty($input['emergency_address'])? encryptString($input['emergency_address']):'';
                    $emergencyContact->relationship = !empty($input['emergency_relationship'])? encryptString($input['emergency_relationship']):'';
                    $emergencyContact->save();
                }
                else
                {
                    $emergencyContact = new EmergencyContact();
                    $emergencyContact->customer_id = $id;
                    $emergencyContact->first_name = !empty($input['emergency_first_name'])? encryptString($input['emergency_first_name']):'';
                    $emergencyContact->last_name = !empty($input['emergency_last_name'])? encryptString($input['emergency_last_name']):'';
                    $emergencyContact->phone_no = !empty($input['emergency_phone_no'])? encryptString($input['emergency_phone_no']):'';
                    $emergencyContact->email = !empty($input['emergency_email'])? encryptString($input['emergency_email']):'';
                    //$emergencyContact->address = !empty($input['emergency_address'])? encryptString($input['emergency_address']):'';
                    $emergencyContact->relationship = !empty($input['emergency_relationship'])? encryptString($input['emergency_relationship']):'';
                    $emergencyContact->save();
                }

                $oauth_user = $this->oauth_user::where(['user_fk'=>$id])
                ->update([
                    'username'  => encryptString($request->email),
                    'email'     => encryptString($request->email),
                    'mobile'    => encryptString($request->phone),
                ]);
                DB::commit();
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROFILE_UPDATED'), 'records' =>$save]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerCount()
    {
        try {

            $customersActive = customer::where(['register_step'=>4,'register_status'=>'completed','status'=>'active'])->count();
            $customersInactive = customer::where(['register_step'=>4,'register_status'=>'completed','status'=>'inactive'])->count();

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' =>  config('service.CUSTOMER_API.MESSAGE.CUSTOMER_COUNT'), 'customersActive' => $customersActive , 'customersInactive' => $customersInactive]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerCheckEmailPhoneExist(Request $request)
    {
        $message = array(
            'email.required' => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED'),
            'phone.required' => config('service.CUSTOMER_API.VALIDATION.PHONE_REQUIRED'),
            'email.unique' => config('service.CUSTOMER_API.VALIDATION.EMAIL_UNIQUE'),
            'phone.unique' => config('service.CUSTOMER_API.VALIDATION.PHONE_UNIQUE')
        );
        $validator = Validator::make($request->all(), [
            //'email' => 'required|email|max:100|unique:oauth_users',
            'email' => 'required|max:100|unique:oauth_users',
            //'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:oauth_users,mobile',
            'phone' => 'required|unique:oauth_users,mobile',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()]);
        }else{
            return response()->json(['success' => true,'message' => 'continue the registering process.']);
        }
    }


    public function verifyTwoFactor(Request $request)
    {
        $this->validate($request, [
            'email' => 'required',
        ]);

        try{
            $input    = $request->all();
            $twoFactorCode = $input['two_factor_code'];
            $email  = $input['email'];
            if(!empty($email) && !empty($twoFactorCode))
            {
                $customer = Customer::with(['plan'])->where(['email'=>$email,'two_factor_code'=>$twoFactorCode])
                ->first();
                if($customer)
                {
                    $codeTime = $customer->two_factor_expires_at;
                    $validityTime = strtotime(date("Y-m-d H:i:s"));
                    if(($validityTime - $codeTime) > config('service.CUSTOMER_API.CUSTOMER_OTP_EXPIRE_TIME'))
                    {
                        $customer->resetTwoFactorCode();
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.OTP_EXPIRE')]);
                    }
                    else
                    {
                        $customer->is_blocked = "no";
                        $customer->login_attempt = null;
                        $customer->login_attempt_time = null;
                        $customer->save();

                        $customer->resetTwoFactorCode();

                        $response =  (new OauthServerController)->handleAccessTokenBruteForce();
                        unset($response['scope']);

                        $response['first_name']         =  $customer->first_name;
                        $response['last_name']          =  $customer->last_name;
                        $response['birth_date']         =  $customer->birth_date;
                        $response['gender']             =  $customer->gender;
                        $response['phone']              =  $customer->phone;
                        $response['email']              =  $customer->email;
                        $response['register_status']    =  $customer->register_status;
                        $response['register_step']      =  $customer->register_step;
                        $response['email_confirmed']    =  $customer->email_confirmed;
                        $response['image']              = $customer->image;
                        $response['two_factor']         = $customer->two_factor;
                        $response['plan']               = $customer->plan;
                        $response['current_provider_id'] = $customer->current_provider_id;

                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_LOGIN'), 'records' => $response]);
                    }
                }
                else
                {
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.WRONG_OTP')]);

                }
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerRegisterStep1And2(Request $request)
    {
        //has_scope('customers.write');
        $message = array(
            'first_name.required'   => config('service.CUSTOMER_API.VALIDATION.FIRST_NAME_REQUIRED'),
            'last_name.required'    => config('service.CUSTOMER_API.VALIDATION.LAST_NAME_REQUIRED'),
            'birth_date.required'   => config('service.CUSTOMER_API.VALIDATION.BIRTH_DATE_REQUIRED'),
            'birth_date.date'       => config('service.CUSTOMER_API.VALIDATION.BIRTH_DATE_FORMAT'),
            'gender.required'       => config('service.CUSTOMER_API.VALIDATION.GENDER_REQUIRED'),
            'email.required'        => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED'),
            'email.unique'          => config('service.CUSTOMER_API.VALIDATION.EMAIL_UNIQUE'),
            'phone_code.required'   => config('service.CUSTOMER_API.VALIDATION.PHONE_CODE_REQUIRED'),
            'phone.required'        => config('service.CUSTOMER_API.VALIDATION.PHONE_REQUIRED'),
            'phone.unique'          => config('service.CUSTOMER_API.VALIDATION.PHONE_UNIQUE'),
            'address.required'      => config('service.CUSTOMER_API.VALIDATION.ADDRESS_REQUIRED'),
            'relationship_id.required'   => config('service.CUSTOMER_API.VALIDATION.RELATIONSHIP_REQUIRED'),
            'password.required'   => config('service.CUSTOMER_API.VALIDATION.PASSWORD_REQUIRED'),
            'password.regex'        => config('service.CUSTOMER_API.VALIDATION.PASSWORD_REGEX'),
            'confirm_password.required'        => config('service.CUSTOMER_API.VALIDATION.CONFIRM_PASSWORD_REQUIRED'),
            'confirm_password.same'        => config('service.CUSTOMER_API.VALIDATION.CONFIRM_PASSWORD_SAME'),
        );
        $validator = Validator::make($request->all(), [
            'first_name' => 'required|max:100',
            'last_name' => 'required|max:100',
            'birth_date' => 'required|date|',
            'gender' => 'required',
            //'email' => 'required|email|max:100|unique:oauth_users',
            'email' => 'required|max:100|unique:oauth_users',
            'phone_code' => 'required|max:5',
            //'phone' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:oauth_users,mobile',
            'phone' => 'required|unique:oauth_users,mobile',
            'address' => 'required|max:2000',
            'relationship_id'=>'required',
            'password' => [
                'required',
            'min:8', // must be at least 8 characters in length
            'regex:/[a-z]/', // must contain at least one lowercase letter
            'regex:/[A-Z]/', // must contain at least one uppercase letter
            'regex:/[0-9]/', // must contain at least one digit
            'regex:/[@$!%*#?&]/', // must contain a special character
        ],
        'confirm_password' => 'required|same:password',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }


        try {

            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['image']))
            {
                $fileExtension = substr(strrchr($input['image'],'.'),1);
                $allExtension = array("jpg", "jpeg", "png");
                if(!in_array($fileExtension, $allExtension))
                {
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROFILE_FILE_EXTENSION')]);
                }
            }
            $input['register_step'] = '2';
            $input['register_status'] = 'in_progress';
            $input['email_confirmed'] = 'no';
            $input['race_id'] = !empty($request->race_id) ? $request->race_id:null;
            $input['other_race'] = !empty($request->other_race) ? $request->other_race:null;
            $input['ethnicity_id'] = !empty($request->ethnicity_id) ? $request->ethnicity_id:null;
            $input['religious_id'] = !empty($request->religious_id) ? $request->religious_id:null;
            $input['password'] = $this->hashPassword($request->password);
            $customer =  Customer::create($input);

            if($customer)
            {
                $oauth = new $this->oauth_user;
                $oauth->user_fk  = $customer->id;
                $oauth->email    = $request->email;
                $oauth->username = $request->email;
                $oauth->mobile   = $request->phone;
                $oauth->password = $input['password'];
                $oauth->save();

                /* user password history */
                   $userPasswordHistory = new UserPasswordHistory();
                   $userPasswordHistory->user_id = $customer->id;
                   $userPasswordHistory->email = $customer->email;
                   $userPasswordHistory->password = $this->hashPassword($request->password);
                   $userPasswordHistory->save();
                /* user password history  */

                DB::commit();
                $response =  (new OauthServerController)->handleAccessTokenBruteForce();
                unset($response['scope']);
                $response['image']   =  $customer->image;
                $response['first_name']   =  $customer->first_name;
                $response['last_name']    =  $customer->last_name;
                $response['birth_date']   =  $customer->birth_date;
                $response['gender']   =  $customer->gender;
                $response['phone_code']   =  $customer->phone_code;
                $response['phone']   =  $customer->phone;
                $response['email']   =  $customer->email;
                $response['race_id']   =  $customer->race_id;
                $response['ethnicity_id']   =  $customer->ethnicity_id;
                $response['religious_id']   =  $customer->religious_id;
                $response['relationship_id']   =  $customer->relationship_id;

                $response['register_status'] =  $customer->register_status;
                $response['register_step']   =  $customer->register_step;
                $response['email_confirmed']   =  $customer->email_confirmed;
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_REGISTER_STEP_1'), 'records' => $response]);
            }

        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
        // This should work as well
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function hashPassword($password)
    {
        return sha1($password);
    }


    public function raceList()
    {
        try {
            //$locale = request()->header('locale');
            //App::setLocale($locale);
            //dd(App::getLocale());
            $races = Race::all();
            $data = [];
            foreach ($races as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'title' => $value->title
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.RACE_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.RACE_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function ethnicityList()
    {
        try {
            $ethnicities = Ethnicity::all();
            $data = [];
            foreach ($ethnicities as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'title' => $value->title
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.ETHNICITY_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.ETHNICITY_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function religiousList()
    {
        try {

            $religiouses = Religious::all();
            $data = [];
            foreach ($religiouses as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'title' => $value->title
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.RELIGIOUS_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.RELIGIOUS_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function  relationshipList()
    {
        try {
            $relationships = Relationship::all();
            $data = [];
            foreach ($relationships as $key => $value) {
                $data[] = [
                    'id' => $value->id,
                    'title' => $value->title
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.RELATIONSHIP_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.RELATIONSHIP_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function  intakeQuestionOptions()
    {
        try {

            $intakeQuestions = IntakeQuestion::with(['options'])->get();
            $data = [];
            foreach ($intakeQuestions as $key => $value) {
                $optionData = [];
                foreach ($value->options as $key => $option) {
                    $optionData[] = [
                        'id' => $option->id,
                        'title'=> $option->title
                    ];
                }
                $data[] = [
                    'id' => $value->id,
                    'question' => $value->title,
                    'options' => $optionData
                ];
            }
            if (empty($data)) {
                throw new Exception(config('service.INTAKE_QUESTION_OPTIONS_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.SUCCESS_STATUS'), 'message' => config('service.INTAKE_QUESTION_OPTIONS_LIST'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerRegisterStep3(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'intake_questions'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['intake_questions']))
            {
                $delete = CustomerIntakeQuestion::where('customer_id', $this->user_id)->delete();
                foreach($input['intake_questions'] as $intake_questions)
                {
                    if(empty($intake_questions['options']))
                    {
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.INTAKE_OPTION_REQ')]);
                    }
                    $intakeQuestion = new CustomerIntakeQuestion();
                    $intakeQuestion->customer_id = $this->user_id;
                    $intakeQuestion->intake_question_id =  $intake_questions['intake_question_id'];
                    $intakeQuestion->save();

                    if(!empty($intake_questions['options']))
                    {
                        foreach($intake_questions['options'] as $option)
                        {
                            $intakeQuestionAnswer = new CustomerIntakeQuestionAnswer();
                            $intakeQuestionAnswer->customer_intake_question_id  = $intakeQuestion->id;
                            $intakeQuestionAnswer->intake_question_option_id  = $option;
                            $intakeQuestionAnswer->save();
                        }
                    }
                }
            }
            $customer = Customer::find($this->user_id);
            $customer->register_step = '3';
            $customer->save();
            DB::commit();
            $data = CustomerIntakeQuestion::with(['intakeQuestionAnswer'=>function($query) {
                $query->select('*');
            }])
            ->where(['customer_id'=>$this->user_id])
            ->get();

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_REGISTER_STEP_3'), 'records' => $data]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerRegisterStep4(Request $request)
    {
        //has_scope('customers.write');
        $message = array(
            'provider_preference.required' => config('service.CUSTOMER_API.VALIDATION.PROVIDER_PREFERENCE_REQUIRED'),
            'provider_preference.in' => config('service.CUSTOMER_API.VALIDATION.PROVIDER_PREFERENCE_IN'),
        );
        $validator = Validator::make($request->all(), [
            'provider_preference'  => 'required|in:male,female,no_preference',
        ],$message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            $input = $request->all();
            $token   = Str::random(60);
            $customer = Customer::find($this->user_id);
            $customer->provider_preference = $input['provider_preference'];
            $customer->register_step = '4';
            $customer->register_status = 'completed';
            $customer->email_confirm_token = $token;

            if($customer->save())
            {
                /* Notification setting */
                   $this->notificationSetting($customer->id);
                /* Notification setting  */

                $templateName = "Consumer welcome email";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                $greeting = 'Hi ' .decryptString($customer->first_name)." ".decryptString($customer->last_name).',';
                if($templateData)
                {
                    /* Welcome email */
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                        dispatch($sendMail);
                    /* End welcome email  */
                }

                $templateName = "Consumer account verification";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Confirm email */
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'admin/customer-confirm-email/'. $token;
                    $sendMail = new CustomerConfirmEmailJob(decryptString($customer->email), $greeting, $body, $link, $type);
                    dispatch($sendMail);
                    /* End confirm email  */
                }

                $templateName = "New consumer";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    $admin = User::where(['id'=>1])->first();
                    /* new customer email to admin */
                    $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'admin/customers/';
                    $sendMail = new CustomerRegisteredEmailJob($admin->email, $greeting, $body, $link, $type);
                    dispatch($sendMail);
                    /* End email  */
                }

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->register($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_REGISTER_STEP_4'), 'records' =>$customer->provider_preference]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function notificationSetting($customerId)
    {
        $updateSetting = NotificationSetting::updateOrCreate([
            'customer_id'   => $customerId,
        ],[
            'customer_id'  => $customerId,
            'show_notification' => 'active',
            'appointment_reminder' => 'active',
            'call_notification' => 'active',
            'new_message_notification' => 'active',
        ]);
        if($updateSetting){
            return true;
        }
    }


    public function customerConfirmEmail(Request $request)
    {
        try {
            $token = $request->token;
            $customer = Customer::where(['email_confirm_token'=>$token])->first();
            if($customer)
            {
                $customer->email_confirmed = "yes";
                $customer->email_confirm_token = null;
                $customer->save();
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_EMAIL_VERIFIED')]);
            }
            else
            {
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.TOKEN_EXPIRED')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function forgotPasswordOtp(Request $request)
    {
        $message = array(
            'email.exists'   => config('service.CUSTOMER_API.VALIDATION.EMAIL_EXISTS'),
            'phone.exists'   => config('service.CUSTOMER_API.VALIDATION.PHONE_EXISTS'),
            'email.required_without'   => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED_WITHOUT'),
        );
        $validator = Validator::make($request->all(), [
            'email' => 'required_without:phone|exists:customers',
            'phone' => 'required_without:email|exists:customers',
        ],$message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try{
            $digits = 4;
            $otpCode = rand(pow(10, $digits-1), pow(10, $digits)-1);
            if(!empty($request->email))
            {
                $customer = Customer::where(['email'=>$request->email])->first();
            }
            if(!empty($request->phone))
            {
                $customer = Customer::where(['phone'=>$request->phone])->first();
            }
            if(!empty($customer))
            {
                $customer->otp = $otpCode;
                $customer->otp_validity = strtotime(date("Y-m-d H:i:s"));
                $customer->save();

                //send otp
                $greeting = config('service.CUSTOMER_API.MESSAGE.DEAR').decryptString($customer->first_name) . ',';
                $body     = config('service.CUSTOMER_API.MESSAGE.EMAIL_OTP')." ".$otpCode;
                $type     = 'OTP';

                //customer mail
                $sendMail = new CustomerForgotPasswordMailJob(decryptString($customer->email), $greeting, $body, $type);
                dispatch($sendMail);
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.EMAIL_OTP_SENT')]);
            } else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.ARCHIVED_CUSTOMER')]);
            }
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }


    public function verifyForgotPasswordOtp(Request $request)
    {
        $message = array(
            'otp.required'   => config('service.CUSTOMER_API.VALIDATION.OTP_REQUIRED'),
            'email.required_without'   => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED_WITHOUT'),
        );
        $validator = Validator::make($request->all(), [
            'otp' => 'required',
            'email' => 'required_without:phone',
            'phone' => 'required_without:email',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try {

            if(!empty($request->email) && !empty($request->otp))
            {
                $customer = Customer::where(['email'=>$request->email,'otp'=>$request->otp])->first();

            } 
            if(!empty($request->phone) && !empty($request->otp))
            {
                $customer = Customer::where(['phone'=>$request->phone,'otp'=>$request->otp])->first();

            } 
              
            if($customer)
            {
                $otpValidity = $customer->otp_validity;
                $currentDateTime = strtotime(date('Y-m-d h:i:s'));

                if(($currentDateTime - $otpValidity) > config('service.CUSTOMER_API.FORGOT_PASS_OTP_EXPIRE_TIME')){
                    $customer->otp = null;
                    $customer->otp_validity = null;
                    $customer->save();
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.OTP_EXPIRED')]);
                }else{
                    $customer->otp = null;
                    $customer->otp_validity = null;
                    $customer->save();
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.OTP_VERIFY')]);
                }
            }
            else
            {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.INVALID_OTP')]);
            }

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function forgotPassword(Request $request)
    {
        $message = array(
            'email.required_without'   => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED_WITHOUT'),
            'password.required'   => config('service.CUSTOMER_API.VALIDATION.PASSWORD_REQUIRED'),
            'password.regex'        => config('service.CUSTOMER_API.VALIDATION.PASSWORD_REGEX'),
            'password_confirmation.required'        => config('service.CUSTOMER_API.VALIDATION.CONFIRM_PASSWORD_REQUIRED'),
            'password_confirmation.same'        => config('service.CUSTOMER_API.VALIDATION.CONFIRM_PASSWORD_SAME'),
        );
        $validator = Validator::make($request->all(), [
            'email' => 'required_without:phone',
            'phone' => 'required_without:email',
            'password' => [
            'required',
            'min:8', // must be at least 8 characters in length
            'max:30', // must be max 30 characters in length
            'regex:/[a-z]/', // must contain at least one lowercase letter
            'regex:/[A-Z]/', // must contain at least one uppercase letter
            'regex:/[0-9]/', // must contain at least one digit
            'regex:/[@$!%*#?&]/', // must contain a special character
        ],
            'password_confirmation' => 'required|same:password',
        ],$message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try
        {
            if(!empty($request->email)){
                $customer = Customer::where('email', $request->email)->first();
            }
            if(!empty($request->phone)){
                $customer = Customer::where('phone', $request->phone)->first();
            }
            if(!$customer) {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS') , 'message' => config('service.CUSTOMER_API.MESSAGE.INVALID_USER') ]);
            }

            $oldPassword = UserPasswordHistory::where(
                                [
                                    'user_id' => $customer->id,
                                    'email' => $customer->email
                                ]
                           )
                           ->orderBy('id', 'desc')
                           ->take(config('service.LAST_PASSWORD_COUNT'))
                           ->pluck('password')
                           ->toArray();
            if(count($oldPassword)){
                foreach ($oldPassword as $key => $password) {
                    if (sha1($request->password) == $password) {
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.OLD_PASSWORD_CHECK')]);
                    }
                }
            }
            DB::beginTransaction();
            $customer->password = sha1($request->password);
            $customer->save();
            $save_data['password'] = sha1($request->password);
            $result = $this->oauth_user::where('user_fk', $customer->id)
            ->update($save_data);

            /* user password history */
               $userPasswordHistory = new UserPasswordHistory();
               $userPasswordHistory->user_id = $customer->id;
               $userPasswordHistory->email = $customer->email;
               $userPasswordHistory->password = sha1($request->password);
               $userPasswordHistory->save();
            /* user password history  */
            DB::commit();

            $templateName = "Consumer password reset";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $greeting = 'Hi ' .decryptString($customer->first_name).' '.decryptString($customer->last_name).',';
                $body     = $templateData->text;
                $type     = $templateData->title;

                $sendMail = new CustomerMailJob(decryptString($customer->email), $greeting, $body, $type);
                dispatch($sendMail);
            }

            /* Push and db notification */
            $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
            if($notificationSetting->show_notification == 'active'){
                $customerId = $customer->id;
                (new CustomerNotificationController)->forgotPassword($customerId);
            }
            /* End push and db notification */

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS') , 'message' => config('service.CUSTOMER_API.MESSAGE.PASSWORD_UPDATED') ]);
        }
        catch(Exception $e)
        {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage()]);
        }
        catch(Throwable $e)
        {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS') , 'message' => $e->getMessage()]);
        }
    }


    public function changePassword(Request $request)
    {
        $message = array(
            'current_password.required'   => config('service.CUSTOMER_API.VALIDATION.CURRENT_PASSWORD_REQUIRED'),
            'new_password.required'        => config('service.CUSTOMER_API.VALIDATION.NEW_PASSWORD_REQUIRED'),
            'new_password.regex'        => config('service.CUSTOMER_API.VALIDATION.PASSWORD_REGEX'),
            'confirm_password.same'        => config('service.CUSTOMER_API.VALIDATION.CHANGE_CONFIRM_PASSWORD_SAME')
        );

        $validator = Validator::make($request->all(), [
            'current_password' => 'required',
            'new_password' => [
            'required',
            'min:8', // must be at least 8 characters in length
            'regex:/[a-z]/', // must contain at least one lowercase letter
            'regex:/[A-Z]/', // must contain at least one uppercase letter
            'regex:/[0-9]/', // must contain at least one digit
            'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            'confirm_password' => 'required|same:new_password',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
                $currentPassword = $request->current_password;
                $currentPasswordEncrypted = $this->hashPassword($currentPassword);
                $customer = Customer::where('password', $currentPasswordEncrypted)
                ->where('id', $this->user_id)
                ->first();
                if (!isset($customer)) {
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => config('service.CUSTOMER_API.MESSAGE.WRONG_CURRENT_PASSWORD')],200);
                }
                $newPassword = $this->hashPassword($request->new_password);
                if ($newPassword == $customer->password) {
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CURRENT_PASSWORD')]);
                }
                $oldPassword = UserPasswordHistory::where(
                                    [
                                        'user_id' => $this->user_id,
                                        'email' => $customer->email
                                    ]
                               )
                               ->orderBy('id', 'desc')
                               ->take(config('service.LAST_PASSWORD_COUNT'))
                               ->pluck('password')
                               ->toArray();
                if(count($oldPassword)){
                    foreach ($oldPassword as $key => $password) {
                        if ($newPassword == $password) {
                            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.OLD_PASSWORD_CHECK')]);
                        }
                    }
                }   
                DB::beginTransaction();
                $customer = Customer::find($this->user_id);
                $customer->password = $newPassword;
                $customer->save();
                $save_data['password'] = $newPassword;
                $result = DB::table('oauth_users')
                ->where('user_fk', $this->user_id)
                ->update($save_data);

                /* user password history */
                $userPasswordHistory = new UserPasswordHistory();
                $userPasswordHistory->user_id = $customer->id;
                $userPasswordHistory->email = $customer->email;
                $userPasswordHistory->password = $newPassword;
                $userPasswordHistory->save();
                /* user password history  */

                DB::commit();
                /*$greeting = 'Hi ' . $customer->first_name . '!';
                $body     = ' Your password has been changed successfully!';
                $type     =  'Password Updated';
                $sendMail = new SendMailJob($customer->email, $greeting, $body, $type);
                dispatch($sendMail);*/

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->changePassword($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PASSWORD_UPDATED')]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []], 400);
        }
    }


    public function profile()
    {
        try {

            $customer = Customer::with(['relationshipStatus','race','ethnicity','religious','emergencyContact'=>function($query){
                $query->select('id','customer_id','first_name','last_name','email','phone_no','address','relationship');
            }])
            ->where(['id'=>$this->user_id])
            ->first();
            if (empty($customer)) {
                throw new Exception(config('service.CUSTOMER_API.MESSAGE.CUSTOMER_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_DETAILS'), 'records' => $customer]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function updateCustomerProfile(Request $request)
    {
        $message = array(
            'first_name.required'   => config('service.CUSTOMER_API.VALIDATION.FIRST_NAME_REQUIRED'),
            'last_name.required'    => config('service.CUSTOMER_API.VALIDATION.LAST_NAME_REQUIRED'),
            'birth_date.required'   => config('service.CUSTOMER_API.VALIDATION.BIRTH_DATE_REQUIRED'),
            'birth_date.date'       => config('service.CUSTOMER_API.VALIDATION.BIRTH_DATE_FORMAT'),
            'gender.required'       => config('service.CUSTOMER_API.VALIDATION.GENDER_REQUIRED'),
            'email.required'        => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED'),
            'email.unique'          => config('service.CUSTOMER_API.VALIDATION.EMAIL_UNIQUE'),
            'phone_code.required'   => config('service.CUSTOMER_API.VALIDATION.PHONE_CODE_REQUIRED'),
            'phone.required'   => config('service.CUSTOMER_API.VALIDATION.PHONE_REQUIRED'),
            'phone.unique'   => config('service.CUSTOMER_API.VALIDATION.PHONE_UNIQUE'),
            'address.required'   => config('service.CUSTOMER_API.VALIDATION.ADDRESS_REQUIRED'),
            'relationship_id.required'   => config('service.CUSTOMER_API.VALIDATION.RELATIONSHIP_REQUIRED'),
        );
        $validator = Validator::make($request->all(), [
            'first_name' => 'required|max:100',
            'last_name' => 'required|max:100',
            'birth_date' => 'required|date|',
            //'height' => 'required|max:8',
            //'weight' => 'required',
            'relationship_id' => 'required',
            'gender' => 'required',
            //'email' => "required|email|unique:customers,email,$this->user_id",
            //'email' => "required|unique:customers,email,$this->user_id",
            'phone_code' => 'required|max:5',
            //'phone' => "required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10|unique:customers,phone,$this->user_id",
            'phone' => "required|unique:customers,phone,$this->user_id",
            'address' => 'required|max:2000',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['image']))
            {
                $fileExtension = substr(strrchr($input['image'],'.'),1);
                $allExtension = array("jpg", "jpeg", "png");
                if(!in_array($fileExtension, $allExtension))
                {
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROFILE_FILE_EXTENSION')]);
                }
                $image = $input['image'];
            }
            if($input['birth_date'] > date("Y-m-d"))
            {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_BIRTH_DATE')]);
            }
            $customer =  Customer::find($this->user_id);
            if (!empty($image)) {
                $customer->image = $image;
            }
            $customer->first_name = $input['first_name'];
            $customer->last_name  = $input['last_name'];
            $customer->birth_date = $input['birth_date'];
            $customer->height_feet = !empty($input['height_feet']) ? $input['height_feet']:'';
            $customer->height_inch = !empty($input['height_inch']) ? $input['height_inch']:'';
            $customer->weight = !empty($input['weight']) ? $input['weight']:'';
            $customer->relationship_id = $input['relationship_id'];
            $customer->gender = $input['gender'];
            $customer->other_gender = !empty($input['other_gender']) ? $input['other_gender']:'';
            $customer->other_race = !empty($input['other_race']) ? $input['other_race']:null;
            $customer->race_id = !empty($input['race_id']) ? $input['race_id']:null;
            $customer->email = $input['email'];
            $customer->phone_code = $input['phone_code'];
            $customer->phone = $input['phone'];
            if(!empty($input['timezone'])){
                $customer->timezone = $input['timezone'];
            }
            $customer->address = $input['address'];
            $customer->apartment = !empty($input['apartment']) ? $input['apartment']:null;
            $customer->suite_number = !empty($input['suite_number']) ? $input['suite_number']:null;
            $customer->ethnicity_id = !empty($input['ethnicity_id']) ? $input['ethnicity_id']:null;
            $customer->religious_id = !empty($input['religious_id']) ? $input['religious_id']:null ;
            $customer->last_updated_at = date("Y-m-d H:i:s");
            $customer = $customer->save();
            if($customer)
            {
                $oauth = $this->oauth_user::where(['user_fk'=>$this->user_id])
                ->update(['username'=>$request->email,'email'=>$request->email,'mobile'=>$request->phone]);
                DB::commit();
                $customer = Customer::find($this->user_id);
                $response['image']   =  $customer->image;
                $response['first_name']   =  $customer->first_name;
                $response['last_name']    =  $customer->last_name;
                $response['email']    =  $customer->email;
                $response['phone_code']   =  $customer->phone_code;
                $response['phone']   =  $customer->phone;
                $response['address']   =  $customer->address;
                $response['birth_date']   =  $customer->birth_date;
                $response['gender']   =  $customer->gender;
                $response['other_gender']   =  $customer->other_gender;
                $response['height_feet']   =  $customer->height_feet;
                $response['height_inch']   =  $customer->height_inch;
                $response['weight']   =  $customer->weight;

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->updateCustomerProfile($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_UPDATE_PROFILE'), 'records' => $response]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function recommendedWithOtherProviders()
    {
        try {

            $customerIntakeQuestionAnswers = CustomerIntakeQuestion::with(['intakeQuestionAnswer'=>function($query){
                $query->select('*');
            }])
            ->where(['customer_id'=>$this->user_id,'intake_question_id'=>1])
            ->first();
            if($customerIntakeQuestionAnswers){
                /* Fetch all answers id given by customer */
                $customerAnswersIds = $customerIntakeQuestionAnswers->intakeQuestionAnswer->pluck('intake_question_option_id');
                /* Fetch all answers id given by customer */

                /* Relation mapping data upon answers and specialization */
                $specializationsIds = IntakeQuestionOptionSpecialization::whereIn('intake_question_option_id',$customerAnswersIds)->pluck('specialization_id');
                /* Relation mapping data upon answers and specialization */

                /* Providers list associated with specialization */
                $providersIds  = ProviderSpecialization::whereIn('specialization_id',$specializationsIds)
                ->groupBy('provider_id')
                ->pluck('provider_id')
                ->toArray();
                /* Providers list associated with specialization */

                $customer = Customer::find($this->user_id);
                $providerLanguages = ProviderLanguage::where('language_id', $customer->language_id)->pluck('provider_id');

                /* recommended provider */
                $providers['recommended_providers'] = Provider::with([
                    'languages'=>function($query){
                        $query->select('*');
                    },'specializations'
                ])
                ->select('id','first_name','last_name','email','birth_date','gender','phone_code','phone','address','experience','bio','image','rating','reviews','next_available_slot','other_language','other_specialization')
                ->where(['register_step'=>5,'register_status'=>'completed','application_status'=>'Approved','status'=>'active'])
                ->where('gender',$customer->provider_preference)
                ->orWhereIn('id',$providerLanguages)
                ->WhereIn('id',$providersIds)
                //->orderBy('rating','desc')
                ->take(5)
                ->get();
                $recommendedProviders = $providers['recommended_providers']->pluck('id')->toArray();
                //dd($recommendedProviders);
                /* end recommended provider */

                /* All other providers  */
                $providerPreference = $customer->provider_preference;
                if($providerPreference=="male"){
                    $condition = 'FIELD(gender, "male", "female", "other")';
                }
                if($providerPreference=="female"){
                    $condition = 'FIELD(gender, "female", "male", "other")';
                }
                if($providerPreference=="no_preference"){
                    $condition = 'FIELD(gender, "male", "female", "other")';
                }
                $finalProvider=array_diff($providersIds,$recommendedProviders);
                $providers['all_other_providers'] = Provider::with([
                    'languages'=>function($query){
                        $query->select('*');
                    },'specializations'
                ])
                ->select('id','first_name','last_name','email','birth_date','gender','phone_code','phone','address','experience','bio','image','rating','reviews','next_available_slot','other_language','other_specialization')
                //->whereIn('id',$providerLanguages)
                ->where(['register_step'=>5,'register_status'=>'completed','application_status'=>'Approved','status'=>'active'])
                //->WhereIn('id', $finalProvider)
                ->whereNotIn('id', $recommendedProviders)
                ->orderByRaw($condition)
                //->orderBy('rating','desc')
                ->paginate(10);
                 /* All other providers  */

                if(!count($providers['recommended_providers']) && !count($providers['all_other_providers'])){
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_NOT_FOUND')]);
                }
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.RECOMMENDDED_PROVIDERS'), 'records' => $providers]);

            }

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage()]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage()]);
        }
    }


    public function recommendedProvidersList(Request $request)
    {
        try {

            $customerIntakeQuestionAnswers = CustomerIntakeQuestion::with(['intakeQuestionAnswer'=>function($query) {
                $query->select('*');
            }])
            ->where(['customer_id'=>$this->user_id,'intake_question_id'=>1])
            ->first();
            if($customerIntakeQuestionAnswers){
                /* Fetch all answers id given by customer */
                $customerAnswersIds = [];
                foreach ($customerIntakeQuestionAnswers->intakeQuestionAnswer as $key => $customerIntakeQuestionAnswer)
                {
                    $customerAnswersIds[] = $customerIntakeQuestionAnswer->intake_question_option_id;
                }
                /* Fetch all answers id given by customer */
                /* Relation mapping data upon answers and specialization */
                $specializationsIds = IntakeQuestionOptionSpecialization::whereIn('intake_question_option_id',$customerAnswersIds)->pluck('specialization_id');
                /* Relation mapping data upon answers and specialization */

                /* Providers list associated with specialization */
                $providersIds  = ProviderSpecialization::whereIn('specialization_id',$specializationsIds)
                ->groupBy('provider_id')
                ->pluck('provider_id');
                /* Providers list associated with specialization */
                $customer = Customer::find($this->user_id);
                $providers = Provider::with([
                    'languages'=>function($query){
                        $query->select('*');
                    }
                ])
                ->select('id','first_name','last_name','email','birth_date','gender','phone_code','phone','address','experience','bio','image','rating','reviews')
                ->where(function ($q) use ($request) {
                // search
                    if($request->filled('provider_name')) {
                        $q->where('first_name', 'like', '%'.$request->provider_name.'%');
                        $q->orwhere("last_name",'like', '%'.$request->provider_name.'%');
                    }
                })
                ->whereIn('id',$providersIds)
                ->where('gender',$customer->provider_preference)
                ->orderBy('rating','desc')
                ->orderBy('experience','desc')
                ->paginate(10);

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.RECOMMENDDED_PROVIDERS'), 'records' => $providers]);

            }

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function otherProvidersList(Request $request)
    {
        try {

            $customerIntakeQuestionAnswers = CustomerIntakeQuestion::with(['intakeQuestionAnswer'=>function($query) {
                $query->select('*');
            }])
            ->where(['customer_id'=>$this->user_id,'intake_question_id'=>1])
            ->first();
            if($customerIntakeQuestionAnswers){
                /* Fetch all answers id given by customer */
                $customerAnswersIds = [];
                foreach ($customerIntakeQuestionAnswers->intakeQuestionAnswer as $key => $customerIntakeQuestionAnswer)
                {
                    $customerAnswersIds[] = $customerIntakeQuestionAnswer->intake_question_option_id;
                }
                /* Fetch all answers id given by customer */
                /* Relation mapping data upon answers and specialization */
                $specializationsIds = IntakeQuestionOptionSpecialization::whereIn('intake_question_option_id',$customerAnswersIds)->pluck('specialization_id');
                /* Relation mapping data upon answers and specialization */
                /* Providers list associated with specialization */
                $providersIds  = ProviderSpecialization::whereIn('specialization_id',$specializationsIds)
                ->groupBy('provider_id')
                ->pluck('provider_id');
                /* Providers list associated with specialization */
                $providers = Provider::with([
                    'languages'=>function($query){
                        $query->select('*');
                    }
                ])
                ->select('id','first_name','last_name','email','birth_date','gender','phone_code','phone','address','experience','bio','image','rating','reviews')
                ->where(function ($q) use ($request) {
                // search
                    if($request->filled('provider_name')) {
                        $q->where('first_name', 'like', '%'.$request->provider_name.'%');
                        $q->orwhere("last_name",'like', '%'.$request->provider_name.'%');
                    }
                })
                ->whereNotIn('id', $providersIds)
                ->where([
                    'register_step'=>5,
                    'register_status'=>'completed'
                ])
                ->orderBy('rating','desc')
                ->orderBy('experience','desc')
                ->paginate(10);

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.OTHER_PROVIDERS_LIST'), 'records' => $providers]);

            }

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function searchProviderList(Request $request)
    {
        try {
            if(!empty($request->name))
            {
                $pieces = explode(" ", $request->name);
                if(count($pieces)){
                    $fName = !empty($pieces[0]) ? $pieces[0] : $request->name; 
                    $lName = !empty($pieces[1]) ? $pieces[1] : $request->name; 
                }
                
                $providers = Provider::with([
                    'languages'=>function($query){
                        $query->select('*');
                    },
                    'certifications'=>function($query){
                        $query->select('*');
                    },
                    'specializations'=>function($query){
                        $query->select('*');
                    },
                ])
                ->select('id','first_name','last_name','email','birth_date','gender','phone_code','phone','address','experience','bio','image','rating','reviews','next_available_slot','other_language')
                ->where('first_name', 'like', '%'.$request->name.'%')
                ->orwhere("last_name",'like', '%'.$request->name.'%')
                ->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%'.$fName." ".$lName.'%')
                ->orWhere(DB::raw('CONCAT(last_name, " ",first_name)'), 'like', '%'.$fName." ".$lName.'%')
                ->where([
                    'register_step'=>5,
                    'register_status'=>'completed'
                ])
                ->orderBy('rating','desc')
                ->orderBy('experience','desc')
                ->paginate(10);

                if ($providers->count()){
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_DETAILS'), 'records' => $providers]);
                }
                else{
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOT_FOUND')]);
                }
            }

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function bookSession(Request $request)
    {
        $message = array(
            'provider_id.required'   => config('service.CUSTOMER_API.VALIDATION.PROVIDER_ID_REQUIRED'),
            'date.required'   => config('service.CUSTOMER_API.VALIDATION.DATE_REQUIRED'),
            'date.date'       => config('service.CUSTOMER_API.VALIDATION.DATE_FORMAT'),
        );
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
            'date'         => 'required|date',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try {
            $input = $request->all();
            $result['provider'] = Provider::select('id','first_name','last_name','experience','image','rating','reviews')->where(['id'=>$input['provider_id']])->first();
            $day  = date('l', strtotime($input['date']));
            $availabilityLists = ProviderAvailability::where(['provider_id'=>$input['provider_id'],'day'=>$day])->get();
            $fData['f_half']= [];
            $sData['s_half']= [];
            $appDate = dateFormat($input['date']);
            if($availabilityLists->count()){
                foreach ($availabilityLists as $key => $value)
                {
                    if($value->type=="first_half")
                    {
                        $startTime = strtotime(convertTimeUtcToLocal($value->start_time,request()->customer_timezone));
                        $endTime    = strtotime(convertTimeUtcToLocal($value->end_time,request()->customer_timezone));
                        $fData['f_half'] = [];
                        for ($i = $startTime; $i < $endTime; $i+=3600) {     // 3600 second => 1 hour
                            $time = gmdate("h:i A", $i);
                            $utcTime = convertTimeLocalToUtc($time,request()->customer_timezone);
                            //dd($utcTime);
                            $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$input['provider_id'],'date'=>$appDate,'time'=> $utcTime])->first();
                            if($appointment && empty($appointment->cancell_appointment)){
                                $fData['f_half'][] =  [
                                    'time' => gmdate("h:i A", $i),
                                    'available' => 'No',
                                ];
                            }else{
                                $fData['f_half'][] =  [
                                    'time' => gmdate("h:i A", $i),
                                    'available' => 'Yes',
                                ];
                            }
                        }
                    }
                    if($value->type=="second_half")
                    {
                        $startTime = strtotime(convertTimeUtcToLocal($value->start_time,request()->customer_timezone));
                        //dd(date("h:i A",$startTime));
                        $endTime    = strtotime(convertTimeUtcToLocal($value->end_time,request()->customer_timezone));
                        $sData['s_half'] = [];
                        for ($i = $startTime; $i < $endTime; $i+=3600) {     // 3600 second => 1 hour
                            $time = gmdate("h:i A", $i);
                            $utcTime = convertTimeLocalToUtc($time,request()->customer_timezone);
                            $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$input['provider_id'],'date'=>$appDate,'time'=>$utcTime])->first();
                            if($appointment && empty($appointment->cancell_appointment)){
                                $sData['s_half'][] =  [
                                    'time' => gmdate("h:i A", $i),
                                    'available' => 'No',
                                ];
                               
                            }else{
                                $sData['s_half'][] =  [
                                    'time' => gmdate("h:i A", $i),
                                    'available' => 'Yes',
                                ];
                            }
                        }
                    }
                }
            }
            //dd($fData['f_half']);
            $data['first_half_leave'] = [];
            foreach ($fData['f_half'] as $key => $first_half) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $input['provider_id']])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($appDate >= $startDate)  && ($appDate <= $endDate))
                    {
                        if((strtotime($appDate." ".$first_half['time']) >= strtotime($appDate." ".$value->start_time) && strtotime($appDate." ".$first_half['time']) < strtotime($appDate." ".$value->end_time)))
                        {
                            $data['first_half_leave'][] =  [
                                'time' => $first_half['time'],
                                'available' => $first_half['available']
                            ];
                        }
                    }
                }
            }
            $data['second_half_leave'] = [];
            foreach ($sData['s_half'] as $key => $second_half) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $input['provider_id']])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($appDate >= $startDate)  && ($appDate <= $endDate))
                    {
                        if((strtotime($appDate." ".$second_half['time']) >= strtotime($appDate." ".$value->start_time) && strtotime($appDate." ".$second_half['time']) < strtotime($appDate." ".$value->end_time)))
                        {
                            $data['second_half_leave'][] =  [
                                'time' => $second_half['time'],
                                'available' => $second_half['available']
                            ];
                        }
                    }
                }
            }
            $arrayFirstHalf = $fData['f_half'];
            $arrayFirstHalfLeave = $data['first_half_leave'];
            $arraySecondHalf = $sData['s_half'];
            $arraySecondHalfLeave = $data['second_half_leave'];
            $result['first_half'] = checkFirstHalfLeave($arrayFirstHalf, $arrayFirstHalfLeave);
            $result['second_half'] = checkSecondHalfLeave($arraySecondHalf, $arraySecondHalfLeave);

            // ratin reviews
            $result['rating_review'] = ProviderRatingReview::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$input['provider_id'],'published'=>'active'])
            ->get();

            // total rating review
            $result['total_rating_review'] = $result['rating_review']->count();

            // average rating
            $result['avg_rating_review'] = $result['rating_review']->avg('session_rating');

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_AVAILABLE_TIME'), 'records' => $result]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function bookSessionV2(Request $request)
    {
        $message = array(
            'provider_id.required'   => config('service.CUSTOMER_API.VALIDATION.PROVIDER_ID_REQUIRED'),
            'date.required'   => config('service.CUSTOMER_API.VALIDATION.DATE_REQUIRED'),
            'date.date'       => config('service.CUSTOMER_API.VALIDATION.DATE_FORMAT'),
        );
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
            'date'         => 'required|date',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try {
            $input = $request->all();
            $result['provider'] = Provider::select('id','first_name','last_name','experience','image','rating','reviews')->where(['id'=>$input['provider_id']])->first();
            
            $prev_day  = date('Y-m-d', strtotime($input['date']." -1 day"));
            $curr_day  = date('Y-m-d', strtotime($input['date']));
            $next_day  = date('Y-m-d', strtotime($input['date']." +1 day"));

            $dates = [$prev_day, $curr_day, $next_day];

            $availableSots = [];
            foreach ($dates as $date) {
                $day  = date('l', strtotime($date));
                $slots = ProviderSlot::where(['provider_id' => $input['provider_id'], 'day' => $day])->get();
                if($slots->count()){
                    foreach ($slots as $key => $value)
                    {     
                        $datetime = $date." ".$value->slot;
                        /* convert data time utc to local  */
                        $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, request()->customer_timezone));
                        /* convert data time utc to local  */

                        $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$input['provider_id'],'date'=>$date,'time'=> $value->slot])->orderBy('id','desc')->first();
                        if($appointment && empty($appointment->cancell_appointment)){
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'time' => date("h:i A", $slotDateTime),
                                'available' => 'No',
                            ];
                        }else{
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'time' => date("h:i A", $slotDateTime),
                                'available' => 'Yes',
                            ];
                        }
                    }
                }
            }

            $allSlots = !empty($availableSots[$curr_day]) ? $availableSots[$curr_day] : [];

            /* provider applied day off  */
            $leaves = [];
            foreach ($allSlots as $key => $slot) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $input['provider_id']])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($curr_day >= $startDate)  && ($curr_day <= $endDate))
                    {
                        if((strtotime($curr_day." ".$slot['time']) >= strtotime($curr_day." ".$value->start_time) && strtotime($curr_day." ".$slot['time']) < strtotime($curr_day." ".$value->end_time)))
                        {
                            $leaves[] =  [
                                'time' => $slot['time'],
                                'available' => $slot['available']
                            ];
                        }
                    }
                }
            }
            /* provider applied day off  */

            $finalSlots = checkLeave($allSlots, $leaves);

            $result['first_half']= [];
            $result['second_half']= [];
            foreach($finalSlots as $key => $value){
                if(strtotime($value['time']) <= strtotime(date("12:00 PM"))){
                    $result['first_half'][] =  [
                        'time' => $value['time'],
                        'available' => $value['available']
                    ];
                }else{
                     $result['second_half'][] =  [
                        'time' => $value['time'],
                        'available' => $value['available']
                    ];
                }
            }

            // ratin reviews
            $result['rating_review'] = ProviderRatingReview::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$input['provider_id'],'published'=>'active'])
            ->get();

            // total rating review
            $result['total_rating_review'] = $result['rating_review']->count();

            // average rating
            $result['avg_rating_review'] = number_format((float)$result['rating_review']->avg('session_rating'), 1, '.', '');

            // current active plan
            $customer = Customer::select('plan_id','appointment_session','stripe_subscription_id')->where('id',$this->user_id)->first();
            if(!empty($customer->plan_id)){
                $result['plan_active'] = 'yes';
            }else{
                $result['plan_active'] = 'no';
            }

            // remaining session
            $result['appointment_session'] = !empty($customer->appointment_session) ? $customer->appointment_session : 0;
            

            /* Check change provider request  */
                $checkRequest = ChangeProvider::where(['customer_id' => $this->user_id, 'status' => 'pending'])->first();
                if(!empty($checkRequest)){
                    $result['change_provider_request'] = "yes";
                }else{
                    $result['change_provider_request'] = "no";
                }
            /* End check provider request  */

            /* Check subscription is cancel */
                if(!empty($customer->plan_id) && empty($customer->stripe_subscription_id)){
                    $result['subscription_cancelled'] = "yes";
                }else{
                    $result['subscription_cancelled'] = "no";
                }
            /* Check subscription is cancel */

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_AVAILABLE_TIME'), 'records' => $result]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function bookSessionV3(Request $request)
    {
        $message = array(
            'provider_id.required'   => config('service.CUSTOMER_API.VALIDATION.PROVIDER_ID_REQUIRED'),
            'date.required'   => config('service.CUSTOMER_API.VALIDATION.DATE_REQUIRED'),
            'date.date'       => config('service.CUSTOMER_API.VALIDATION.DATE_FORMAT'),
        );
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
            'date'         => 'required|date',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }
        try {
            $input = $request->all();
            $result['provider'] = Provider::select('id','first_name','last_name','experience','image','rating','reviews')->where(['id'=>$input['provider_id']])->first();
            
            $prev_day  = date('Y-m-d', strtotime($input['date']." -1 day"));
            $curr_day  = date('Y-m-d', strtotime($input['date']));
            $next_day  = date('Y-m-d', strtotime($input['date']." +1 day"));

            $dates = [$prev_day, $curr_day, $next_day];

            $availableSots = [];
            foreach ($dates as $date) {
                $day  = date('l', strtotime($date));
                $slots = ProviderSlot::where(['provider_id' => $input['provider_id'], 'day' => $day])->get();
                if($slots->count()){
                    foreach ($slots as $key => $value)
                    {     
                        if(!empty($slots[$key+1])){
                            if(strtotime($value->slot) > strtotime($slots[$key+1]->slot)){
                                /* convert data time utc to local  */
                                $slotDateTime = strtotime($date." ".convertTimeUtcToLocal($value->slot,request()->customer_timezone));
                                /* convert data time utc to local  */
                            }else{
                                $datetime = $date." ".$value->slot;
                                /* convert data time utc to local  */
                                $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, request()->customer_timezone));
                                /* convert data time utc to local  */
                            }
                        }else{
                            $datetime = $date." ".$value->slot;
                            /* convert data time utc to local  */
                            $slotDateTime = strtotime(convertDateTimeUtcToLocal($datetime, request()->customer_timezone));
                            /* convert data time utc to local  */
                        }

                        $appointment = Appointment::with(['cancell_appointment'])->where(['provider_id'=>$input['provider_id'],'date'=>$date,'time'=> $value->slot])->orderBy('id','desc')->first();
                        if($appointment && empty($appointment->cancell_appointment)){
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'time' => date("h:i A", $slotDateTime),
                                'available' => 'No',
                            ];
                        }else{
                            $availableSots[date("Y-m-d", $slotDateTime)][] =  [
                                'time' => date("h:i A", $slotDateTime),
                                'available' => 'Yes',
                            ];
                        }
                    }
                }
            }

            $allSlots = !empty($availableSots[$curr_day]) ? $availableSots[$curr_day] : [];

            /* provider applied day off  */
            $leaves = [];
            foreach ($allSlots as $key => $slot) {
                $providerTimeOff = ProviderTimeOff::where(['provider_id' => $input['provider_id']])->get();
                foreach ($providerTimeOff as $key => $value) {
                    $startDate = dateFormat($value->start_date);
                    $endDate = dateFormat($value->end_date);

                    if(($curr_day >= $startDate)  && ($curr_day <= $endDate))
                    {
                        if((strtotime($curr_day." ".$slot['time']) >= strtotime($curr_day." ".$value->start_time) && strtotime($curr_day." ".$slot['time']) < strtotime($curr_day." ".$value->end_time)))
                        {
                            $leaves[] =  [
                                'time' => $slot['time'],
                                'available' => $slot['available']
                            ];
                        }
                    }
                }
            }
            /* provider applied day off  */

            $finalSlots = checkLeave($allSlots, $leaves);

            $result['first_half']= [];
            $result['second_half']= [];
            foreach($finalSlots as $key => $value){
                if(strtotime($value['time']) <= strtotime(date("12:00 PM"))){
                    $result['first_half'][] =  [
                        'time' => $value['time'],
                        'available' => $value['available']
                    ];
                }else{
                     $result['second_half'][] =  [
                        'time' => $value['time'],
                        'available' => $value['available']
                    ];
                }
            }

            // ratin reviews
            $result['rating_review'] = ProviderRatingReview::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['provider_id'=>$input['provider_id'],'published'=>'active'])
            ->get();

            // total rating review
            $result['total_rating_review'] = $result['rating_review']->count();

            // average rating
            $result['avg_rating_review'] = number_format((float)$result['rating_review']->avg('session_rating'), 1, '.', '');

            // current active plan
            $customer = Customer::select('plan_id','appointment_session','stripe_subscription_id')->where('id',$this->user_id)->first();
            if(!empty($customer->plan_id)){
                $result['plan_active'] = 'yes';
            }else{
                $result['plan_active'] = 'no';
            }

            // remaining session
            $result['appointment_session'] = !empty($customer->appointment_session) ? $customer->appointment_session : 0;
            

            /* Check change provider request  */
                $checkRequest = ChangeProvider::where(['customer_id' => $this->user_id, 'status' => 'pending'])->first();
                if(!empty($checkRequest)){
                    $result['change_provider_request'] = "yes";
                }else{
                    $result['change_provider_request'] = "no";
                }
            /* End check provider request  */

            /* Check subscription is cancel */
                if(!empty($customer->plan_id) && empty($customer->stripe_subscription_id)){
                    $result['subscription_cancelled'] = "yes";
                }else{
                    $result['subscription_cancelled'] = "no";
                }
            /* Check subscription is cancel */

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_AVAILABLE_TIME'), 'records' => $result]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function providerFullProfile($id)
    {
        try {
            $provider = Provider::with([
                'certifications'=>function($query){
                    $query->select('*');
                },
                'specializations'=>function($query){
                    $query->select('*');
                },
            ])
            ->select('id','first_name','last_name','email','birth_date','gender','phone_code','phone','experience','bio','image','rating','reviews','other_specialization','other_language')
            ->where(['id'=>$id])
            ->first();
            if (empty($provider)) {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_NOT_FOUND')]);
            }
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_DETAILS'), 'records' => $provider]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerAddAppointment(Request $request)
    {
        $message = array(
            'provider_id.required'   => config('service.CUSTOMER_API.VALIDATION.PROVIDER_ID_REQUIRED'),
            'appointment_date.required'   => config('service.CUSTOMER_API.VALIDATION.APPOINTMENT_DATE_REQUIRED'),
            'appointment_date.date'       => config('service.CUSTOMER_API.VALIDATION.APPOINTMENT_DATE_FORMAT'),
            'appointment_time.required'   => config('service.CUSTOMER_API.VALIDATION.APPOINTMENT_TIME_REQUIRED'),
        );
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
            'appointment_date'  => 'required|date',
            'appointment_time'  => 'required',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            $input = $request->all();

            $customer = Customer::findOrFail($this->user_id);
            if(empty($customer->plan_id)){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_PLAN_CHECK')]);
            }

            $appDate = dateFormat($input['appointment_date']);

            /* check if consumer creating new appointment for advance */
            if($customer->appointment_session == 0){

                if($customer->purchased_date){
                   /*$purchasedDate = Carbon::createFromFormat('Y-m-d', $customer->purchased_date);
                   $days = !empty($customer->plan) ? $customer->plan->expiry_days : 0;
                   $expiryDate = $purchasedDate->addDays($days);
                   $expiryDate =  $expiryDate->format('Y-m-d h:i A');*/
                   $expiryDate  = date('Y-m-d', strtotime($customer->expired_date))." ".date("h:i A");

                   if(strtotime($appDate." ".$input['appointment_time']) < strtotime($expiryDate)) 
                   {
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_CHECK_SESSION')]);
                   }  
                }
            }  
            /* check if consumer creating new appointment for advance */

            /* check plan cancel for future appointment */
            if($customer->purchased_date){
                /*$purchasedDate = Carbon::createFromFormat('Y-m-d', $customer->purchased_date);
                $days = !empty($customer->plan) ? $customer->plan->expiry_days : 0;
                $expiryDate = $purchasedDate->addDays($days);
                $expiryDate =  $expiryDate->format('Y-m-d h:i A');*/
                $expiryDate  = date('Y-m-d', strtotime($customer->expired_date))." ".date("h:i A");

                if(empty($customer->stripe_subscription_id) && strtotime($appDate." ".$input['appointment_time']) > strtotime($expiryDate)){
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_PLAN_CHECK')]);
                }

            }
            /* end check plan cancel for future appointment */
            
            /*  Check appointment date time should be greater than current date time */
            $utcTime = date("h:i A");  // defualt time utc is set in env
            $timeLocal = convertTimeUtcToLocal($utcTime,request()->customer_timezone);
            if(strtotime($appDate." ".$input['appointment_time']) < strtotime(date('Y-m-d')." ".$timeLocal)){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.PAST_APPOINTMENT')]);
            }
            /*  end Check appointment date time should be greater than current date time */

            /*  Check appointment if exist already */
            $currentDateTimeUtc = strtotime(date('Y-m-d H:i'));
            $appointment = Appointment::with(['cancell_appointment'])->where(['customer_id' => $this->user_id, 'provider_id' => $input['provider_id']])
            ->where('date_time', '>=', $currentDateTimeUtc)
            ->first();
            if($appointment && empty($appointment->cancell_appointment)){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_ALREADY_CREATED')]);
            }
            /*  end Check appointment if exist already */

            /*  Check change provider request */
            $changeProvider = ChangeProvider::where(['customer_id' => $this->user_id, 'status' => 'pending'])->first();
            if($changeProvider){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.REQUEST_CHANGE_PROVIDER')]);
            }
            /* end change provider request */


            $endTime = date('h:i A', strtotime("+45 minutes", strtotime($input['appointment_time'])));

            $appointmentId = Appointment::getNextAppointmentId();
            $appointment = new Appointment();
            $appointment->appointment_id = $appointmentId;
            $appointment->customer_id = $this->user_id;
            $appointment->provider_id = $input['provider_id'];
            $appointment->date_time = strtotime(convertDateTimeLocalToUtc($input['appointment_date']." ".$input['appointment_time'],request()->customer_timezone));
            $appointment->date = $input['appointment_date'];
            $appointment->day =  date('l', strtotime($input['appointment_date']));
            $appointment->time = convertTimeLocalToUtc($input['appointment_time'],request()->customer_timezone);
            $appointment->end_time = convertTimeLocalToUtc($endTime,request()->customer_timezone);
            $appointment->customer_note = !empty($input['customer_note']) ? $input['customer_note']:'';
            $appointment->appointment_status = "Pending";

            if($appointment->save())
            { 
                /* Customer note send as message */
                    $this->customerNoteSendAsMessage($input, $appointment->id);
                /* Customer note send as message */     
                
                /* Mails   */
                    $this->sendAppointmentEmail($input);
                /* Mails end */
                
                /* Push and db notification */
                    (new CustomerNotificationController)->customerAddAppointment($this->user_id,$input['provider_id']);
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.ADD_APPOINTMENT'), 'records' =>$appointment]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    private function customerNoteSendAsMessage($input,$appointmentId)
    {
        if(!empty($input['customer_note'])){
           /* Send message to provider  */
           $uniq_sender_reciever_id = $this->user_id . '-'. $input['provider_id'];
           $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
           $room = MessageRoom::where('room_id',$uniqueRoomId)->first();
           if(!empty($room))
           {
               // Update room with latest message
               $room = MessageRoom::find($room->id);
               $input['last_message'] = $input['customer_note'];
               $input['appointment_id'] = $appointmentId;
               $input['last_message_date_time'] = date('Y-m-d H:i:s');
               $room->update($input);
               $reciever_id = !empty($room->provider_id) ? $input['provider_id'] : $room->provider_id;

               // save user message
               $input['room_id_db'] = $this->user_id . '-' .$reciever_id;
               $input['sender_id']   = $this->user_id;
               $input['sender_type']   = "customer";
               $input['receiver_type']   = "provider";
               $input['receiver_id'] = $reciever_id;
               $input['room_id'] = $uniqueRoomId;
               $input['message'] = $input['customer_note'];
               $input['msg_date_time'] = date('Y-m-d H:i:s');
               $input['firebase_sender_id'] = !empty(request()->firebase_sender_id) ? request()->firebase_sender_id:'';
               $input['firebase_peer_id'] = !empty(request()->firebase_peer_id) ? request()->firebase_peer_id:'';
               
               MessageRoomChat::create($input);
               
           }
           else
           {
               $input['customer_id'] = $this->user_id;
               $input['provider_id'] = $input['provider_id'];
               $input['room_id'] = $uniqueRoomId;
               $input['last_message'] = $input['customer_note'];
               $input['appointment_id'] = $appointmentId;
               $input['last_message_date_time'] = date('Y-m-d H:i:s');
               MessageRoom::create($input);
               // save user message
               $input['room_id_db'] = $this->user_id . '-' . $input['provider_id'];
               $input['sender_id']   = $this->user_id;
               $input['sender_type']   = "customer";
               $input['receiver_type']   = "provider";
               $input['receiver_id'] = $input['provider_id'];
               $input['message'] = $input['customer_note'];
               $input['msg_date_time'] = date('Y-m-d H:i:s');
               $input['firebase_sender_id'] = !empty(request()->firebase_sender_id) ? request()->firebase_sender_id:'';
               $input['firebase_peer_id'] = !empty(request()->firebase_peer_id) ? request()->firebase_peer_id:'';
               
               MessageRoomChat::create($input);
           }
           /*  End send message to provider */                     
        }

    }

    private function sendAppointmentEmail($input)
    {
        $customer = Customer::where(['id'=>$this->user_id])->first();
        $customer->current_provider_id = $input['provider_id'];
        $customer->save();

        $appointmentTime = convertTimeLocalToUtc($input['appointment_time'],$input['timezone']);
        /* Send mail to customer  */
        $language = Language::findOrFail($customer->language_id);
        $greeting = config('service.CUSTOMER_API.MESSAGE.HI').' '.decryptString($customer->first_name)." ".decryptString($customer->last_name).',';
        if(!empty($language) && $language->code == "es"){
            $body     = 'Su cita ha sido reservada exitosamente para el '.$input['appointment_date'].', '.$appointmentTime;
            $type     = 'PM Ahora | Cita Reservada';
            $sendMail = new CustomerAppointmentMailJob(decryptString($customer->email), $greeting, $body, $type);
            dispatch($sendMail);
        }else{
            $body     = 'Your appointment has been booked successfully for '.$input['appointment_date'].', '.$appointmentTime;
            $type     = 'PM Now | Appointment Booked';
            $sendMail = new CustomerAppointmentMailJob(decryptString($customer->email), $greeting, $body, $type);
            dispatch($sendMail);
        }
        /* End mail to customer */

        /* Send mail to provider  */
        $provider = Provider::where(['id'=>$input['provider_id']])->first();
        $providerTimezone = !empty($provider->timezone) ? $provider->timezone : "America/New_York";
        // $appointmentUtcTime = convertTimeLocalToUtc($input['appointment_time'],request()->customer_timezone);
        $appointmentUtcTime = convertTimeLocalToUtc($input['appointment_time'],$input['timezone']);
        $bookTime = convertTimeUtcToLocal($appointmentUtcTime,$providerTimezone);

        $greeting = 'Hi ' .decryptString($provider->first_name)." ".decryptString($provider->last_name). ',';
        $body     = "Someone booked a new appointment with you on".$input['appointment_date'].", ".$bookTime." Please check your app now to view details.";
        $type     = 'PM Now | New Appointment';
        $sendMail = new CustomerAppointmentMailJob(decryptString($provider->email), $greeting, $body, $type);
        dispatch($sendMail);
        /* End mail to provider */


        /* new appointment email to admin */
        $templateName = "Appointment booked";
        $templateData = EmailTemplate::where('name',$templateName)->first();
        if($templateData)
        {
            $admin = User::where(['id'=>1])->first();
           
            $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
            $body     = $templateData->text;
            $type     = $templateData->title;
            $link     = env('FRONT_END') . '/' . 'upcoming/appointments/';
            $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
            dispatch($sendMail);
        }
        /* End email */
    }

    public function customerRescheduleAppointment(Request $request)
    {
        $message = array(
            'id.required'   => config('service.CUSTOMER_API.VALIDATION.ID_REQUIRED'),
            'provider_id.required'   => config('service.CUSTOMER_API.VALIDATION.PROVIDER_ID_REQUIRED'),
            'appointment_date.required'   => config('service.CUSTOMER_API.VALIDATION.APPOINTMENT_DATE_REQUIRED'),
            'appointment_date.date'       => config('service.CUSTOMER_API.VALIDATION.APPOINTMENT_DATE_FORMAT'),
            'appointment_time.required'   => config('service.CUSTOMER_API.VALIDATION.APPOINTMENT_TIME_REQUIRED'),
        );
        $validator = Validator::make($request->all(), [
            'id'  => 'required',
            'provider_id'  => 'required',
            'appointment_date'  => 'required|date',
            'appointment_time'  => 'required',
        //'customer_note'  => 'required|max:2000',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            $input = $request->all();

            $customer = Customer::findOrFail($this->user_id);
            if(empty($customer->plan_id)){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_PLAN_CHECK')]);
            }

            $appDate = dateFormat($input['appointment_date']);

            /* check if consumer creating new appointment for advance */
            if($customer->appointment_session == 0){
                if($customer->purchased_date){
                   $expiryDate  = date('Y-m-d', strtotime($customer->expired_date))." ".date("h:i A");
                   if(strtotime($appDate." ".$input['appointment_time']) < strtotime($expiryDate)) 
                   {
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_CHECK_SESSION')]);
                   }  
                }
            }  
            /* check if consumer creating new appointment for advance */

            /* check plan cancel for future appointment */
            if($customer->purchased_date){
                $expiryDate  = date('Y-m-d', strtotime($customer->expired_date))." ".date("h:i A");
                if(empty($customer->stripe_subscription_id) && strtotime($appDate." ".$input['appointment_time']) > strtotime($expiryDate)){
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_PLAN_CHECK')]);
                }
            }
            /* end check plan cancel for future appointment */

            /*  Check appointment date time should be greater than current date time */
            $utcTime = date("h:i A");  // defualt time utc is set in env
            $timeLocal = convertTimeUtcToLocal($utcTime,request()->customer_timezone);
            if(strtotime($appDate." ".$input['appointment_time']) < strtotime(date('Y-m-d')." ".$timeLocal)){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.PAST_APPOINTMENT')]);
            }
            /*  end Check appointment date time should be greater than current date time */

            DB::beginTransaction();
            $endTime = date('h:i A', strtotime("+45 minutes", strtotime($input['appointment_time'])));
            $appointment = Appointment::where(['id'=>$input['id'],'customer_id'=>$this->user_id])->first();

            $appointmentDateTime = $appointment->date_time;
            $appointmentDateTime = date('Y-m-d H:i', $appointmentDateTime);
            $appointmentDateTime = convertDateTimeUtcToLocal($appointmentDateTime,request()->customer_timezone);
            $currentUtcDateTime = date('Y-m-d H:i');
            $currentDateTime = convertDateTimeUtcToLocal($currentUtcDateTime,request()->customer_timezone);

            if(strtotime($currentDateTime) >= strtotime($appointmentDateTime))
            {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_CAN_NOT_RESCHEDULE')]);
            }

            $appointment->date_time = strtotime(convertDateTimeLocalToUtc($input['appointment_date']." ".$input['appointment_time'],request()->customer_timezone));
            $appointment->date = $input['appointment_date'];
            $appointment->day =  date('l', strtotime($input['appointment_date']));
            $appointment->time = convertTimeLocalToUtc($input['appointment_time'],request()->customer_timezone);
            $appointment->end_time = convertTimeLocalToUtc($endTime,request()->customer_timezone);
            if(!empty($input['customer_note'])){
                $appointment->customer_note = $input['customer_note'];
            }

            if($appointment->save())
            {
                $rescheduleAppointment = new RescheduleAppointment();
                $rescheduleAppointment->appointment_id = $input['id'];
                $rescheduleAppointment->customer_id = $this->user_id;
                $rescheduleAppointment->provider_id = $input['provider_id'];
                $rescheduleAppointment->date = $input['appointment_date'];
                $rescheduleAppointment->day = date('l', strtotime($input['appointment_date']));
                $rescheduleAppointment->time = convertTimeLocalToUtc($input['appointment_time'],request()->customer_timezone);
                if(!empty($input['customer_note'])){
                    $rescheduleAppointment->customer_note = $input['customer_note'];
                }
                $rescheduleAppointment->save();


                /* Send mail to customer  */
                $customer = Customer::where(['id'=>$this->user_id])->first();
                $language = Language::findOrFail($customer->language_id);
                $greeting = config('service.CUSTOMER_API.MESSAGE.HI').' '.decryptString($customer->first_name)." ".decryptString($customer->last_name). ',';
                if($language->code == "es"){
                    $body     = "Su cita ha sido reprogramada exitosamente para el ".$input['appointment_date'].", ".$input['appointment_time'];
                    $type     = 'PM Ahora | Cita reprogramada';
                    $sendMail = new CustomerAppointmentMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                }else{
                    $body     = "Your appointment has been rescheduled successfully for ".$input['appointment_date'].", ".$input['appointment_time'];
                    $type     = 'PM Now | Appointment Rescheduled';
                    $sendMail = new CustomerAppointmentMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                }
                /* End mail to customer */

                /* Send mail to provider  */
                $provider = Provider::where(['id'=>$input['provider_id']])->first();
                $providerTimezone = !empty($provider->timezone) ? $provider->timezone : "America/New_York";
                $appointmentUtcTime = convertTimeLocalToUtc($input['appointment_time'],request()->customer_timezone);
                $bookTime = convertTimeUtcToLocal($appointmentUtcTime,$providerTimezone);

                $greeting = 'Hi ' .decryptString($provider->first_name)." ".decryptString($provider->last_name). ' ,';
                $body     = "Someone rescheduled the appointment for ".$input['appointment_date']." , ".$bookTime." Please login to your app to review the details.";
                $type     = 'PM Now | Appointment Rescheduled';
                $sendMail = new CustomerAppointmentMailJob(decryptString($provider->email), $greeting, $body, $type);
                dispatch($sendMail);
                /* End mail to provider */

                /* Push and db notification */
                    (new CustomerNotificationController)->customerRescheduleAppointment($this->user_id,$input['provider_id']);
                /* End push and db notification */

                DB::commit();
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.RESCHEDULE_APPOINTMENT'), 'records' =>$appointment]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerCancelAppointment(Request $request)
    {
        $message = array(
            'id.required'   => config('service.CUSTOMER_API.VALIDATION.ID_REQUIRED'),
            'provider_id.required'   => config('service.CUSTOMER_API.VALIDATION.PROVIDER_ID_REQUIRED'),
            'cancel_reason.required'   => config('service.CUSTOMER_API.VALIDATION.CANCEL_REASON_REQUIRED'),
        );
        $validator = Validator::make($request->all(), [
            'id'  => 'required',
            'provider_id'  => 'required',
            'cancel_reason'  => 'required|max:2000',
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            $input = $request->all();
            $appointment = AppointmentCancelled::where(['appointment_id' => $input['id'],'customer_id' => $this->user_id])->first();
            if($appointment){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_ALREADY_CANCEL')]);
            }

            $appointment = Appointment::where(['id'=>$input['id']])->first();
            $appointmentDateTime = $appointment->date_time;
            $appointmentDateTime = date('Y-m-d H:i', $appointmentDateTime);
            $appointmentDateTime = convertDateTimeUtcToLocal($appointmentDateTime,request()->customer_timezone);
            $currentUtcDateTime = date('Y-m-d H:i');
            $currentDateTime = convertDateTimeUtcToLocal($currentUtcDateTime,request()->customer_timezone);

            if(strtotime($currentDateTime) >= strtotime($appointmentDateTime))
            {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' =>config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_CAN_NOT_CANCEL')]);
            }

            $providerCancelRequest = $appointment->provider_cancel_request;
            $appointmentCancelled = new AppointmentCancelled();
            $appointmentCancelled->appointment_id = $input['id'];
            $appointmentCancelled->customer_id = $this->user_id;
            $appointmentCancelled->provider_id = $input['provider_id'];
            $appointmentCancelled->cancel_reason = $input['cancel_reason'];
            if(!empty($providerCancelRequest)){ 
                $appointmentCancelled->provider_cancel_request = $providerCancelRequest;
            }

            if($appointmentCancelled->save())
            {
                /*  appointment cancel status */
                $appointment = Appointment::findOrFail($input['id']);
                $appointment->appointment_cancel = 'yes';
                $appointment->save();
                /*  end appointment cancel status */

                /* Send mail to customer  */
                $customer = Customer::where(['id'=>$this->user_id])->first();
                $customerTimezone = !empty($customer->timezone) ? $customer->timezone : "America/New_York";
                $bookTime = convertTimeUtcToLocal($appointment->time,$customerTimezone);

                $language = Language::findOrFail($customer->language_id);
                $greeting = config('service.CUSTOMER_API.MESSAGE.HI').' '.decryptString($customer->first_name)." ".decryptString($customer->last_name). ',';
                if($language->code == "es"){
                    $body     = "Su cita para el ".showDateFormat($appointment->date).", ".$bookTime." ha sido cancelada exitosamente.";
                    $type     = 'PM Ahora | Cita cancelada';
                    $sendMail = new CustomerAppointmentMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                }else{
                    $body     = "Your appointment has been cancelled successfully for ".showDateFormat($appointment->date).", ".$bookTime;
                    $type     = 'PM Now | Appointment Cancelled';
                    $sendMail = new CustomerAppointmentMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                }
                /* End mail to customer */

                /* Send mail to provider  */
                $provider = Provider::where(['id'=>$input['provider_id']])->first();
                $appointment = Appointment::findOrFail($input['id']);
                $providerTimezone = !empty($provider->timezone) ? $provider->timezone : "America/New_York";
                $bookTime = convertTimeUtcToLocal($appointment->time,$providerTimezone);


                $greeting = 'Hi ' .decryptString($provider->first_name)." ".decryptString($provider->last_name). ' ,';
                $body     = "Your appointment for ".showDateFormat($appointment->date)." ,".$bookTime." has been cancelled. Please login to your app to review the details.";
                $type     = 'PM Now | Appointment Cancelled';
                $sendMail = new CustomerAppointmentMailJob(decryptString($provider->email), $greeting, $body, $type);
                dispatch($sendMail);
                /* End mail to provider */

                /* appointment cancel email to admin */
                $templateName = "Appointment cancelled";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    $admin = User::where(['id'=>1])->first();
                   
                    $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'cancelled/appointments/';
                    $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                    dispatch($sendMail);
                }
                /* End email */

                /* Push and db notification */
                    (new CustomerNotificationController)->customerCancelAppointment($this->user_id,$input['provider_id']);
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CANCEL_APPOINTMENT'), 'records' =>$appointmentCancelled]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerMyAppointments(Request $request)
    {

        try {
                $pieces = explode(" ", request('name'));
                if(count($pieces)){
                    $fName = !empty($pieces[0]) ? $pieces[0] : request('name'); 
                    $lName = !empty($pieces[1]) ? $pieces[1] : request('name'); 
                }

                //$appointmentIds = AppointmentCancelled::where('customer_id', $this->user_id)->pluck('appointment_id')->toArray();
                $appointments = Appointment::with(['provider'=>function($query){
                    $query->select('id','first_name','last_name','email','address','image');
                }])
                ->whereHas('provider', function ($query) use($request, $fName, $lName) {
                    $query->where('first_name', 'like', '%'.request('name').'%');
                    $query->orwhere("last_name",'like', '%'.request('name').'%');
                    $query->orWhere(DB::raw('CONCAT(first_name, " ",last_name)'), 'like', '%'.$fName." ".$lName.'%');
                    $query->orWhere(DB::raw('CONCAT(last_name, " ",first_name)'), 'like', '%'.$fName." ".$lName.'%');
                })
                ->where(['customer_id'=>$this->user_id])
                //->whereNotIn('id', $appointmentIds)
                ->orderBy('id', 'DESC')
                ->get();
                if($appointments->count()){
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_APPOINTMENT_LIST'), 'records' => $appointments]);
                }else{
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_APPOINTMENT_NOT_FOUND')]);
                }

            } catch (Exception $e) {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            } catch (Throwable $e) {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
            }
    }


    public function customerSessionDetail($id)
    {
        try {
            $data['appointment'] = Appointment::with(['provider'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            },'customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['id'=>$id,'customer_id'=>$this->user_id])
            ->first();
            if (empty($data['appointment'])) {
                throw new Exception(config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_NOT_FOUND'));
            }

            $uniq_sender_reciever_id = $this->user_id . '-'. $data['appointment']->provider_id ;
            $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
            $roomExist = MessageRoom::where(['room_id'=>$uniqueRoomId])->first();
            if($roomExist){
                $data['room_id'] = $uniqueRoomId;
            }else{
                $data['room_id'] = "";
            }

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_DETAILS'), 'records' => $data]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerUpdateEmergencyContact(Request $request)
    {
        $message = array(
            'first_name.required'   => config('service.CUSTOMER_API.VALIDATION.FIRST_NAME_REQUIRED'),
            'last_name.required'    => config('service.CUSTOMER_API.VALIDATION.LAST_NAME_REQUIRED'),
            'email.required'        => config('service.CUSTOMER_API.VALIDATION.EMAIL_REQUIRED'),
            'phone_no.required'   => config('service.CUSTOMER_API.VALIDATION.PHONE_NO_REQUIRED'),
            'relationship.required'   => config('service.CUSTOMER_API.VALIDATION.EMERGENCY_RELATIONSHIP_REQUIRED'),
        );
        $validator = Validator::make($request->all(), [
            'first_name'  => 'required|max:100',
            'last_name'   => 'required|max:100',
            //'email'  => 'required|email|max:100',
            'email'  => 'required|max:100',
            //'phone_no'  => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:10',
            'phone_no'  => 'required',
            'relationship'  => 'required|max:100'
        ], $message);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            $input = $request->all();

            $checkExist = EmergencyContact::where(['customer_id'=>$this->user_id])->first();
            if($checkExist){
                $emergencyContact = EmergencyContact::updateOrCreate([
                    'customer_id'   => $this->user_id,
                ],[
                    'first_name'    => $input['first_name'],
                    'last_name'     => $input['last_name'],
                    'email'         => $input['email'],
                    'phone_no'      => $input['phone_no'],
                    'relationship'  => $input['relationship']
                ]);
                $success =[
                    'id'           => $emergencyContact->id,
                    'customer_id'  => $emergencyContact->customer_id,
                    'first_name'   => $emergencyContact->first_name,
                    'last_name'    => $emergencyContact->last_name,
                    'email'        => $emergencyContact->email,
                    'phone_no'     => $emergencyContact->phone_no,
                    'relationship' => $emergencyContact->relationship,
                ];

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->customerUpdateEmergencyContact($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.UPDATE_EMERGENCY_CONTACT'), 'records' =>$success]);
            }else{
                $emergencyContact = EmergencyContact::updateOrCreate([
                    'customer_id'   => $this->user_id,
                ],[
                    'first_name'    => $input['first_name'],
                    'last_name'     => $input['last_name'],
                    'email'         => $input['email'],
                    'phone_no'      => $input['phone_no'],
                    'relationship'  => $input['relationship']
                ]);
                $success =[
                    'id'           => $emergencyContact->id,
                    'customer_id'  => $emergencyContact->customer_id,
                    'first_name'   => $emergencyContact->first_name,
                    'last_name'    => $emergencyContact->last_name,
                    'email'        => $emergencyContact->email,
                    'phone_no'     => $emergencyContact->phone_no,
                    'relationship' => $emergencyContact->relationship,
                ];

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->customerUpdateEmergencyContact($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.ADD_EMERGENCY_CONTACT'), 'records' =>$success]);
            }

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerHomePage(Request $request)
    {

        try {
            $customer = Customer::select('plan_id','purchased_date','expired_date','appointment_session','current_provider_id','stripe_subscription_id')->where(['id'=>$this->user_id])->first();
            if($customer->plan_id){
                $planId  = $customer->plan_id;
                $data['plan'] = Plan::where(['id'=>$planId])->first();
                /*$purchasedDate = Carbon::createFromFormat('Y-m-d', $customer->purchased_date);
                $days = $data['plan']->expiry_days;
                $expiryDate = $purchasedDate->addDays($days);
                $data['renewal_date'] =  $expiryDate->format('m-d-Y');*/
                $data['renewal_date'] = date('m-d-Y', strtotime($customer->expired_date));
                /*$sessionTaken = Appointment::where(['customer_id'=>$this->user_id])
                ->whereBetween('date', [$customer->purchased_date, $expiryDate->format('Y-m-d')])
                ->where('date_time', '<=', strtotime(date("Y-m-d H:i")))
                ->count();
                $data['remaining_sessions'] = ($data['plan']->no_of_session - $sessionTaken);*/
                $data['remaining_sessions'] = (int)$customer->appointment_session;
                $data['total_sessions'] = (int)$customer->total_session_of_plan;
            }else{
                $data['plan'] = null;
                $data['renewal_date'] = null;
                $data['remaining_sessions'] = null;
                $data['total_sessions'] = null;
            }
            $appointmentIds = AppointmentCancelled::where('customer_id', $this->user_id)->pluck('appointment_id')->toArray();


            /*$data['appointment'] = Appointment::with(['provider'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            },'customer'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
            }])
            ->where(['customer_id'=>$this->user_id])
            ->whereNotIn('id', $appointmentIds)
            ->where('date_time', '>=', strtotime(date("Y-m-d H:i")))
            ->first();
            if($data['appointment']){
                $uniq_sender_reciever_id = $this->user_id . '-'. $data['appointment']->provider_id ;
                $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
                $roomExist = MessageRoom::where(['room_id'=>$uniqueRoomId])->first();
                if($roomExist){
                    $data['room_id'] = $uniqueRoomId;
                }else{
                    $data['room_id'] = "";
                }
            }*/

            $appointment = Appointment::with(['provider'=>function($query){
                $query->select('id','first_name','last_name','email','address','image');
                },'customer'=>function($query){
                    $query->select('id','first_name','last_name','email','address','image');
                }])
                ->where(['provider_id' => $customer->current_provider_id])
                ->whereNotIn('id', $appointmentIds)
                ->orderBy('id', 'desc')
                ->first();

            if($appointment){
                $appointmentDateTime = strtotime("+45 minutes", $appointment->date_time);
                if($appointmentDateTime >= strtotime(date("Y-m-d H:i"))){
                    $data['appointment'] = $appointment;
                    if($data['appointment']){
                        $uniq_sender_reciever_id = $this->user_id . '-'. $data['appointment']->provider_id ;
                        $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
                        $roomExist = MessageRoom::where(['room_id'=>$uniqueRoomId])->first();
                        if($roomExist){
                            $data['room_id'] = $uniqueRoomId;
                        }else{
                            $data['room_id'] = "";
                        }
                    }

                }
            }


            $data['provider_id'] = $customer->current_provider_id;
            /* Check change provider request  */
                $checkRequest = ChangeProvider::where(['customer_id' => $this->user_id, 'status' => 'pending'])->first();
                if(!empty($checkRequest)){
                    $data['change_provider_request'] = "yes";
                }else{
                    $data['change_provider_request'] = "no";
                }
            /* End check provider request  */

            /* Check subscription is cancel */
                if(!empty($customer->plan_id) && empty($customer->stripe_subscription_id)){
                    $data['subscription_cancelled'] = "yes";
                }else{
                    $data['subscription_cancelled'] = "no";
                }
            /* Check subscription is cancel */
            
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_HOME_APPOINTMENT'), 'records' => $data]);

        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerChangeProvider(Request $request)
    {
        //has_scope('customers.write');
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
            'issue_type'  => 'required',
            'comment'  => 'required|max:2000',
        ]);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            $input = $request->all();

            $checkRequest = ChangeProvider::where(['customer_id' => $this->user_id, 'provider_id' => $input['provider_id'], 'status' => 'pending'])->first();
            if($checkRequest){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PROVIDER_REQUEST_SENT')]);
            }

            $changeProvider = new ChangeProvider();
            $changeProvider->customer_id = $this->user_id;
            $changeProvider->provider_id = $input['provider_id'];
            $changeProvider->issue_type = $input['issue_type'];
            $changeProvider->comment = $input['comment'];

            if($changeProvider->save())
            {
                /*$customer = Customer::where(['id'=>$this->user_id])->first();
                $customer->current_provider_id = null;
                $customer->save();

                $appointment = Appointment::where(['customer_id' => $this->user_id,'provider_id' => $input['provider_id']])
                ->where(DB::raw('CONCAT(date," ",time)'), '>=',  date("Y-m-d")." ".date("H:i"))
                ->first();
                if($appointment){
                    $appointment->delete();
                }*/
                $customer = Customer::where(['id' => $this->user_id])->first();
                $templateName = "Provider changed request";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    /* Changed provider email */
                        $greeting = config('service.CUSTOMER_API.MESSAGE.HI').' '.decryptString($customer->first_name)." ".decryptString($customer->last_name) . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                        dispatch($sendMail);
                    /* End change provider email  */
                }

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->customerChangeProvider($this->user_id);
                }
                /* End push and db notification */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CHANGE_PROVIDER'), 'records' =>$changeProvider]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerJournalList()
    {
        try {

            $journals = Journal::where(['customer_id'=>$this->user_id])->orderBy('id', 'desc')->get();
            if($journals->count()){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.JOURNAL_LIST'), 'records' => $journals]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.JOURNAL_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerJournalCreate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            //'provider_id'   => 'required',
            'date'          => 'required|date',
            'title'         => 'required|max:100',
            'comment'       => 'required|max:2000',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $journal = new Journal();
            //$journal->provider_id   = $input['provider_id'];
            $journal->date          = $input['date'];
            $journal->customer_id   = $this->user_id;
            $journal->title         = $input['title'];
            $journal->comment         = $input['comment'];
            if($journal->save()){
                $customer = Customer::select('current_provider_id')->where('id',$this->user_id)->first();
                if(!empty($customer->current_provider_id)){
                    $providerId = $customer->current_provider_id;

                    $templateName = "Consumer new journal";
                    $templateData = EmailTemplate::where('name',$templateName)->first();
                    if($templateData)
                    {
                        /* Send mail to provider  */
                        $provider = Provider::where(['id'=>$providerId])->first();
                        
                        $greeting = 'Hi ' .decryptString($provider->first_name)." ".decryptString($provider->last_name). ' ,';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $sendMail = new CustomerCommonMailJob(decryptString($provider->email), $greeting, $body, $type);
                        dispatch($sendMail);
                        /* End mail to provider */
                    }    
                
                    /* Push and db notification */
                        (new CustomerNotificationController)->customerJournalCreate($this->user_id,$providerId);
                    /* End push and db notification */
                }
            }

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.JOURNAL_CREATE')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerJournalUpdate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id'           => 'required',
            //'provider_id'   => 'required',
            'date'          => 'required|date',
            'title'         => 'required|max:100',
            'comment'       => 'required|max:2000',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $journal = Journal::where(['id'=>$input['id'], 'customer_id'=>$this->user_id])->first();
            if($journal)
            {
                //$journal->provider_id   = $input['provider_id'];
                $journal->date          = $input['date'];
                $journal->customer_id   = $this->user_id;
                $journal->title         = $input['title'];
                $journal->comment         = $input['comment'];
                $journal->save();

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.JOURNAL_UPDATE')]);
            }
            else
            {
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SOMETHING_WENT_WRONG')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerJournalView($id)
    {
        try {

            $journal = Journal::where(['id'=>$id,'customer_id'=>$this->user_id])->first();
            if (empty($journal)) {
                throw new Exception(config('service.CUSTOMER_API.MESSAGE.JOURNAL_NOT_FOUND'));
            }
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.JOURNAL_DETAILS'), 'records' => $journal]);
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerJournalDelete(Request $request, $id) {
        $journal = Journal::findOrFail($id);
        try {
            $journal  = $journal->delete();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.JOURNAL_DELETED')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }


    public function customerPaymentHistory(Request $request)
    {
        try {
            $payments = Payment::with(['plan'])
            ->where(function ($q) use ($request) {
                if($request->filled('transaction_id')) {
                    $q->where("transaction_id",'like',"%".$request->transaction_id."%");
                }
            })
            ->where(['customer_id'=>$this->user_id])
            ->orderBy('id', 'desc')
            ->paginate(10);
            if($payments->count()){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PAYMENT_HISTORY_LIST'), 'records' => $payments]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PAYMENT_HISTORY_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerChatRoom(Request $request)
    {
        try
        {
            
            
            $all_room = MessageRoom::has('provider')
            ->with(['provider' => function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
            },'customer' => function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
            },
            'unread_message' => function($q) {
                $q->select('room_id', DB::raw('COUNT(id) as unread_message_count'))
                ->where('status', '=', 0)
                ->groupBy('room_id');
            }]);

            $all_room = $all_room->where([['customer_id', '=', $this->user_id],['deleted_by_customer','=',null]])->orderBy('last_message_date_time', 'desc')->paginate(50);
            return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.ROOM_LIST'), 'records' => $all_room]);
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function getMessageById(Request $request, $id)
    {
        try
        {
            $roomId = $id;
            $room = MessageRoom::where(['room_id'=>$roomId])->first();
            if(!empty($room))
            {
                $order = "desc";
                $limit = isset($request->limit) ? $request->limit : 25;
                if($limit > 500) {
                    $limit = 25;
                }
                if(isset($request->order) && strtolower($request->order)=="asc") {
                    $order = $request->order;
                }
                $order_by_column_name = "id";
                if(isset($request->order_by_column_name)) {
                    if(in_array($request->order_by_column_name, $this->columnName())) {
                        $order_by_column_name = $request->order_by_column_name;
                    }
                }
                /*$all_message = MessageRoomChat::with(['customer' => function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
                }])*/
                $all_message = MessageRoomChat::where(function($q) use($room,$request) {
                    $q->where([
                        ['room_id', '=', $room->room_id],
                    //['contractor_delete_chat_status', '=', 'ACTIVE'],
                    ]);
                })
                ->where(function($q) use($request) {
                    // Global searching...
                    if(isset($request->search) && !empty($request->search) && $request->search!="") {
                        if(!empty($this->columnName())) {
                            foreach ($this->columnName() as $key => $col) {
                                if($key==0) {
                                    $q->where($col, 'like', '%' .$request->search. '%');
                                } else {
                                    $q->orWhere($col, 'like', '%' .$request->search. '%');
                                }
                            }
                        }
                    }
                })
                ->where(function($q) use($request) {
                    // Column wise searching filter ...
                    $allowableColumn = $this->columnName();
                    if(isset($request->column_name) && !empty($request->column_name) && $request->column_name!="" && in_array($request->column_name, $allowableColumn)) {
                        $q->where($request->column_name,  'like', '%' .$request->column_value. '%');
                    }
                })
                ->orderBy($order_by_column_name, $order)
                ->paginate(50);

                // find current provider
                $customer = Customer::select('current_provider_id','plan_id','appointment_session')->where(['id' => $room->customer_id, 'current_provider_id' => $room->provider_id])->first();
                if(!empty($customer->current_provider_id)){
                    $isCurrentProvider = "yes";
                }else{
                    $isCurrentProvider = "no";   
                }
                if(!empty($customer->plan_id) && $customer->appointment_session > 0){
                    $isSessionActive = "yes"; 
                }else{
                    $isSessionActive = "no"; 
                }

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGES_LIST'),'is_current_provider' => $isCurrentProvider, 'is_session_active' => $isSessionActive,'records' => $all_message]);
            }
            else
            {
                throw new Exception(config('service.ROOM_NOT_EXISTS'));
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function columnName() {
        $col_name = [];
        $col = MessageRoomChat::first();
        if(!empty($col) && $col!="") {
            return array_keys($col->toArray());
        }
        return ['id'];
    }


    public function sendMessageById(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'message' => isset($request->message) ? 'required|max:256' : "",
            'room_id' => !empty($request->room_id) ? 'required|max:256' : "",
            'appointment_id' => 'required',
            //'firebase_sender_id' => 'required',
            //'firebase_peer_id' => 'required',
            'image' => $request->has('image') ? 'required|mimes:jpeg,png,jpg,svg,gif' : "",
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try
        {
            $uniq_sender_reciever_id = $this->user_id . '-'. $request->id;
            $uniqueRoomId = $this->generateUniqueRoomID($uniq_sender_reciever_id);
            $roomId = !empty($request->room_id) ? $request->room_id : $uniqueRoomId;
            $room = MessageRoom::where('room_id',$roomId)->first();
            if(!empty($room))
            {
                // Update room with latest message
                $room = MessageRoom::find($room->id);
                $input['last_message'] = $request->message;
                $input['appointment_id'] = $request->appointment_id;
                $input['last_message_date_time'] = date('Y-m-d H:i:s');
                $room->update($input);
                $reciever_id = !empty($room->provider_id) ? $request->id : $room->provider_id;

                // save user message
                $input['room_id_db'] = $this->user_id . '-' .$reciever_id;
                $input['sender_id']   = $this->user_id;
                $input['sender_type']   = "customer";
                $input['receiver_type']   = "provider";
                $input['receiver_id'] = $reciever_id;
                $input['room_id'] = $roomId;
                $input['message'] = $request->message;
                $input['msg_date_time'] = date('Y-m-d H:i:s');
                $input['firebase_sender_id'] = !empty($request->firebase_sender_id) ? $request->firebase_sender_id:'';
                $input['firebase_peer_id'] = !empty($request->firebase_peer_id) ? $request->firebase_peer_id:'';
                if($request->has('image')) {
                /*$image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/chats'),$imagename);
                $input['image'] = $imagename;*/
                }
                $message = MessageRoomChat::create($input);
                $resp = MessageRoomChat::with(['provider' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }])->with('chat_room')->find($message->id);

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$reciever_id)->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->new_message_notification == 'active'){
                    (new CustomerNotificationController)->customerSendMessage($reciever_id);
                }
                /* End push and db notification */

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGE_SENT'), 'records' => $resp]);
            }
            else
            {
                // check sender & reciever pair is already exists or not
                $checkRoomExists = MessageRoom::where('room_id', '=', $roomId)->exists();
                if($checkRoomExists) {
                    throw new Exception("Sorry you have already exists room_id pair with this provider. please use room_id to send message to this provider");
                }
                $input['customer_id'] = $this->user_id;
                $input['provider_id'] = $request->id;
                $input['room_id'] = $roomId;
                $input['last_message'] = $request->message;
                $input['appointment_id'] = $request->appointment_id;
                $input['last_message_date_time'] = date('Y-m-d H:i:s');
                MessageRoom::create($input);
                // save user message
                $input['room_id_db'] = $this->user_id . '-' . $request->id;
                $input['sender_id']   = $this->user_id;
                $input['sender_type']   = "customer";
                $input['receiver_type']   = "provider";
                $input['receiver_id'] = $request->id;
                $input['message'] = $request->message;
                $input['msg_date_time'] = date('Y-m-d H:i:s');
                $input['firebase_sender_id'] = $request->firebase_sender_id;
                $input['firebase_peer_id'] = $request->firebase_peer_id;
                if($request->has('image')) {
                /*$image = $request->file('image');
                $imagename = time().'.'.$image->getClientOriginalName();
                $image->move(public_path('assets/images/chats'),$imagename);
                $input['image'] = $imagename;*/
                }
                $message = MessageRoomChat::create($input);
                $resp = MessageRoomChat::with(['provider' => function($q) {
                    $q->select(['id', 'first_name', 'last_name', 'image']);
                }])->with('chat_room')->find($message->id);

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$request->id)->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->new_message_notification == 'active'){
                    (new CustomerNotificationController)->customerSendMessage($request->id);
                }
                /* End push and db notification */

                return response()->json(['status'=> config('service.SUCCESS_STATUS'), 'message' => config('service.CHAT_MESSAGE_SENT'), 'records' => $resp]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function generateUniqueRoomID($uniq_sender_reciever_id) {
        return md5($uniq_sender_reciever_id);
    }


    public function customerAudioVideoSessionList(Request $request)
    {
        try
        {
            $audioVideoCallHistory = AudioVideoCallHistory::with(['customer'=>function($query){
                $query->select('id','first_name','last_name','image');
            },'provider'=> function($q) {
                $q->select(['id', 'first_name', 'last_name', 'image']);
            }, 'appointment'=> function($q) {
                $q->select(['id', 'date', 'time', 'end_time']);
            }])
            ->where(['customer_id'=>$this->user_id])
            ->whereNotNull('provider_id')
            ->orderBy('id','desc')
            ->paginate(10);
            if($audioVideoCallHistory->count()){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.AUDIO_VIDEO_SESSION_LIST'), 'records' => $audioVideoCallHistory]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.AUDIO_VIDEO_SESSION_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerRatingReviewToProvider(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'provider_id'               => 'required',
            'appointment_id'            => 'required',
            'call_rating'               => 'required|integer|min:1|max:5|digits_between: 1,5',
            'session_rating'            => 'required|integer|min:1|max:5|digits_between: 1,5',
            'recommend_service_rating'  => 'required|integer|min:1|max:5|digits_between: 1,5',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();

            $checkRating = ProviderRatingReview::where(['customer_id' => $this->user_id, 'provider_id' => $input['provider_id'], 'appointment_id' => $input['appointment_id']])->first();

            if($checkRating){
                $checkRating->environment_rating   = $input['call_rating'];
                $checkRating->environment_rating_reason  = !empty($input['call_rating_reason']) ? $input['call_rating_reason'] : '';
                $checkRating->session_rating   = $input['session_rating'];
                $checkRating->session_rating_reason   = !empty($input['session_rating_reason']) ? $input['session_rating_reason'] : '';
                $checkRating->recommend_service_rating  = $input['recommend_service_rating'];
                $checkRating->review  = !empty($input['comment']) ? $input['comment'] :'';
                $checkRating->save();
            }else{

                $ratingReview = new ProviderRatingReview();
                $ratingReview->customer_id   = $this->user_id;
                $ratingReview->provider_id   = $input['provider_id'];
                $ratingReview->appointment_id   = $input['appointment_id'];
                $ratingReview->environment_rating   = $input['call_rating'];
                $ratingReview->environment_rating_reason  = !empty($input['call_rating_reason']) ? $input['call_rating_reason'] : '';
                $ratingReview->session_rating   = $input['session_rating'];
                $ratingReview->session_rating_reason   = !empty($input['session_rating_reason']) ? $input['session_rating_reason'] : '';
                $ratingReview->recommend_service_rating  = $input['recommend_service_rating'];
                $ratingReview->review  = !empty($input['comment']) ? $input['comment'] :'';
                $ratingReview->save();
            }

            /* rating and review email to admin */
            $templateName = "Review Rating";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $admin = User::where(['id'=>1])->first();               
                $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                $body     = $templateData->text;
                $type     = $templateData->title;
                $link     = env('FRONT_END') . '/' . 'review-ratings/';
                $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                dispatch($sendMail);
            }
            /* End email */

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.RATING_REVIEW_CREATE')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerNotifications(Request $request)
    {
        try
        {
            $customer = Customer::where(['id'=>$this->user_id])->first();
            $allNotifications = $customer->notifications()->paginate(10);
            if($allNotifications->count()){
                $data = [];
                foreach ($allNotifications as $key => $allNotification) {
                    $data[] = [
                        'id'=> $allNotification->id,
                        'notifiable_id'=> $allNotification->notifiable_id,
                        'data'=> $allNotification->data,
                        'read_at'=> $allNotification->read_at,
                        'created_at'=> date("Y-m-d H:i",strtotime($allNotification->created_at)),
                    ];
                }
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOTIFICATION_LIST'), 'records' => $data]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOTIFICATION_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerNotificationSetting(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'setting_name'  => 'required|max:191',
            'status'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $input = $request->all();
            if(!empty($input['setting_name']))
            {
                if($input['setting_name']=="show_notification"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                elseif($input['setting_name']=="appointment_reminder"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                elseif($input['setting_name']=="call_notification"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                elseif($input['setting_name']=="new_message_notification"){
                    $this->updateNotificationSetting($input['setting_name'],$input['status']);
                }
                else{
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOTIFICATION_SETTING_NAME')]);
                }
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOTIFICATION_SETTING')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function updateNotificationSetting($settingName,$status){
        $updateSetting = NotificationSetting::updateOrCreate([
            'customer_id'   => $this->user_id,
        ],[
            'customer_id'  => $this->user_id,
            $settingName   => $status,
        ]);
        if($updateSetting){
            return true;
        }
    }


    public function customerNotificationSettingList(Request $request)
    {
        try
        {
            $settings = NotificationSetting::select('id','customer_id','show_notification','appointment_reminder','call_notification','new_message_notification')->where(['customer_id'=>$this->user_id])->first();
            if($settings){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOTIFICATION_SETTING_LIST'), 'records' => $settings]);
            }else{
                $settings = [
                                'id' => null,
                                'customer_id' => null,
                                'show_notification' => null,
                                'appointment_reminder' => null,
                                'call_notification' => null,
                                'new_message_notification' => null,
                            ];
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NOTIFICATION_SETTING_LIST'), 'records' => $settings]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerIntakeQuestionAnswerList(Request $request)
    {
        try
        {
            $questionsAnswers = CustomerIntakeQuestion::with(['question','intakeQuestionAnswer.answer'])->where(['customer_id'=>$this->user_id])->get();
            if($questionsAnswers->count()){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.QUESTION_ANSWER_LIST'), 'records' => $questionsAnswers]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.QUESTION_ANSWER_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerUpdateIntakeQuestion(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'intake_questions'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['success' => false,'message' => $validator->errors()->first()],404);
        }

        try {
            DB::beginTransaction();
            $input = $request->all();
            if(!empty($input['intake_questions']))
            {
                $delete = CustomerIntakeQuestion::where('customer_id', $this->user_id)->delete();
                foreach($input['intake_questions'] as $intake_questions)
                {
                    if(empty($intake_questions['options']))
                    {
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.INTAKE_OPTION_REQ')]);
                    }
                    $intakeQuestion = new CustomerIntakeQuestion();
                    $intakeQuestion->customer_id = $this->user_id;
                    $intakeQuestion->intake_question_id =  $intake_questions['intake_question_id'];
                    $intakeQuestion->save();

                    if(!empty($intake_questions['options']))
                    {
                        foreach($intake_questions['options'] as $option)
                        {
                            $intakeQuestionAnswer = new CustomerIntakeQuestionAnswer();
                            $intakeQuestionAnswer->customer_intake_question_id  = $intakeQuestion->id;
                            $intakeQuestionAnswer->intake_question_option_id  = $option;
                            $intakeQuestionAnswer->save();
                        }
                    }
                }
            }
            DB::commit();
            $data = CustomerIntakeQuestion::with(['intakeQuestionAnswer'=>function($query) {
                $query->select('*');
            }])
            ->where(['customer_id'=>$this->user_id])
            ->get();

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_UPDATE_INTAKE_QUESTION'), 'records' => $data]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function customerCurrentActivePlan(Request $request)
    {
        try
        {
            $customer   = Customer::select('plan_id','purchased_date','expired_date','appointment_session','stripe_subscription_id')->where(['id'=>$this->user_id])->first();
            $plan      = Plan::where(['id' =>  $customer->plan_id, 'status' => 'active'])->first();
            if($plan){
                $renewal_date = null;
                if($customer->purchased_date){
                    /*$purchasedDate = Carbon::createFromFormat('Y-m-d', $customer->purchased_date);
                    $days = $plan->expiry_days;
                    $expiryDate = $purchasedDate->addDays($days);
                    $renewal_date =  $expiryDate->format('m-d-Y');*/
                    $renewal_date  = date('m-d-Y', strtotime($customer->expired_date));
                }
                
                /* Check subscription is cancel */
                if(!empty($customer->plan_id) && empty($customer->stripe_subscription_id)){
                    $subscription_cancelled = "yes";
                }else{
                    $subscription_cancelled = "no";
                }
                /* Check subscription is cancel */

                $plan = [
                    "id" => $plan->id,
                    "price" => $plan->price,
                    "type" => $plan->type,
                    "expiry_days" => $plan->expiry_days,
                    "no_of_session" => $plan->no_of_session,
                    "status" => $plan->status,
                    "title" => $plan->title,
                    "description" => $plan->description,
                    "renewal_date" => $renewal_date,
                    "remaining_sessions" => (int)$customer->appointment_session,
                    "total_sessions" => (int)$customer->total_session_of_plan,
                    "subscription_cancelled" => $subscription_cancelled

                ];  

                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.ACTIVE_PLAN'), 'records' => $plan]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PLAN_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function customerSubscriptionPlan(Request $request)
    {
        try
        {
            $plan = Plan::where(['type' => 'Primary', 'status' => 'active'])->first();
            if($plan){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SUBSCRIPTION_PLAN_DETAIL'), 'records' => $plan]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PLAN_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerOtherPlanList(Request $request)
    {
        try
        {
            $customer   = Customer::select('plan_id')->where(['id'=>$this->user_id])->first();
            $plans      = Plan::where([['status','=','active'],['type','=','Secondary'],['id', '<>' , $customer->plan_id]])->get();
            if($plans->count()){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PLAN_LIST'), 'records' => $plans]);
            }else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PLAN_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerCancelSubscriptionPlan(Request $request)
    {
        try
        {
            $customer   = Customer::where(['id' => $this->user_id])->first();
            if($customer->plan_id){

                if(empty($customer->stripe_subscription_id)){
                     return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CANCEL_SUBSCRIPTION')]);
                }

                $stripe = new \Stripe\StripeClient(
                  env('STRIPE_SECRET')
                );
                $cancelSubscription = $stripe->subscriptions->cancel(
                  $customer->stripe_subscription_id,
                );

                if($cancelSubscription->status == "canceled"){

                   /* plan status cancelled  */ 
                   $payment = Payment::where(['transaction_id' => $customer->stripe_subscription_id, 'customer_id' => $this->user_id])->first();
                   $payment->plan_status = "cancelled";
                   $payment->save();
                   /* plan status cancelled */ 

                    //$customer->plan_id = null;
                    //$customer->purchased_date = null;
                    $customer->stripe_subscription_id = null;
                    $customer->save();

                    /*  Consumer no show email to admin  */
                    $templateName = "Plan Cancelled";
                    $templateData = EmailTemplate::where('name',$templateName)->first();
                    if($templateData)
                    {
                        $admin = User::where(['id'=>1])->first();
                        $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ',';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $link     = env('FRONT_END') . '/' . 'admin/customers/';
                        $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                        dispatch($sendMail);
                    }
                    /* End Consumer no show email to admin   */


                    return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CANCEL_PLAN')]);
                }else{
                    return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SOMETHING_WENT_WRONG')]);
                } 
            }
            else{
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.PLAN_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerTwoFactorEnableDisable(Request $request) {
        $validator = Validator::make($request->all(), [
            'two_factor' =>  'required|max:60',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
            $customer = Customer::findOrFail($this->user_id);
            $customer->two_factor = $request->two_factor;
            $customer->save();
            if($request->two_factor=="enable"){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.TWO_FACTOR_ENABLE')]);
            }
            if($request->two_factor=="disable"){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.TWO_FACTOR_DISABLE')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function sessionToken(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }
        try {
                $input = $request->all();


                $checkAppointment = Appointment::where(['id' => $input['appointment_id'],'customer_id' => $this->user_id])->where('appointment_status', '!=' , 'Completed')->first();
                if($checkAppointment)
                {
                    //$appointmentTime = $checkAppointment->time;
                    $appointmentDateTime = $checkAppointment->date_time;
                    $appointmentDateTime = date('Y-m-d H:i', $appointmentDateTime);
                    $appointmentDateTime = convertDateTimeUtcToLocal($appointmentDateTime,request()->customer_timezone);
                    $currentUtcDateTime = date('Y-m-d H:i');
                    $currentDateTime = convertDateTimeUtcToLocal($currentUtcDateTime,request()->customer_timezone);

                    $customer = Customer::findOrFail($this->user_id);
                    if($customer->appointment_session == 0 && strtotime($currentDateTime) >= strtotime("+45 minutes", strtotime($appointmentDateTime))){
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_REMAINING_SESSION')]);
                    }


                    if(strtotime($currentDateTime) >= strtotime($appointmentDateTime) && strtotime($currentDateTime) <= strtotime("+45 minutes", strtotime($appointmentDateTime)))
                    {
                        $checkToken = DB::table('zoom_session_tokens')->select('token','session_name')->where(['appointment_id' => $input['appointment_id']])->first();
                        if($checkToken)
                        {
                            $data = [
                                    'token' => $checkToken->token,
                                    'session_name' => $checkToken->session_name,
                                    'call_end_time' => (strtotime($checkAppointment->end_time_utc) - strtotime(date('H:i'))) / 60,
                                ];
                            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_SESSION_TOKEN'),'records' => $data]);
                        }
                        else
                        {
                            /*  generate unique session name */
                            $sessionId = DB::table('zoom_session_tokens')->orderBy('id','desc')->value('id');
                            $sessionId = ($sessionId)?$sessionId+1:1;
                            $sessionName = 'SESSION-'.str_pad($sessionId,2,"0",STR_PAD_LEFT);
                            /*  end session name */

                            $headers = ['alg'=>'HS256','typ'=>'JWT'];
                            $headers_encoded = $this->base64url_encode(json_encode($headers));

                            $payload = [
                                'app_key' => env('ZOOM_API_KEY', ''),
                                'version' => 1,
                                'role_type' => 1,
                                'iat' => strtotime(date('Y-m-d', strtotime('+1 days'))),
                                'exp' => strtotime("+45 minutes", strtotime(date('Y-m-d', strtotime('+1 days')))),
                                'tpc' => $sessionName
                                ];
                            $payload_encoded = $this->base64url_encode(json_encode($payload));

                            //build the signature
                            $secretKey = env('ZOOM_API_SECRET', '');
                            $signature = hash_hmac('SHA256',"$headers_encoded.$payload_encoded",$secretKey,true);
                            $signature_encoded = $this->base64url_encode($signature);

                            //build and return the token
                            $token = "$headers_encoded.$payload_encoded.$signature_encoded";
                            

                            $data = DB::table('zoom_session_tokens')->insert(
                                ['appointment_id' => $input['appointment_id'] , 'session_name' => $sessionName,'token' => $token]
                            );

                           $noShowTime = Setting::where('name','noshow')->value('value');

                            if($data){
                                $data = [
                                    'token' => $token,
                                    'session_name' => $sessionName,
                                    'noshow_time' => $noShowTime,
                                    'call_end_time' => (strtotime($checkAppointment->end_time_utc) - strtotime(date('H:i'))) / 60,
                                ];
                                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_SESSION_TOKEN'),'records' => $data]);
                            }
                        }
                    }
                    else
                    {
                        $data = [
                                'token' => '',
                                'session_name' => '',
                                'noshow_time' => ''
                            ];
                        return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.APPOINTMENT_TIME'),'records' => $data]);
                    }
                }
                else
                {
                    return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SESSION_TOKEN_NOT_CREATED')]);
                }   
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function base64url_encode($data) 
    {
        return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
    }


    public function customerJoinSession(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
            'provider_id'  => 'required',
            'call_started_time'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            
            /*$checkPlan = Customer::select('plan_id')->where('id', $this->user_id)->first();
            if(empty($checkPlan)){
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_PLAN_CHECK')]);
            }*/

            /* No show data */
            $noShow = NoShow::updateOrCreate([
                'appointment_id'   => $input['appointment_id'],
            ],[
                'appointment_id'   => $input['appointment_id'],
                'customer_id'   => $this->user_id,
            ]);
            /* No show data */
            $audioVideoCallHistory = AudioVideoCallHistory::where(['appointment_id'=>$input['appointment_id']])->first();
            if($audioVideoCallHistory) {
                $audioVideoCallHistory->customer_id = $this->user_id;
                $audioVideoCallHistory->save();

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$input['provider_id'])->first();
                if(!empty($notificationSetting) && $notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){
                    (new CustomerNotificationController)->customerJoinSessionAgain($input['provider_id']);
                }
                /* End push and db notification */

            }else{
                $audioVideoCallHistory = new AudioVideoCallHistory();
                $audioVideoCallHistory->appointment_id = $input['appointment_id'];
                $audioVideoCallHistory->customer_id = $this->user_id;
                $audioVideoCallHistory->call_started_at = convertTimeLocalToUtc($input['call_started_time'],request()->customer_timezone);
                $audioVideoCallHistory->save();

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$input['provider_id'])->first();
                if(!empty($notificationSetting) && $notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){
                    (new CustomerNotificationController)->customerJoinSession($input['provider_id']);
                }
                /* End push and db notification */
            }

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_JOIN_SESSION')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerSaveVideoSessionDetail(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'appointment_id'  => 'required',
            //'duration'  => 'required',
            'call_end_time'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            $audioVideoCallHistory = AudioVideoCallHistory::where(['appointment_id'=>$input['appointment_id'],'customer_id' => $this->user_id])->first();
            if($audioVideoCallHistory){
                if(empty($audioVideoCallHistory->duration)){
                    //$callStartTime = strtotime($audioVideoCallHistory->call_started_at);
                    $callStartTime = strtotime($audioVideoCallHistory->call_started_at_utc);
                    $callEndTime = strtotime(convertTimeLocalToUtc($input['call_end_time'],request()->customer_timezone));
                    $durationCall = ($callEndTime - $callStartTime) / 60; 
                    $audioVideoCallHistory->duration = $durationCall;
                    $audioVideoCallHistory->call_end_at = convertTimeLocalToUtc($input['call_end_time'],request()->customer_timezone);
                    $audioVideoCallHistory->save();
                }
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_SAVE_SESSION_DETAIL')]);
            }

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function customerNoShowTime(Request $request) {

        try {
            $noShowTime = Setting::where('name','noshow')->value('value');
            if($noShowTime){
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NO_SHOW_TIME'), 'records' => $noShowTime]);
            }
            else{
                return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.NO_SHOW_TIME_NOT_FOUND')]);
            }
        } catch (Exception $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        } catch(Throwable $e) {
            return response()->json(['status'=> config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function noShowNotificationToProvider(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            
            $provider = Provider::where(['id'=>$input['provider_id']])->first();
            if($provider){
                /*  push notification   */
                $deviceToken = $provider->device_token;
                $title = "Appointment";
                $message = "Consumer has joined the session, and waiting for you.";
                sendPushNotification($deviceToken,$title,$message);
                /* end push notification   */

                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_JOIN_SESSION')]);
            }
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function sessionDidNotJoinNotificationToProvider(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'provider_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            
            /* Push and db notification */
                $notificationSetting = NotificationSetting::where('provider_id',$input['provider_id'])->first();
                if($notificationSetting->show_notification == 'active' && $notificationSetting->call_notification == 'active'){
                    (new CustomerNotificationController)->providerDidNotJoinSession($input['provider_id']);
                }
            /* End push and db notification */

            /*  Consumer no show email to admin  */
            $templateName = "Provider No Show";
            $templateData = EmailTemplate::where('name',$templateName)->first();
            if($templateData)
            {
                $admin = User::where(['id'=>1])->first();
                $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ',';
                $body     = $templateData->text;
                $type     = $templateData->title;
                $link     = env('FRONT_END') . '/' . 'admin/noshow/appointments/';
                $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                dispatch($sendMail);
            }
            /* End Consumer no show email to admin   */

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_JOIN_SESSION')]);
        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    private function getColumnName()
    {
        $address = $this->customer::first();
        if (!empty($address) && $address != "") {
            return array_keys($address->toArray());
        }
        return ['id'];
    }

    public function customerSubscriptionPayment(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'plan_id'          => 'required',
            'amount'           => 'required',
            'payment_method'   => 'required',
            'plan_info'        => 'required',
            'stripe_token'     => 'required'
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            DB::beginTransaction();

            $customer = Customer::where(['id' => $this->user_id])->first();
            $plan = Plan::where(['id' => $input['plan_id']])->first();

            /* Create customer on stripe  */
            $stripe = new \Stripe\StripeClient(
               env('STRIPE_SECRET')
            );


            if(!empty($customer->stripe_customer_id)){
                $retrieveCustomer = $stripe->customers->retrieve(
                      $customer->stripe_customer_id,
                );
                $stripeCustomer = $stripe->customers->update(
                  $retrieveCustomer->id,
                  ['source' => $input['stripe_token']]
                );
            }else{
                $stripeCustomer = $stripe->customers->create([
                  'name' => decryptString($customer->first_name)." ".decryptString($customer->last_name),
                  'email' => decryptString($customer->email),
                  'source'  => $input['stripe_token']
                ]);
            }
            
            /* End customer */

            /* Create Product */
            $product = $stripe->products->create([
              'name' => $plan->title ,
            ]);
            /* End Product */

            /* Create price  */
            $price = $stripe->prices->create([
              'currency' => 'usd',
              'unit_amount' => $input['amount']*100,
              'recurring' => ['interval' => 'day', 'interval_count' => $plan->expiry_days],
              'product' => $product->id,
            ]);
            /* End price   */

            /* Create subscription */
            $subscription = $stripe->subscriptions->create([
              'customer' => $stripeCustomer->id,
              'items' => [
                ['price' => $price->id],
              ],
            ]);
            /*  End subscription  */

            if($subscription->status == "active"){
                if(!empty($customer->stripe_customer_id) && !empty($customer->stripe_subscription_id)){
                    /* plan status Canceled on upgrade  */ 
                   $payment = Payment::where(['transaction_id' => $customer->stripe_subscription_id, 'customer_id' => $this->user_id])->first();
                   $payment->plan_status = "cancelled_on_upgrade";
                   $payment->save();
                   /* plan status Canceled on upgrade */ 
                }
                $customer->plan_id = $input['plan_id'];
                $customer->total_session_of_plan = $plan->no_of_session;
                $customer->plan_expiry_days = $plan->expiry_days;
                $customer->stripe_customer_id = $stripeCustomer->id;
                $customer->stripe_subscription_id = $subscription->id;
                $customer->appointment_session = $plan->no_of_session;
                $customer->purchased_date = date('Y-m-d');

                $purchasedDate = Carbon::createFromFormat('Y-m-d', date('Y-m-d'));
                $days = $plan->expiry_days;
                $expiryDate = $purchasedDate->addDays($days);
                $customer->expired_date = $expiryDate->format('Y-m-d');
                $customer->save();

                
                $payment = new Payment();
                $payment->transaction_id   = $subscription->id;
                $payment->stripe_customer_id   = $stripeCustomer->id;
                $payment->plan_id          = $input['plan_id'];
                $payment->total_sessions   = $plan->no_of_session;
                $payment->customer_id      = $this->user_id;
                $payment->payment_method   = $input['payment_method'];
                $payment->amount           = $input['amount'];
                $payment->payment_status   = $subscription->status;
                $payment->plan_status      = "ongoing";
                $payment->plan_info        = json_encode($input['plan_info']);
                $payment->payment_response = json_encode($subscription);
                $payment->stripe_plan_info = json_encode($price);
                $payment->purchased_date   = date('Y-m-d');
                $payment->expire_at        = $expiryDate->format('Y-m-d');
                $payment->save();

                DB::commit();

                
                    $plan = Plan::where(['id' => $input['plan_id']])->first();
                    $language = Language::findOrFail($customer->language_id);
                    $greeting = config('service.CUSTOMER_API.MESSAGE.HI').' '.decryptString($customer->first_name)." ".decryptString($customer->last_name) . ',';
                    if($language->code == "es"){
                        /* plan email */
                        $body     = "¡Gracias! Tu compra para nuestro plan $plan->title se ha completado exitosamente.";
                        $type     = 'PM Ahora | Plan comprado';
                        $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                        dispatch($sendMail);
                        /* End plan email  */

                        /* plan amount email */
                            $body     = "¡Gracias! su pago por $".$input['amount']." se ha realizado exitosamente.";
                            $type     = 'PM Ahora | Pago exitoso';
                            $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                            dispatch($sendMail);
                        /* End plan amount email  */
                    }else{
                        /* plan email */
                        $body     = "Thank You! Your purchase for our plan $plan->title has been successfully completed. ";
                        $type     = 'PM Now | Plan purchased';
                        $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                        dispatch($sendMail);
                        /* End plan email  */

                        /* plan amount email */
                            $body     = "Thanks! Your payment for $".$input['amount']." has been successfully made.";
                            $type     = 'PM Now | Payment Successful';
                            $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                            dispatch($sendMail);
                        /* End plan amount email  */
                    }
                
                /* new appointment email to admin */
                    $admin = User::where(['id'=>1])->first();
                    if($admin){
                            $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                        
                            $body     = "A Consumer has purchased the $plan->title subscription plan. Please click here to view the details.";
                            $type     = "PM NOW | New Subscription";
                            $link     = env('FRONT_END') . '/' . 'purchase-history/';
                            $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                            dispatch($sendMail);
                        
                        /* End email */

                        /* plan amount recieve email to admin */
                            $body     = "A payment of $".$input['amount']." was received. To view details, click here.";
                            $type     = 'PM Now | Payment Received';
                            $link     = env('FRONT_END') . '/' . 'purchase-history/';
                            $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                            dispatch($sendMail);
                        /* End plan amount email  */
                    }

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->customerSubscriptionPayment($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SUBSCRIPTION_PAYMENT')]);
            }else{
                $payment = new Payment();
                $payment->transaction_id   = $subscription->id;
                $payment->plan_id          = $input['plan_id'];
                $payment->customer_id      = $this->user_id;
                $payment->payment_method   = $input['payment_method'];
                $payment->amount           = $input['amount'];
                $payment->payment_status   = $subscription->status;
                $payment->plan_info        = json_encode($input['plan_info']);
                $payment->stripe_plan_info = json_encode($price);
                $payment->payment_response = json_encode($subscription);
                $payment->purchased_date   = date('Y-m-d');

                $purchasedDate = Carbon::createFromFormat('Y-m-d', date('Y-m-d'));
                $days = $plan->expiry_days;
                $expiryDate = $purchasedDate->addDays($days);
                $payment->expire_at  = $expiryDate->format('Y-m-d');
                $payment->save();
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SUBSCRIPTION_PAYMENT_FAILED')]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    
    public function subscriptionRecurringPayment(Request $request)
    {
        $result = @file_get_contents('php://input');
        $response = json_decode($result);
        if(!empty($response)){
            DB::table('recurring_payments')->insert(['payment_response' => $result, 'stripe_customer_id' => $response->data->object->customer, 'stripe_subscription_id' => $response->data->object->subscription]);
        }

        if($response->type == "invoice.finalized" && $response->data->object->status == "paid"){
            $stripeCustomerId = $response->data->object->customer;
            $stripeCustomerEmail = $response->data->object->customer_email;
            $stripeSubscriptionId = $response->data->object->subscription;

            $customer = Customer::with(['plan'])->where(['stripe_subscription_id' => $stripeSubscriptionId, 'stripe_customer_id' => $stripeCustomerId])->first();
            if($customer){
                if(!empty($customer->plan)){
                    //$planSessions = $customer->plan->no_of_session;
                    $planSessions = $customer->total_session_of_plan;
                    $customer->appointment_session = $planSessions;
                    $customer->purchased_date = date('Y-m-d');

                    $purchasedDate = Carbon::createFromFormat('Y-m-d', date('Y-m-d'));
                    $days = $customer->plan_expiry_days;
                    $expiryDate = $purchasedDate->addDays($days);
                    $customer->expired_date = $expiryDate->format('Y-m-d');
                    $customer->save();
                }
            }

        }

    }


    public function latestOldcustomerSubscriptionPayment(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'plan_id'           => 'required',
            'amount'           => 'required',
            'payment_method'    => 'required',
            'plan_info'         => 'required',
            'stripe_token'      => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            DB::beginTransaction();


            $stripe = new \Stripe\StripeClient(
              'sk_test_2gxflEphImjn3C1nbCE26ZwO00PMP3SpSM'
            );
            $token = $stripe->tokens->create([
              'card' => [
                'number' => '4242424242424242',
                'exp_month' => 5,
                'exp_year' => 2023,
                'cvc' => '314',
              ],
            ]);

            \Stripe\Stripe::setApiKey('sk_test_2gxflEphImjn3C1nbCE26ZwO00PMP3SpSM');
            
            $charge = \Stripe\Charge::create([
              'amount' => $input['amount']*100,     // this amount is in cents, 1$ == 100 cents
              'currency' => 'usd',
              'description' => 'Consumer subscription for appointment.',
              //'source' => $input['stripe_token'],
              'source' => $token->id,
              'statement_descriptor' => 'Custom descriptor',
            ]);

            if($charge->status == "succeeded"){
                $customer = Customer::where(['id'=>$this->user_id])->first();
                $customer->plan_id = $input['plan_id'];
                $customer->purchased_date = date('Y-m-d');
                $customer->save();

                $payment = new Payment();
                $payment->transaction_id   = $charge->id;
                $payment->plan_id          = $input['plan_id'];
                $payment->customer_id      = $this->user_id;
                $payment->payment_method   = $input['payment_method'];
                $payment->amount           = $input['amount'];
                $payment->payment_status   = $charge->status;
                $payment->plan_info        = json_encode($input['plan_info']);
                $payment->payment_response = json_encode($charge);
                $payment->purchased_date   = date('Y-m-d');
                $payment->save();
                DB::commit();

                /* plan email */
                    $plan = Plan::where(['id' => $input['plan_id']])->first();
                    $greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name) . ' ,';
                    $body     = "Thank You! Your purchase for our plan $plan->title has been successfully completed. ";
                    $type     = 'PM Now | Plan purchased';
                    $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                /* End plan email  */

                /* plan amount email */
                    $greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name) . ' ,';
                    $body     = "Thanks! Your payment for $".$input['amount']." has been successfully made.";
                    $type     = 'PM Now | Payment Successful';
                    $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                    dispatch($sendMail);
                /* End plan amount email  */

                /* new appointment email to admin */
                    $admin = User::where(['id'=>1])->first();
                    $templateName = "Plan purchased";
                    $templateData = EmailTemplate::where('name',$templateName)->first();
                    if($templateData)
                    {
                        $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                        $body     = $templateData->text;
                        $type     = $templateData->title;
                        $link     = env('FRONT_END') . '/' . 'purchase-history/';
                        $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                        dispatch($sendMail);
                    }
                    /* End email */

                    /* plan amount recieve email to admin */
                        $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                        $body     = "A transaction of $".$input['amount']." has been made. To view details, click the below link.";
                        $type     = 'PM Now | Payment Received';
                        $link     = env('FRONT_END') . '/' . 'purchase-history/';
                        $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                        dispatch($sendMail);
                    /* End plan amount email  */

                /* Push and db notification */
                $notificationSetting = NotificationSetting::where('customer_id',$this->user_id)->first();
                if($notificationSetting->show_notification == 'active'){
                    (new CustomerNotificationController)->customerSubscriptionPayment($this->user_id);
                }
                /* End push and db notification */
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SUBSCRIPTION_PAYMENT')]);
            }else{
                $payment = new Payment();
                $payment->transaction_id   = $charge->id;
                $payment->plan_id          = $input['plan_id'];
                $payment->customer_id      = $this->user_id;
                $payment->payment_method   = $input['payment_method'];
                $payment->amount           = $input['amount'];
                $payment->payment_status   = $charge->status;
                $payment->plan_info        = $input['plan_info'];
                $payment->payment_response = $charge;
                $payment->purchased_date   = date('Y-m-d');
                $payment->save();
                return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SUBSCRIPTION_PAYMENT_FAILED')]);
            }
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function oldcustomerSubscriptionPayment(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'transaction_id'    => 'required',
            'plan_id'           => 'required',
            'payment_method'    => 'required',
            'amount'            => 'required',
            'payment_status'    => 'required',
            'plan_info'         => 'required',
            'payment_response'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $validator->errors()->first()],200);
        }

        try {
            $input = $request->all();
            DB::beginTransaction();
            $customer = Customer::where(['id'=>$this->user_id])->first();
            $customer->plan_id = $input['plan_id'];
            $customer->purchased_date = date('Y-m-d');
            $customer->save();

            $payment = new Payment();
            $payment->transaction_id   = $input['transaction_id'];
            $payment->plan_id          = $input['plan_id'];
            $payment->customer_id      = $this->user_id;
            $payment->payment_method   = $input['payment_method'];
            $payment->amount           = $input['amount'];
            $payment->payment_status   = $input['payment_status'];
            $payment->plan_info        = $input['plan_info'];
            $payment->payment_response = $input['payment_response'];
            $payment->purchased_date   = date('Y-m-d');
            $payment->save();
            DB::commit();

            /* plan email */
                $plan = Plan::where(['id' => $input['plan_id']])->first();
                $greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name) . ' ,';
                $body     = "Thank You! Your purchase for our plan $plan->title has been successfully completed. ";
                $type     = 'PM Now | Plan purchased';
                $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                dispatch($sendMail);
            /* End plan email  */

            /* plan amount email */
                $greeting = 'Hi ' . decryptString($customer->first_name)." ".decryptString($customer->last_name) . ' ,';
                $body     = "Thanks! Your payment for $".$input['amount']." has been successfully made.";
                $type     = 'PM Now | Payment Successful';
                $sendMail = new CustomerCommonMailJob(decryptString($customer->email), $greeting, $body, $type);
                dispatch($sendMail);
            /* End plan amount email  */

            /* new appointment email to admin */
                $admin = User::where(['id'=>1])->first();
                $templateName = "Plan purchased";
                $templateData = EmailTemplate::where('name',$templateName)->first();
                if($templateData)
                {
                    $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                    $body     = $templateData->text;
                    $type     = $templateData->title;
                    $link     = env('FRONT_END') . '/' . 'purchase-history/';
                    $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                    dispatch($sendMail);
                }
                /* End email */

                /* plan amount recieve email to admin */
                    $greeting = 'Hi ' . $admin->first_name." ".$admin->last_name . ' ,';
                    $body     = "A transaction of $".$input['amount']." has been made. To view details, click the below link.";
                    $type     = 'PM Now | Payment Received';
                    $link     = env('FRONT_END') . '/' . 'purchase-history/';
                    $sendMail = new CustomerLinkEmailJob($admin->email, $greeting, $body, $link, $type);
                    dispatch($sendMail);
                /* End plan amount email  */

            /* Push and db notification */
            $notificationSetting = NotificationSetting::where('provider_id',$this->user_id)->first();
            if($notificationSetting->show_notification == 'active'){
                (new CustomerNotificationController)->customerSubscriptionPayment($this->user_id);
            }
            /* End push and db notification */

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.SUBSCRIPTION_PAYMENT')]);
        }
        catch (Exception $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }

    public function changeLanguage(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'language_id'  => 'required',
        ]);
        if($validator->fails()){
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'),'message' => $validator->errors()->first()],200);
        }

        try {
            $customer = Customer::findOrFail($this->user_id);
            $customer->language_id = $request->language_id;
            $customer->save();
            
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_CHANGE_LANGUAGE')]);

        }
        catch (Exception $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
        catch (Throwable $e) {
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'records' => []]);
        }
    }


    public function deleteCustomerAccount() 
    {
        try {
            
            $customer = Customer::find($this->user_id);
            DB::beginTransaction();
            $email = $customer->email;
            $id = $customer->id;
            $customer->forceDelete();
            $this->oauth_user::where(['user_fk' => $id,'email'=>$email])->delete();
            DB::commit();

            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.SUCCESS_STATUS'), 'message' => config('service.CUSTOMER_API.MESSAGE.CUSTOMER_DELETED')]);
        }
        catch (Exception $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
        catch (Throwable $e) {
             DB::rollBack();
            return response()->json(['status' => config('service.CUSTOMER_API.MESSAGE.ERROR_STATUS'), 'message' => $e->getMessage(), 'result' => []]);
        }
    }




}